/**
 * Copyright (C) 2005 - Bull S.A.
 *
 * Jimys - J2EE is monitoring your system
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id:  $
 * --------------------------------------------------------------------------
 */
package org.objectweb.jimys.ads.adengine.web;

import org.objectweb.jimys.commons.configuration.Configuration;
import org.objectweb.jimys.commons.exceptions.ProblemException;
import org.objectweb.jimys.commons.rule.RuleHelper;

import javax.ejb.EJBObject;
import java.rmi.RemoteException;
import java.util.Collection;

/**
 * The EngineWeb interface
 *
 * @author Georges Goebel, Marc Calvisi
 */
public interface EngineWeb extends EJBObject {
    /**
     * gets all rule activator from the rule engine
     *
     * @return collection with RuleActivator's
     * @throws RemoteException
     */
    public Collection getRuleActivators() throws RemoteException;

    /**
     * returns the actual configuration of the rule engine
     *
     * @return actual configuration
     * @throws RemoteException
     */
    public Configuration getConfiguration() throws RemoteException;

    /**
     * activates a rule in the rule engine that is deactive
     *
     * @param ruleset  the rulset the rule belongs to
     * @param rulename the rulename
     * @throws RemoteException
     * @throws ProblemException if the ruleset or rulename does not exists.
     */
    public void activateRule(String ruleset, String rulename) throws RemoteException, ProblemException;

    /**
     * deactivates a rule in the rule engine that is active
     *
     * @param ruleset  the rulset the rule belongs to
     * @param rulename the rulename
     * @throws RemoteException
     * @throws ProblemException if the ruleset or rulename does not exists.
     */
    public void deactivateRule(String ruleset, String rulename) throws RemoteException, ProblemException;

    /**
     * Modifies the actual configuration
     *
     * @param c the new configuration
     * @throws RemoteException
     */
    public void modifiyConfiguration(Configuration c) throws RemoteException;

    /**
     * gets the details of a rule
     *
     * @param ruleset  the ruleset the rule belongs to
     * @param rulename the name of the rule
     * @return the rule details
     * @throws RemoteException
     * @throws ProblemException if the ruleset or rulename does not exists
     */
    public RuleHelper getRule(String ruleset, String rulename) throws RemoteException, ProblemException;
}
