/**
 * Copyright (C) 2005 - Bull S.A.
 *
 * Jimys - J2EE is monitoring your system
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id:  $
 * --------------------------------------------------------------------------
 */


package org.objectweb.jimys.ads.helper;

import org.objectweb.jimys.commons.cartography.*;

import java.util.Collection;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Iterator;
/**
 *
 * Helper to print out the cartographay
 *
 * @author Georges Goebel, Marc Calvisi
 */
public class CartographyHelper {
    /**
     * get the controller of the cartography
     *
     * @param c the cartography to print
     */
    public static void getController(Cartography c) {
        if (c.getAttributeByKey("Controller") != null) {
            Collection col = (Collection) c.getAttributeByKey("Controller");
            Iterator it = col.iterator();
            while (it.hasNext()) {
                CartoController pc = (CartoController) it.next();
                System.out.println("Identifier : " + pc.getIdentifier());
                Hashtable attr = pc.getAttributes();
                Enumeration e = attr.keys();
                while (e.hasMoreElements()) {
                    String key = (String) e.nextElement();
                    System.out.println(key + " : " + pc.getAttributeByKey(key));
                }
                System.out.println("--------------------");
            }
        } else {
            System.out.println(c.getAttributeByKey("Controller"));
        }
    }

    /**
     * get the disk partition of the cartography
     *
     * @param c the cartography to print
     */
    public static void getDiskPartition(Cartography c) {
        if (c.getAttributeByKey("DiskPartition") != null) {
            Collection col = (Collection) c.getAttributeByKey("DiskPartition");
            Iterator it = col.iterator();
            while (it.hasNext()) {
                CartoDiskPartition dp = (CartoDiskPartition) it.next();
                System.out.println("Identifier : " + dp.getIdentifier());
                Hashtable attr = dp.getAttributes();
                Enumeration e = attr.keys();
                while (e.hasMoreElements()) {
                    String key = (String) e.nextElement();
                    System.out.println(key + " : " + dp.getAttributeByKey(key));
                }
                System.out.println("--------------------");
            }
        } else {
            System.out.println(c.getAttributeByKey("DiskPartition"));
        }
    }

    /**
     * get the IPPort in the cartography
     *
     * @param c the cartography to print
     */
    public static void getIPPort(Cartography c) {
        if (c.getAttributeByKey("IPPort") != null) {
            Collection col = (Collection) c.getAttributeByKey("IPPort");
            Iterator it = col.iterator();
            while (it.hasNext()) {
                CartoIPPort ipp = (CartoIPPort) it.next();
                System.out.println("Identifier : " + ipp.getIdentifier());
                Hashtable attr = ipp.getAttributes();
                Enumeration e = attr.keys();
                while (e.hasMoreElements()) {
                    String key = (String) e.nextElement();
                    System.out.println(key + " : " + ipp.getAttributeByKey(key));
                }
                System.out.println("--------------------");
            }
        } else {
            System.out.println(c.getAttributeByKey("IPPort"));
        }
    }

    /**
     * get the memory in the cartography
     *
     * @param c the cartography to print
     */
    public static void getMemory(Cartography c) {
        if (c.getAttributeByKey("Memory") != null) {
            Collection col = (Collection) c.getAttributeByKey("Memory");
            Iterator it = col.iterator();
            while (it.hasNext()) {
                CartoMemory m = (CartoMemory) it.next();
                System.out.println("Identifier : " + m.getIdentifier());
                Hashtable attr = m.getAttributes();
                Enumeration e = attr.keys();
                while (e.hasMoreElements()) {
                    String key = (String) e.nextElement();
                    System.out.println(key + " : " + m.getAttributeByKey(key));
                }
                System.out.println("--------------------");
            }
        } else {
            System.out.println(c.getAttributeByKey("Memory"));
        }
    }

    /**
     * get the operating system in the cartography
     *
     * @param c the cartography to print
     */
    public static void getOperatingSystem(Cartography c) {
        if (c.getAttributeByKey("OperatingSystem") != null) {
            Collection col = (Collection) c.getAttributeByKey("OperatingSystem");
            Iterator it = col.iterator();
            while (it.hasNext()) {
                CartoOperatingSystem os = (CartoOperatingSystem) it.next();
                System.out.println("Identifier : " + os.getIdentifier());
                Hashtable attr = os.getAttributes();
                Enumeration e = attr.keys();
                while (e.hasMoreElements()) {
                    String key = (String) e.nextElement();
                    System.out.println(key + " : " + os.getAttributeByKey(key));
                }
                System.out.println("--------------------");
            }
        } else {
            System.out.println(c.getAttributeByKey("OperatingSystem"));
        }
    }

    /**
     * get the processor in the cartography
     *
     * @param c the cartography to print
     */
    public static void getProcessor(Cartography c) {
        if (c.getAttributeByKey("Processor") != null) {
            Collection col = (Collection) c.getAttributeByKey("Processor");
            Iterator it = col.iterator();
            while (it.hasNext()) {
                CartoProcessor p = (CartoProcessor) it.next();
                System.out.println("Identifier : " + p.getIdentifier());
                Hashtable attr = p.getAttributes();
                Enumeration e = attr.keys();
                while (e.hasMoreElements()) {
                    String key = (String) e.nextElement();
                    System.out.println(key + " : " + p.getAttributeByKey(key));
                }
                System.out.println("--------------------");
            }
        } else {
            System.out.println(c.getAttributeByKey("Processor"));
        }
    }

}
