/**
 * Copyright (C) 2005 - Bull S.A.
 *
 * Jimys - J2EE is monitoring your system
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id:  $
 * --------------------------------------------------------------------------
 */

package org.objectweb.jimys.ads.helper;

import org.objectweb.jimys.ads.adengine.communication.MDBConnector;
import org.objectweb.jimys.commons.clusterstatus.*;
import org.objectweb.jimys.commons.probe.Admin;

import java.util.Date;
import java.util.Enumeration;
import java.util.Hashtable;

/**
 * helper to use with the clusterstatus
 *
 * @author Georges Goebel, Marc Calvisi
 */
public class ClusterStatusHelper {
    /**
     * evaluates the last clusterstatus
     *
     * @param a     the last admin event received, to check the clusterstatus
     * @param cs    the old clusterstatus
     * @param pumps the pumps beeing part of thecluster
     * @return the new clusterstatus
     */
    public static ClusterStatus setClusterStatus(Admin a, ClusterStatus cs, Hashtable pumps) {
        if (a.isRemoveHost()) {
            cs.removeHost(a.getIdentifierHostname());
            Date d = new Date();
            cs.setTimestamp(d.getTime());
            System.out.println("!!!! CLUSTERSTATUS : WRITE TO DB AFERT HOST REMOVED");
            MDBConnector mdb = new MDBConnector();
            mdb.sendClusterStatus(cs);
            mdb = null;
        } else {
            Enumeration e = a.getAttributes().keys();
            String statusInstance = AbstractStatus.OK;
            String statusHost = AbstractStatus.OK;
            boolean updatedb = false;
            while (e.hasMoreElements()) {
                String key = (String) e.nextElement();
                Status s = (Status) a.getAttributeByKey(key);
                if (!s.getStatus().equals(AbstractStatus.OK)) {
                    if (!s.getIdentifierDomain().equals("domain") && !s.getIdentifierInstance().equals("instance")) {
                        statusInstance = AbstractStatus.NOK;
                    }
                    statusHost = AbstractStatus.NOK;
                }
                HostStatus hs = cs.getHostStatus(s.getIdentifierHostname());
                if (hs == null) {
                    hs = new HostStatus(s.getIdentifierHostname(), statusHost, s
                            .getTimestamp());
                    cs.addHostStatus(hs);
                    //System.out.println("hs new");
                    updatedb = true;
                } else {
                    if (!hs.getStatus().equals(statusHost)) {
                        hs.setStatus(statusHost);
                        updatedb = true;
                        //	System.out.println("hs changed");
                    }
                }
                PumpStatus pump = (PumpStatus) pumps.get(s.getIdentifierHostname());
                PumpStatus hsp = null;
                if (pump == null) {
                    pump = new PumpStatus(s.getIdentifierHostname(), AbstractStatus.NOT_CHECKED, s.getTimestamp());
                    hs.setPumpStatus(pump);
                    updatedb = true;
                    //System.out.println("hps new 1");
                } else {
                    hsp = hs.getPumpStatus();
                    if (hsp == null) {
                        hs.setPumpStatus(new PumpStatus(s.getIdentifierHostname(), AbstractStatus.NOT_CHECKED, s.getTimestamp()));
                        updatedb = true;
                        //	System.out.println("hps new 2");
                    } else {
                        if (!pump.getStatus().equals(hsp.getStatus())) {
                            hs.setPumpStatus(pump);
                            updatedb = true;
                            //		System.out.println("hps new changed");
                        }
                    }
                }
                hsp = hs.getPumpStatus();
                if (s.getIdentifierDomain().equals("domain") && s.getIdentifierInstance().equals("instance")) {
                    ProbeStatus ps = hsp.getProbeStatus(s.getIdentifierHostname() + ";" + s.getIdentifierDomain() + ";" + s.getIdentifierInstance() + ";probe;" + s.getIdentifierProbe());
                    if (ps == null) {
                        ps = new ProbeStatus(s.getIdentifierHostname() + ";" + s.getIdentifierDomain() + ";" + s.getIdentifierInstance() + ";probe;" + s.getIdentifierProbe(), s.getStatus(), s.getTimestamp());
                        pump.addProbeStatus(ps);
                        updatedb = true;
                        //System.out.println("pumpprob new");
                    } else {
                        if (!ps.getStatus().equals(s.getStatus())) {
                            ps.setStatus(s.getStatus());
                            updatedb = true;
                            //	System.out.println("pumpprob changed");
                        }
                    }
                } else {
                    DomainStatus ds = hs.getDomainStatus(s.getIdentifierHostname() + ";" + s.getIdentifierDomain());
                    if (ds == null) {
                        ds = new DomainStatus(s.getIdentifierHostname() + ";" + s.getIdentifierDomain(), statusInstance, s
                                .getTimestamp());
                        hs.addDomainStatus(ds);
                        updatedb = true;
                        //	System.out.println("ds new");
                    } else {
                        if (!ds.getStatus().equals(statusInstance)) {
                            ds.setStatus(statusInstance);
                            updatedb = true;
                            //		System.out.println("ds changed");
                        }
                    }
                    InstanceStatus is = ds.getInstanceStatus(s.getIdentifierHostname() + ";" + s.getIdentifierDomain() + ";" + s.getIdentifierInstance());
                    if (is == null) {
                        is = new InstanceStatus(s.getIdentifierHostname() + ";" + s.getIdentifierDomain() + ";" + s.getIdentifierInstance(), statusInstance, s
                                .getTimestamp());
                        ds.addInstanceStatus(is);
                        updatedb = true;
                        //	System.out.println("is new");
                    } else {
                        if (!is.getStatus().equals(statusInstance)) {
                            is.setStatus(statusInstance);
                            updatedb = true;
                            //		System.out.println("is changed");
                        }
                    }
                    ProbeStatus ps = is.getProbeStatus(s.getIdentifierHostname() + ";" + s.getIdentifierDomain() + ";" + s.getIdentifierInstance() + ";probe;" + s.getIdentifierProbe());
                    if (ps == null) {
                        ps = new ProbeStatus(s.getIdentifierHostname() + ";" + s.getIdentifierDomain() + ";" + s.getIdentifierInstance() + ";probe;" + s.getIdentifierProbe(), s.getStatus(), s.getTimestamp());
                        is.addProbeStatus(ps);
                        updatedb = true;
                        //	System.out.println("ps new");
                    } else {
                        if (!ps.getStatus().equals(s.getStatus())) {
                            ps.setStatus(s.getStatus());
                            updatedb = true;
                            //		System.out.println("ps changed");
                        }
                    }
                }
            }
            if (updatedb) {
                Date d = new Date();
                cs.setTimestamp(d.getTime());
                cs.setStatus(statusHost);
                System.out.println("!!!! CLUSTERSTATUS : WRITE TO DB");
                MDBConnector mdb = new MDBConnector();
                mdb.sendClusterStatus(cs);
                mdb = null;
            }
        }

        return cs;
    }

    /**
     * prints out the actual clusterstatus
     *
     * @param cs the actual clusterstatus
     */
    public static void getStatus(ClusterStatus cs) {
        Hashtable hosts = cs.getHosts();
        Enumeration ehost = hosts.keys();
        while (ehost.hasMoreElements()) {
            String khost = (String) ehost.nextElement();
            HostStatus hs = cs.getHostStatus(khost);
            if (hs == null) {
                System.out.println("No status for host : " + khost);
            } else {
                System.out.println("Status for host : " + hs.getIdentifier() + " = "
                        + hs.getStatus());
                PumpStatus pump = hs.getPumpStatus();
                if (pump == null) {
                    System.out.println("No status for pump on host : " + khost);
                } else {
                    System.out.println("Status for host : " + pump.getIdentifier() + ":pump = "
                            + pump.getStatus());
                    Enumeration pumpsProbe = pump.getProbes().keys();
                    while (pumpsProbe.hasMoreElements()) {
                        String kpumpprobe = (String) pumpsProbe.nextElement();
                        ProbeStatus ps = pump.getProbeStatus(kpumpprobe);
                        if (ps == null) {
                            System.out
                                    .println("No status for probe on instance on domain on host : "
                                            + khost
                                            + ":" + kpumpprobe);

                        } else {
                            System.out.println("Status for host : "
                                    + ps.getIdentifier() + " = "
                                    + ps.getStatus());
                        }
                    }
                }
                Enumeration edomain = hs.getDomains().keys();
                while (edomain.hasMoreElements()) {
                    String kdomain = (String) edomain.nextElement();
                    DomainStatus ds = hs.getDomainStatus(kdomain);
                    if (ds == null) {
                        System.out.println("No status for domain on host : "
                                + khost + ":" + kdomain);
                    } else {
                        System.out.println("Status for host : " + khost + ":" + kdomain
                                + " = " + ds.getStatus());
                        System.out.println("Status for host : " + ds.getIdentifier()
                                + " = " + ds.getStatus());
                        Enumeration einstance = ds.getInstances().keys();
                        while (einstance.hasMoreElements()) {
                            String kinstance = (String) einstance.nextElement();
                            InstanceStatus is = ds.getInstanceStatus(kinstance);
                            if (is == null) {
                                System.out
                                        .println("No status for instance on domain on host : "
                                                + khost + ":" + kdomain + ":" + kinstance);
                            } else {
                                System.out.println("Status for host : " + is.getIdentifier() + " = "
                                        + is.getStatus());
                                Enumeration eprobe = is.getProbes().keys();
                                while (eprobe.hasMoreElements()) {
                                    String kprobe = (String) eprobe.nextElement();
                                    ProbeStatus ps = is.getProbeStatus(kprobe);
                                    if (ps == null) {
                                        System.out
                                                .println("No status for probe on instance on domain on host : "
                                                        + khost
                                                        + ":"
                                                        + kdomain
                                                        + ":"
                                                        + kinstance + ":" + kprobe);

                                    } else {
                                        System.out.println("Status for host : "
                                                + ps.getIdentifier() + " = "
                                                + ps.getStatus());
                                    }
                                }
                            }
                        }
                    }
                }
            }
        }
    }

}
