/**
 * Copyright (C) 2005 - Bull S.A.
 *
 * Jimys - J2EE is monitoring your system
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id:  $
 * --------------------------------------------------------------------------
 */

package org.objectweb.jimys.ads.helper;

import org.objectweb.jimys.commons.clusterstatus.Status;
import org.objectweb.jimys.commons.probe.AbstractProbe;
import org.objectweb.jimys.commons.probe.Admin;

import java.util.Iterator;
import java.util.List;

/**
 * Helper to filter events out
 *
 * @author Georges Goebel, Marc Calvisi
 */
public class FilterHelper {
    /**
     * check if the objects in memory is the same as the new instered objects.
     *
     * @param objects          a list with all the events of a class, drools.getObjects(Cpu.class)
     * @param ap               the event to check against the list
     * @param attributeToCheck the attribute that needs to be equals for both identifiers
     * @param sign             this specifies the sign for the if-condition if an Integer is used
     * @return true or false, dependant if the object is one or more times in the working memory
     */
    public static boolean isNotInMemory(List objects, AbstractProbe ap, String attributeToCheck, String sign) {
        Iterator it = objects.iterator();
        int counter = 0;
        while (it.hasNext()) {
            AbstractProbe o = (AbstractProbe) it.next();
            if (ap.getIdentifier().equals(o.getIdentifier())) {
                if (ap.getAttributeByKey(attributeToCheck) instanceof Integer) {
                    if (sign.equals(">")) {
                        if (((Integer) ap.getAttributeByKey(attributeToCheck)).intValue() > ((Integer) o.getAttributeByKey(attributeToCheck)).intValue()) {
                            counter++;
                        }
                    } else if (sign.equals(">=")) {
                        if (((Integer) ap.getAttributeByKey(attributeToCheck)).intValue() >= ((Integer) o.getAttributeByKey(attributeToCheck)).intValue()) {
                            counter++;
                        }
                    } else if (sign.equals("<")) {
                        if (((Integer) ap.getAttributeByKey(attributeToCheck)).intValue() < ((Integer) o.getAttributeByKey(attributeToCheck)).intValue()) {
                            counter++;
                        }
                    } else if (sign.equals("<=")) {
                        if (((Integer) ap.getAttributeByKey(attributeToCheck)).intValue() <= ((Integer) o.getAttributeByKey(attributeToCheck)).intValue()) {
                            counter++;
                        }
                    } else if (sign.equals("==")) {
                        if (((Integer) ap.getAttributeByKey(attributeToCheck)).intValue() == ((Integer) o.getAttributeByKey(attributeToCheck)).intValue()) {
                            counter++;
                        }
                    } else if (sign.equals("!=")) {
                        if (((Integer) ap.getAttributeByKey(attributeToCheck)).intValue() != ((Integer) o.getAttributeByKey(attributeToCheck)).intValue()) {
                            counter++;
                        }
                    }
                } else if (ap.getAttributeByKey(attributeToCheck) instanceof String) {
                    if (ap.getAttributeByKey(attributeToCheck).equals(o.getAttributeByKey(attributeToCheck))) {
                        counter++;
                    }
                }
            }
        }
        //because the list contains also the new inserted objects, it is necessary to check against 1
        return (counter <= 1);
    }

    /**
     * check if the objects in memory is the same as the new instered objects, got the same values
     * and are in a specific time interval
     *
     * @param objects          a list with all the events of a class, drools.getObjects(Cpu.class)
     * @param ap               the event to check against the list
     * @param attributeToCheck the attribute that needs to be equals for both identifiers
     * @param sign             this specifies the sign for the if-condition if an Integer is used
     * @return true or false, dependant if the object is one or more times in the working memory
     */
    public static boolean isNotInMemoryTime(List objects, AbstractProbe ap, String attributeToCheck, String sign, long timeInterval) {
        Iterator it = objects.iterator();
        int counter = 0;
        while (it.hasNext()) {
            AbstractProbe o = (AbstractProbe) it.next();
            if (ap.getIdentifier().equals(o.getIdentifier())) {
                if (o.getTimeStamp() - ap.getTimeStamp() == timeInterval) {
                    if (ap.getAttributeByKey(attributeToCheck) instanceof Integer) {
                        if (sign.equals(">")) {
                            if (((Integer) ap.getAttributeByKey(attributeToCheck)).intValue() > ((Integer) o.getAttributeByKey(attributeToCheck)).intValue()) {
                                counter++;
                            }
                        } else if (sign.equals(">=")) {
                            if (((Integer) ap.getAttributeByKey(attributeToCheck)).intValue() >= ((Integer) o.getAttributeByKey(attributeToCheck)).intValue()) {
                                counter++;
                            }
                        } else if (sign.equals("<")) {
                            if (((Integer) ap.getAttributeByKey(attributeToCheck)).intValue() < ((Integer) o.getAttributeByKey(attributeToCheck)).intValue()) {
                                counter++;
                            }
                        } else if (sign.equals("<=")) {
                            if (((Integer) ap.getAttributeByKey(attributeToCheck)).intValue() <= ((Integer) o.getAttributeByKey(attributeToCheck)).intValue()) {
                                counter++;
                            }
                        } else if (sign.equals("==")) {
                            if (((Integer) ap.getAttributeByKey(attributeToCheck)).intValue() == ((Integer) o.getAttributeByKey(attributeToCheck)).intValue()) {
                                counter++;
                            }
                        } else if (sign.equals("!=")) {
                            if (((Integer) ap.getAttributeByKey(attributeToCheck)).intValue() != ((Integer) o.getAttributeByKey(attributeToCheck)).intValue()) {
                                counter++;
                            }
                        }
                    } else if (ap.getAttributeByKey(attributeToCheck) instanceof String) {
                        if (ap.getAttributeByKey(attributeToCheck).equals(o.getAttributeByKey(attributeToCheck))) {
                            counter++;
                        }
                    }
                }
            }
        }
        //because the list contains also the new inserted objects, it is necessary to check against 1
        return (counter <= 1);
    }

    /**
     * check if the objects in memory is the same as the new instered objects, and checks for an string attribute
     * if it contains the same words
     *
     * @param objects          a list with all the events of a class, drools.getObjects(Cpu.class)
     * @param ap               the event to check against the list
     * @param attributeToCheck the attribute that needs to be equals for both identifiers
     * @param contains         the string that should be in the attribute to check
     * @return true or false, dependant if the object is one or more times in the working memory
     */
    public static boolean isNotInMemoryContainsString(List objects, AbstractProbe ap, String attributeToCheck, String contains) {
        Iterator it = objects.iterator();
        int counter = 0;
        while (it.hasNext()) {
            AbstractProbe o = (AbstractProbe) it.next();
            if (ap.getIdentifier().equals(o.getIdentifier())) {
                if (o.getAttributeByKey(attributeToCheck) instanceof String) {
                    if (((String) o.getAttributeByKey(attributeToCheck)).indexOf(contains) > -1) {
                        counter++;
                    }
                }
            }
        }
        //because the list contains also the new inserted objects, it is necessary to check against 1
        return (counter <= 1);
    }


    public static boolean filterAdmin(List objects, Admin admin, String attributeToCheck, String sign) {
        System.out.println("List : " + objects.size());
        Iterator it = admin.getAttributes().values().iterator();
        String status = "";
        while (it.hasNext()) {
            Status s = (Status) it.next();
            if (s.getIdentifierProbe().equals(attributeToCheck)) {
                status = s.getStatus();
                break;
            }
        }
        int counter = 0;
        it = objects.iterator();
        while (it.hasNext()) {
            Admin a = (Admin) it.next();
            if (!a.isRemoveHost()) {
                if (admin.getIdentifier().equals(a.getIdentifier())) {
                    Iterator ait = a.getAttributes().values().iterator();
                    while (ait.hasNext()) {
                        Status o = (Status) ait.next();
                        if (o.getIdentifierProbe().equals(attributeToCheck)) {
                            if (o.getStatus().equals(status)) counter++;
                        }
                    }
                }
            }
        }
        return (counter == 1);
    }


    public static boolean filterAdmin(List objects, Admin admin, String attributeToCheck, long timeinterval) {
        System.out.println("List : " + objects.size());
        Iterator it = admin.getAttributes().values().iterator();
        String status = "";
        while (it.hasNext()) {
            Status s = (Status) it.next();
            if (s.getIdentifierProbe().equals(attributeToCheck)) {
                status = s.getStatus();
                break;
            }
        }
        int counter = 0;
        it = objects.iterator();
        while (it.hasNext()) {
            Admin a = (Admin) it.next();
            if (!a.isRemoveHost()) {
                if (admin.getIdentifier().equals(a.getIdentifier())) {
                    if (admin.getTimeStamp() - a.getTimeStamp() == timeinterval) {
                        Iterator ait = a.getAttributes().values().iterator();
                        while (ait.hasNext()) {
                            Status o = (Status) ait.next();
                            if (o.getIdentifierProbe().equals(attributeToCheck)) {
                                if (o.getStatus().equals(status)) counter++;
                            }
                        }
                    }
                }
            }
        }
        return (counter == 1);
    }

}
