/**
 * Copyright (C) 2005 - Bull S.A.
 *
 * Jimys - J2EE is monitoring your system
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id:  $
 * --------------------------------------------------------------------------
 */


package org.objectweb.jimys.ads.helper;

import org.objectweb.jimys.dbConnection.valueObjects.ResultProbeValueObject;
import org.objectweb.jimys.dbConnection.valueObjects.ResultValueObject;

import java.util.*;

/**
 * helper used to create the different ValueObjects
 *
 * @author Georges Goebel, Marc Calvisi
 */
public class ResultFactory {
    /**
     * creates a resultValueObject
     *
     * @param resultMessage the message
     * @param level         the alarm level
     * @param rule          the rule that released the consequence
     * @param ruleset       the ruleset the rule belongs to
     * @return ResultValueObject
     */
    public ResultValueObject createResultValueObject(String resultMessage, String level, String rule, String ruleset) {
        ResultValueObject rvp = new ResultValueObject();
        rvp.setLevel(level);
        rvp.setResultMessage(resultMessage);
        Date d = new Date();
        rvp.setTimeStamp(new Long(d.getTime()));
        rvp.setRule(rule);
        rvp.setRuleSet(ruleset);
        return rvp;
    }

    /**
     * creates a ResultProbeValueObject from one attribute of a probe
     *
     * @param identifier     the probe identifier
     * @param attributeName  the attribute name
     * @param attributeValue the attribute value
     * @return ResultProbeValueObject
     */
    public ResultProbeValueObject createResultProbeValueObject(String identifier, String attributeName,
                                                               String attributeValue) {
        ResultProbeValueObject rpvo = new ResultProbeValueObject();
        rpvo.setAttributeName(attributeName);
        rpvo.setAttributeValue(attributeValue);
        rpvo.setProbeIdentifier(identifier);
        return rpvo;
    }

    /**
     * creates a ResultProbeValueObject from one attribute of a probe
     *
     * @param identifier the probe identifier
     * @param attributes all probes attributes
     * @return Collection of ResultProbeValueObject
     */
    public Collection createResultProbeValueObjectCollection(String identifier, Hashtable attributes) {
        Collection rpvoc = new ArrayList();
        Enumeration e = attributes.keys();
        while (e.hasMoreElements()) {
            String key = (String) e.nextElement();
            Object value = attributes.get(key);
            ResultProbeValueObject rpvo = new ResultProbeValueObject();
            rpvo.setAttributeName(key);
            rpvo.setAttributeValue(value.toString());
            rpvo.setProbeIdentifier(identifier);
            rpvoc.add(rpvo);
        }
        return rpvoc;
    }
}
