/**
 * Copyright (C) 2005 - Bull S.A.
 *
 * Jimys - J2EE is monitoring your system
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id:  $
 * --------------------------------------------------------------------------
 */

package org.objectweb.jimys.ads.helper;

import org.objectweb.jimys.commons.probe.AbstractProbe;

import java.util.Date;
import java.util.Iterator;
import java.util.List;

/**
 * Helper to check parameters during a specific time interval
 *
 * @author Georges Goebel, Marc Calvisi
 */
public class TimeperiodHelper {
    /**
     * checks if a specific object has the same value during a specific time
     *
     * @param objects          list with all objects in the working memory
     * @param ap               the objects the objects in the working memory need to correspond to
     * @param attributeName    the attributename to check
     * @param attributeValue   the value it should have
     * @param timeperiod       the time period
     * @param deviation        the deviation the attribute value of the objects may have
     * @param minEventsToCheck minimum of objects to have to check
     * @return true if the average value of the objects is in the limit of the attribute value
     */
    public static boolean isValueSameInPeriod(List objects, AbstractProbe ap, String attributeName, int attributeValue, long timeperiod, int deviation, int minEventsToCheck) {
        Iterator it = objects.iterator();
        int counter = 0;
        int values = 0;
        Date d = new Date();
        long startTime = d.getTime() - timeperiod;
        long timestamp = d.getTime();
        while (it.hasNext()) {
            AbstractProbe object = (AbstractProbe) it.next();
            if (object.getIdentifier().equals(ap.getIdentifier())) {
                if (object.getTimeStamp() < timestamp) timestamp = object.getTimeStamp();
                if (object.getTimeStamp() >= startTime) {
                    Integer value = ((Integer) object.getAttributeByKey(attributeName));
                    if (value.intValue() >= 0) {
                        values += value.intValue();
                        counter++;
                    }
                }
            }
        }
        /*
        System.out.println("nbr objects : "+counter);
        Format formatter = new SimpleDateFormat("HH:mm:ss dd/MM/yyyy");
        System.out.println("actual timestamp : "+formatter.format(new Date(d.getTime())));
        System.out.println("smallest timestamp : "+formatter.format(new Date(timestamp)));
        */
        if (counter >= minEventsToCheck && counter > 1) {
            double result = (double) values / counter;
            double devA = (double) attributeValue + ((double) (attributeValue * deviation) / 100);
            double devB = (double) attributeValue - ((double) (attributeValue * deviation) / 100);
            if (devB < 0) devB = 0;
            if (devA > 100) devA = 100;
            return (result >= devB && result <= devA);
        }
        return false;
    }

    public static boolean isValueSameInPeriod(List objects, AbstractProbe ap, String attributeName, int attributeValue, long timeperiod, int minEventsToCheck, String sign) {
        Iterator it = objects.iterator();
        int counter = 0;
        Date d = new Date();
        long startTime = d.getTime() - timeperiod;
        //long timestamp = d.getTime();
        while (it.hasNext()) {
            AbstractProbe object = (AbstractProbe) it.next();
            if (object.getIdentifier().equals(ap.getIdentifier())) {
                //if (object.getTimeStamp() < timestamp) timestamp = object.getTimeStamp();
                if (object.getTimeStamp() >= startTime) {
                    Integer value = ((Integer) object.getAttributeByKey(attributeName));
                    if (sign.equals(">")) {
                        if (value.intValue() > attributeValue) counter++;
                    } else if (sign.equals(">=")) {
                        if (value.intValue() >= attributeValue) counter++;
                    } else if (sign.equals("<")) {
                        if (value.intValue() < attributeValue) counter++;
                    } else if (sign.equals("<=")) {
                        if (value.intValue() <= attributeValue) counter++;
                    } else if (sign.equals("==")) {
                        if (value.intValue() == attributeValue) counter++;
                    } else if (sign.equals("!=")) {
                        if (value.intValue() != attributeValue) counter++;
                    }
                }
            }
        }
        return (counter >= minEventsToCheck && counter > 1);
    }
}
