/**
 * Copyright (C) 2005 - Bull S.A.
 *
 * Jimys - J2EE is monitoring your system
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id:  $
 * --------------------------------------------------------------------------
 */
package org.objectweb.jimys.dbConnection.cartography.entity;

import javax.ejb.CreateException;
import javax.ejb.EJBLocalHome;
import javax.ejb.FinderException;
import java.util.Collection;

/**
* @author Georges Goebel, Marc Calvisi
* All the information is related to the life cycle of a cartography
*/

public interface LocalCartographyHome extends EJBLocalHome {

    /**
     * EJB PK finder : The PK is created by the container
     * @param key The PK (java.lang.Integer)
     * @return The related LocalCartography
     * @throws FinderException
     */
    public LocalCartography findByPrimaryKey(Object key) throws FinderException;

    /**
     * Find the cartography of the host at a given time
     * Nasty hack. Because I need to find the last cartography for a given host
     * I must first look for the maxtimestamp of a given host and afterwards use this
     * timestamp to look for the cartography. cmp limitation
     * @param hostname The hostname
     * @param timestamp The timestamp
     * @return The Cartography
     * @throws FinderException
     */
    public Collection findCartography(String hostname, Long timestamp) throws FinderException;

    /**
     * Select the max timestamp for a given host
     * @param hostname The hostname
     * @return The timestamp
     * @throws FinderException
     */
    public Long selectMaxTimeStamp(String hostname) throws FinderException;

    /**
     * EJB create : creates a new Cartography. The PK is automatically generated
     * @return The new Cartography
     * @throws CreateException
     */
    public LocalCartography create() throws CreateException;
}
