/**
 * Copyright (C) 2005 - Bull S.A.
 *
 * Jimys - J2EE is monitoring your system
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id:  $
 * --------------------------------------------------------------------------
 */
package org.objectweb.jimys.dbConnection.entity.result;

import javax.ejb.CreateException;
import javax.ejb.EJBLocalHome;
import javax.ejb.FinderException;
import java.util.Collection;

/**
 *
 * LocalResultHome
 *
 * @author Georges Goebel, Marc Calvisi
 */
public interface LocalResultHome extends EJBLocalHome {
    /**
     * Find a result by the primary key.
     * Hint : You can use the getId() methode of the resultvalueobject to
     * look for a specific object. Attention, the object has to be a java.lang.Integer
     * @param key The key (java.lang.Integer)
     * @return The LocalResult
     * @throws FinderException
     */
    public LocalResult findByPrimaryKey(Object key) throws FinderException;

    /**
     * Find all the results
     * @return List of all the results
     * @throws FinderException
     */
    public Collection findAll() throws FinderException;

    /**
     * Find the next 20 results started by the row number indicated
     * @param rowLines The row number to start
     * @return List of maximum 20 results
     * @throws FinderException
     */
    public Collection findAllOptimized(Integer rowLines) throws FinderException;

    /**
     * Find all the results alter a specified time
     * @param time The time to start
     * @return List of all the Results after the specified time
     * @throws FinderException
     */
    public Collection findAllAfterTime(String time) throws FinderException;

    /**
     * Find all the Results which were produced by a specific rule
     * @param ruleset The rule set name
     * @param rule The rule name
     * @return List of the LocalResults which were produced by the specified rule
     * @throws FinderException
     */
    public Collection findAllRule(String ruleset, String rule) throws FinderException;

    /**
     * Find all the Results which were produced by a specific rule after a time
     * @param ruleset The rule set name
     * @param rule The rule name
     * @param time The time to start
     * @return List of the LocalResults which were produced by the specified rule after the time
     * @throws FinderException
     */
    public Collection findAllRuleAfterTime(String ruleset, String rule, String time) throws FinderException;

    /**
     * Find all the results with the indicated level
     * @param level The level to look for
     * @return List of the results which match the level
     * @throws FinderException
     */
    public Collection findAllResultWithLevel(String level) throws FinderException;

    /**
     * Find all the results with the indicated level and which contain a certain probeidentifier
     * @param level The level to look for
     * @param probIdentifier The probe identifier
     * @return List of the results which match the level and probeidentifier
     * @throws FinderException
     */
    public Collection findAllResultWithLevel(String level, String probIdentifier) throws FinderException;

    /**
     * Find all the rule names in the database
     * @return The list of all the rule names (strings)
     * @throws FinderException
     */
    public Collection selectRule() throws FinderException;

    /**
     * EJB create : Creates a new instance of LocalResult
     * A primary key is automatically generated by the conainer
     * @return the new LocalResult
     * @throws CreateException
     */
    public LocalResult create() throws CreateException;
}
