/**
 * Copyright (C) 2005 - Bull S.A.
 *
 * Jimys - J2EE is monitoring your system
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id:  $
 * --------------------------------------------------------------------------
 */
package org.objectweb.jimys.dbConnection.entity.resultProbe;

import javax.ejb.CreateException;
import javax.ejb.EJBLocalHome;
import javax.ejb.FinderException;
import java.util.Collection;

/**
 *
 * LocalResultProbeHome
 *
 * @author Georges Goebel, Marc Calvisi
 */
public interface LocalResultProbeHome extends EJBLocalHome {
    /**
     * Find a resultprobe by the primary key.
     * Hint : You can use the getId() methode of the resultvalueobject to
     * look for a specific object. Attention, the object has to be a java.lang.Integer
     * @param key The key (java.lang.Integer)
     * @return The localresultprobe
     * @throws FinderException
     */
    public LocalResultProbe findByPrimaryKey(Object key) throws FinderException;

    /**
     * ejb create : The primary key is generated by the container
     * @return The new LocalResultProbe
     * @throws CreateException
     */
    public LocalResultProbe create() throws CreateException;

    /**
     * Find all the LocalResultProbe for a given attributename and a probeidentifier
     * @param attributeName The attribute name
     * @param probeIdentifier The probeidentifier
     * @return Collection of the LocalResultProbe which match the pattern
     * @throws FinderException
     */
    public Collection findAllAttributes(String attributeName, String probeIdentifier) throws FinderException;

    /**
     * Find all the LocalResultProbe for a given attributename and a probeidentifier
     * after a certain time
     * @param attributeName The attribute name
     * @param probeIdentifier The probeidentifier
     * @param timestamp The timestamp to begin
     * @return Collection of the LocalResultProbe which match the pattern
     * @throws FinderException
     */
    public Collection findAllAttributes(String attributeName, String probeIdentifier, String timestamp) throws FinderException;

    /**
     * Find all the attributenames of the probes which are used by a rule
     * @param rulename The rulename
     * @return The list (string) of the attributenames
     * @throws FinderException
     */
    public Collection selectAllAttributesOfARule(String rulename) throws FinderException;

    /**
     * These methodes still works but are no longuer used. Needed for cmp2 which is replaced
     * by hibernate
     * @deprecated
     * @return
     * @throws FinderException
     */
    public Collection selectAllDistinctProbes() throws FinderException;

    /**
     * These methodes still works but are no longuer used. Needed for cmp2 which is replaced
     * by hibernate
     * @deprecated
     * @param probeName
     * @return
     * @throws FinderException
     */
    public Collection selectAllDistinctProbeAttributes(String probeName) throws FinderException;


    /**
     * These methodes still works but are no longuer used. Needed for cmp2 which is replaced
     * by hibernate
     * @param query
     * @param status
     * @return
     * @throws FinderException
     */
    public Collection findAllProbesWithStatus(String query, String status) throws FinderException;
}
