/**
 * Copyright (C) 2005 - Bull S.A.
 *
 * Jimys - J2EE is monitoring your system
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id:  $
 * --------------------------------------------------------------------------
 */
package org.objectweb.jimys.dbConnection.mdb;

import org.objectweb.jimys.dbConnection.session.LocalDBManager;
import org.objectweb.jimys.dbConnection.session.LocalDBManagerHome;
import org.objectweb.jimys.dbConnection.valueObjects.ResultValueObject;
import org.objectweb.jimys.commons.cartography.Cartography;
import org.objectweb.jimys.commons.clusterstatus.ClusterStatus;

import javax.ejb.CreateException;
import javax.ejb.EJBException;
import javax.ejb.MessageDrivenBean;
import javax.ejb.MessageDrivenContext;
import javax.jms.JMSException;
import javax.jms.Message;
import javax.jms.MessageListener;
import javax.jms.ObjectMessage;
import javax.naming.Context;
import javax.naming.InitialContext;
import javax.naming.NamingException;
import java.io.*;
import java.net.URL;
import java.util.Collection;
import java.util.Hashtable;
import java.util.Properties;

/**
 *
 * DBConnectorBean
 *
 * @author Georges Goebel, Marc Calvisi
 */
public class DBConnectorBean implements MessageDrivenBean, MessageListener {
    private LocalDBManager ldbm = null;


    public DBConnectorBean() {
        URL config = Thread.currentThread().getContextClassLoader().getResource("dbmanager.properties");
        Properties props = new Properties();
        try {
            props.load(new FileInputStream(new File(config.getFile())));
        } catch (FileNotFoundException e1) {
            e1.printStackTrace();
        } catch (IOException e1) {
            e1.printStackTrace();
        }
        try {
            String home = "";
            if (props.getProperty("database.local.home") != null) {
                home = props.getProperty("database.local.home");
            }
            //ictx = new InitialContext(properties);
            Context ictx = new InitialContext();
            LocalDBManagerHome ldbmh = (LocalDBManagerHome) ictx.lookup(home);
            ldbm = ldbmh.create();
        } catch (NamingException e) {
            e.printStackTrace();
        } catch (CreateException e) {
            e.printStackTrace();
        }
    }

    public void onMessage(Message message) {
        Serializable object = null;
        try {
            object = ((ObjectMessage) message).getObject();
        } catch (JMSException e1) {
            e1.printStackTrace();
        }
        if (object instanceof ClusterStatus) {
            ldbm.addClusterStatus((ClusterStatus) object);
        } else if (object instanceof Hashtable) {
            Hashtable value = (Hashtable) object;
            ResultValueObject rvo = (ResultValueObject) value.get("resultValueObject");
            Collection col = (Collection) value.get("results");
            ldbm.addResult(rvo, col);
        } else if (object instanceof Cartography) {
            ldbm.addCartography((Cartography) object);
        }

    }

    public void ejbRemove() throws EJBException {
        ldbm = null;
    }

    public void setMessageDrivenContext(MessageDrivenContext messageDrivenContext) throws EJBException {
    }

    public void ejbCreate() throws CreateException {
    }
}
