/**
 * Copyright (C) 2005 - Bull S.A.
 *
 * Jimys - J2EE is monitoring your system
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id:  $
 * --------------------------------------------------------------------------
 */
package org.objectweb.jimys.dbConnection.session;

import org.hibernate.Query;
import org.hibernate.Session;
import org.hibernate.SessionFactory;
import org.hibernate.Transaction;
import org.hibernate.cfg.Configuration;
import org.jfree.data.xy.XYSeries;
import org.jfree.data.xy.XYSeriesCollection;
import org.objectweb.jimys.commons.cartography.Cartography;
import org.objectweb.jimys.commons.clusterstatus.*;
import org.objectweb.jimys.commons.level.LevelHelper;
import org.objectweb.jimys.commons.probe.AbstractProbe;
import org.objectweb.jimys.dbConnection.cartography.entity.LocalCartography;
import org.objectweb.jimys.dbConnection.cartography.entity.LocalCartographyHome;
import org.objectweb.jimys.dbConnection.entity.result.LocalResult;
import org.objectweb.jimys.dbConnection.entity.result.LocalResultHome;
import org.objectweb.jimys.dbConnection.entity.resultProbe.LocalResultProbe;
import org.objectweb.jimys.dbConnection.entity.resultProbe.LocalResultProbeHome;
import org.objectweb.jimys.dbConnection.entity.resultProbeName.LocalResultProbeNameHome;
import org.objectweb.jimys.dbConnection.hibernate.beans.*;
import org.objectweb.jimys.dbConnection.status.entity.cluster.LocalClusterStatus;
import org.objectweb.jimys.dbConnection.status.entity.cluster.LocalClusterStatusHome;
import org.objectweb.jimys.dbConnection.status.entity.domain.LocalDomainStatus;
import org.objectweb.jimys.dbConnection.status.entity.host.LocalHostStatus;
import org.objectweb.jimys.dbConnection.status.entity.instance.LocalInstanceStatus;
import org.objectweb.jimys.dbConnection.status.entity.probe.LocalProbeStatus;
import org.objectweb.jimys.dbConnection.status.entity.pump.LocalPumpStatus;
import org.objectweb.jimys.dbConnection.valueObjects.ResultProbeValueObject;
import org.objectweb.jimys.dbConnection.valueObjects.ResultValueObject;

import javax.ejb.*;
import javax.naming.Context;
import javax.naming.InitialContext;
import javax.naming.NamingException;
import java.net.URL;
import java.util.*;
import java.util.logging.Level;

/**
 *
 * DBManagerBean
 *
 * @author Georges Goebel, Marc Calvisi
 */
public class DBManagerBean implements SessionBean {

    private static java.util.logging.Logger logger = java.util.logging.Logger.getLogger("org.objectweb.jimys");
    /**
     * This variable indicated if hibernate is bound to JNDI
     * static variables are nasty but the integration of hibernate was done quickly :-(
     */
    private static boolean bound = false;

    /**
     * Hibernate is configured when the first session bean is instatiated
     */
    public DBManagerBean() {
        if (! bound) {
            Context initalContext = null;
            try {
                // Se if the hibernate session factory is bound
                initalContext = new InitialContext();
                Object o = initalContext.lookup("hibernate");
            } catch (NamingException e) {
                if (initalContext != null) {
                    // retrieve the url to the hibernate configuration file
                    URL url = Thread.currentThread().getContextClassLoader().getResource("hibernate.cfg.xml");
                    logger.log(Level.INFO, "hibernate url = " + url);

                    // configuring hibernate with the configuration file
                    Configuration conf = new Configuration().configure(url);
                    // creating the sessionfactory (Session factory is ThreadSave !)
                    // session is NOT ThreadSave !
                    // the sessionfactory is automatically bound to jndi when you specify the "name"
                    // attribute in the xml configuration file (in the element of the sessionfactory)
                    // We had no time to test the possibility to integrate the transaction service of jonas
                    // in hibernate. So all the transaction you find are hibernate transactions and no JTA
                    // transactions. It is possible to integrate JOnAS JTA in hibernate but we had no time
                    // to test
                    SessionFactory sf = conf.buildSessionFactory();
                    logger.log(Level.INFO, "hibernate session factory created " + sf);
                    /*
                    // No need to rebind the sessionfactory as there is the name attribute
                    // specified in the hibernate configuration file
                    try {
                        initalContext.rebind("hibernate", sf);
                    } catch (NamingException e1) {
                        e1.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
                    }
                    */
                    // setting the check variable to true -> sessionfactory is bound
                    // nasty and not thread save :-(
                    bound = true;
                } else {
                    logger.log(Level.SEVERE, "ERROR WHILE INITIAL CONTEXT");
                }
            }
        }

    }

    public void ejbCreate() throws CreateException {
    }

    public void setSessionContext(SessionContext sessionContext) throws EJBException {
    }

    public void ejbRemove() throws EJBException {
    }

    public void ejbActivate() throws EJBException {
    }

    public void ejbPassivate() throws EJBException {
    }

    private void fullDefendencyHack() {
        new HClusterStatus();
        new HHostStatus();
        new HDomainStatus();
        new HInstanceStatus();
        new HPumpStatus();
        new HProbeStatus();
    }

    public void addResult(ResultValueObject rvo, Collection resultProbeValues) {
        Context initalContext = null;
        SessionFactory sf = null;
        try {
            initalContext = new InitialContext();
            sf = (SessionFactory) initalContext.lookup("hibernate");
        } catch (NamingException e) {
            e.printStackTrace();
        }

        if (sf != null) {
            Session session = sf.openSession();
            Transaction tx = session.beginTransaction();

            Result result = new Result();
            //result.setId(new Integer(4));
            result.setLevel(rvo.getLevel());
            result.setResultMessage(rvo.getResultMessage());
            result.setRule(rvo.getRule());
            result.setRuleSet(rvo.getRuleSet());
            result.setTimestamp(rvo.getTimeStamp());
            session.save(result);
            result.setId(result.getJpk());
            session.update(result);

            Set set = new TreeSet();
            for (Iterator iterator = resultProbeValues.iterator(); iterator.hasNext();) {
                ResultProbeValueObject resultProbeValueObject = (ResultProbeValueObject) iterator.next();

                ProbeName pn = new ProbeName();
                pn.setAttributeName(resultProbeValueObject.getAttributeName());
                pn.setProbeIdentifier(resultProbeValueObject.getProbeIdentifier());
                session.saveOrUpdate(pn);

                ResultProbe probe = new ResultProbe();
                probe.setProbeName(pn);
                probe.setResult(result);
                probe.setTimestamp(rvo.getTimeStamp());
                probe.setAttributeValue(resultProbeValueObject.getAttributeValue());
                probe.setId(probe.getJpk());
                session.save(probe);
                probe.setId(probe.getJpk());
                session.update(probe);
                set.add(probe);
            }

            result.setResultProbes(set);
            session.saveOrUpdate(result);

            tx.commit();
            session.close();

        } else {
            logger.log(Level.SEVERE, "ERROR RECEIVINF HIBERNATE FACTORY");
        }
    }

    /*

    // This methode uses CMP 2. If you want to eliminate hibernate, uncomment this
    // methode and comment the previous methode. Both methodes do the same thing

public void addResult(ResultValueObject rvo, Collection resultProbeValues) {
try {
Context initalContext = new InitialContext();

LocalResultProbeHome lrph = (LocalResultProbeHome) initalContext.lookup("ResultProbeEJBHome_L");
LocalResultHome lrh = (LocalResultHome) initalContext.lookup("ResultEJBHome_L");
LocalResultProbeNameHome lrphn = (LocalResultProbeNameHome) initalContext.lookup("ResultProbeNameEJBHome_L");

LocalResult lr = lrh.create();
lr.setLevel(rvo.getLevel());
lr.setResultMessage(rvo.getResultMessage());
lr.setTimeStamp(rvo.getTimeStamp());
lr.setRule(rvo.getRule());
lr.setRuleSet(rvo.getRuleSet());

String id = lr.getPrimaryKey().toString();
lr.setId(id);

Collection col = new ArrayList();
for (Iterator iterator = resultProbeValues.iterator(); iterator.hasNext();) {
ResultProbeValueObject resultProbeValueObject = (ResultProbeValueObject) iterator.next();

ResultProbeNamePK pk = new ResultProbeNamePK();
pk.attributeName = resultProbeValueObject.getAttributeName();
pk.probeIdentifier = resultProbeValueObject.getProbeIdentifier();
LocalResultProbeName lrpn = null;
try {
lrpn = lrphn.findByPrimaryKey(pk);
} catch (FinderException e) {
//e.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
lrpn = lrphn.create(pk);
}

LocalResultProbe lrp = lrph.create();
lrp.setId(lrp.getPrimaryKey().toString());
lrp.setAttributeValue(resultProbeValueObject.getAttributeValue());
lrp.setResult(lr);
lrp.setTimestamp(rvo.getTimeStamp());
lrp.setResultProbeName(lrpn);
col.add(lrp);
}
//          lr.setResult1(col);
} catch (NamingException e) {
e.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
} catch (CreateException e) {
e.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
}


}
    */

    public Collection findAllResults() {
        Collection result = new ArrayList();
        try {
            Context initalContext = new InitialContext();

            LocalResultHome lrh = (LocalResultHome) initalContext.lookup("ResultEJBHome_L");

            Collection col = lrh.findAll();
            for (Iterator iterator = col.iterator(); iterator.hasNext();) {
                LocalResult localResult = (LocalResult) iterator.next();

                ResultValueObject rvo = new ResultValueObject();
                rvo.setLevel(localResult.getLevel());
                rvo.setResultMessage(localResult.getResultMessage());
                rvo.setTimeStamp(localResult.getTimeStamp());
                rvo.setId(localResult.getId());
                rvo.setRule(localResult.getRule());
                rvo.setRuleSet(localResult.getRuleSet());


                Collection resultProbes = localResult.getResult1();
                Collection valueObjectResult = new ArrayList();
                for (Iterator iterator1 = resultProbes.iterator(); iterator1.hasNext();) {
                    LocalResultProbe localResultProbe = (LocalResultProbe) iterator1.next();
                    ResultProbeValueObject rpvo = new ResultProbeValueObject();
                    rpvo.setAttributeName(localResultProbe.getResultProbeName().getAttributeName());
                    rpvo.setAttributeValue(localResultProbe.getAttributeValue());
                    rpvo.setProbeIdentifier(localResultProbe.getResultProbeName().getProbeIdentifier());
                    rpvo.setId(localResultProbe.getId());
                    rpvo.setTimestamp(localResultProbe.getTimestamp());
                    valueObjectResult.add(rpvo);
                }

                rvo.setResultProbes(valueObjectResult);


                result.add(rvo);
            }
        } catch (NamingException e) {
            e.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
        } catch (FinderException e) {
            e.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
        }
        return result;  //To change body of implemented methods use File | Settings | File Templates.
    }

    public Collection findAllResultsAfterTime(String time) {
        Collection result = new ArrayList();
        try {
            Context initalContext = new InitialContext();

            LocalResultHome lrh = (LocalResultHome) initalContext.lookup("ResultEJBHome_L");

            Collection col = lrh.findAllAfterTime(time);
            for (Iterator iterator = col.iterator(); iterator.hasNext();) {
                LocalResult localResult = (LocalResult) iterator.next();
                ResultValueObject rvo = new ResultValueObject();
                rvo.setLevel(localResult.getLevel());
                rvo.setResultMessage(localResult.getResultMessage());
                rvo.setTimeStamp(localResult.getTimeStamp());
                rvo.setId(localResult.getId());
                rvo.setRule(localResult.getRule());
                rvo.setRuleSet(localResult.getRuleSet());


                Collection resultProbes = localResult.getResult1();
                Collection valueObjectResult = new ArrayList();
                for (Iterator iterator1 = resultProbes.iterator(); iterator1.hasNext();) {
                    LocalResultProbe localResultProbe = (LocalResultProbe) iterator1.next();
                    ResultProbeValueObject rpvo = new ResultProbeValueObject();
                    rpvo.setAttributeName(localResultProbe.getResultProbeName().getAttributeName());
                    rpvo.setAttributeValue(localResultProbe.getAttributeValue());
                    rpvo.setProbeIdentifier(localResultProbe.getResultProbeName().getProbeIdentifier());
                    rpvo.setId(localResultProbe.getId());
                    rpvo.setTimestamp(localResultProbe.getTimestamp());
                    valueObjectResult.add(rpvo);
                }
                rvo.setResultProbes(valueObjectResult);

                result.add(rvo);
            }
        } catch (NamingException e) {
            e.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
        } catch (FinderException e) {
            e.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
        }
        return result;  //To change body of implemented methods use File | Settings | File Templates.
    }


    public Collection monitorRule(String ruleset, String rule, String begintime) {
        Collection result = new ArrayList();
        try {
            Context initalContext = new InitialContext();

            LocalResultHome lrh = (LocalResultHome) initalContext.lookup("ResultEJBHome_L");

            Collection col = lrh.findAllRuleAfterTime(ruleset, rule, begintime);

            logger.log(Level.FINEST, "l.size() = " + col.size());
            logger.log(Level.FINEST, "Begin time " + System.currentTimeMillis());

            for (Iterator iterator = col.iterator(); iterator.hasNext();) {
                LocalResult localResult = (LocalResult) iterator.next();

                ResultValueObject rvo = new ResultValueObject();
                rvo.setLevel(localResult.getLevel());
                rvo.setResultMessage(localResult.getResultMessage());
                rvo.setTimeStamp(localResult.getTimeStamp());
                rvo.setId(localResult.getId());
                rvo.setRule(localResult.getRule());
                rvo.setRuleSet(localResult.getRuleSet());


                Collection resultProbes = localResult.getResult1();
                Collection valueObjectResult = new ArrayList();
                for (Iterator iterator1 = resultProbes.iterator(); iterator1.hasNext();) {
                    LocalResultProbe localResultProbe = (LocalResultProbe) iterator1.next();
                    ResultProbeValueObject rpvo = new ResultProbeValueObject();
                    rpvo.setAttributeName(localResultProbe.getResultProbeName().getAttributeName());
                    rpvo.setAttributeValue(localResultProbe.getAttributeValue());
                    rpvo.setProbeIdentifier(localResultProbe.getResultProbeName().getProbeIdentifier());
                    rpvo.setId(localResultProbe.getId());
                    rpvo.setTimestamp(localResultProbe.getTimestamp());
                    valueObjectResult.add(rpvo);
                }
                rvo.setResultProbes(valueObjectResult);

                result.add(rvo);
            }
            logger.log(Level.FINEST, "End time " + System.currentTimeMillis());
        } catch (NamingException e) {
            e.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
        } catch (FinderException e) {
            e.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
        }
        return result;  //To change body of implemented methods use File | Settings | File Templates.
    }

    /*
    // The same as the previous methode. It only uses hibernate instead of cmp2
    // No performance improvemence have been seen when using this methode instead of the one
    // using cmp2. Even cmp2 performs better
    public Collection monitorRule(String ruleset, String rule, String begintime) {
        Collection result = new ArrayList();

        Context initalContext = null;
        SessionFactory sf = null;
        try {
            initalContext = new InitialContext();
            sf = (SessionFactory) initalContext.lookup("hibernate");
        } catch (NamingException e) {
            e.printStackTrace();
        }

        if (sf != null) {
            Session session = sf.openSession();

            Criteria criteria = session.createCriteria(Result.class);
            criteria.add(Expression.eq(("ruleSet"), ruleset));
            criteria.add(Expression.eq(("rule"), rule));
            criteria.add(Expression.ge(("timestamp"), new Long(Long.parseLong(begintime))));

            List l = criteria.list();

            System.out.println("l.size() = " + l.size());
            System.out.println("Begin time "+System.currentTimeMillis());
            for (int i = 0; i < l.size(); i++) {
                Result result1 = (Result) l.get(i);
                System.out.println("i = " + i);
                ResultValueObject rvo = new ResultValueObject();
                rvo.setLevel(result1.getLevel());
                rvo.setResultMessage(result1.getResultMessage());
                rvo.setTimeStamp(result1.getTimestamp());
                rvo.setId(result1.getId().toString());
                rvo.setRule(result1.getRule());
                rvo.setRuleSet(result1.getRuleSet());


                Collection resultProbes = result1.getResultProbes();

                Collection valueObjectResult = new ArrayList();
                for (Iterator iterator1 = resultProbes.iterator(); iterator1.hasNext();) {
                    ResultProbe localResultProbe = (ResultProbe) iterator1.next();
                    ResultProbeValueObject rpvo = new ResultProbeValueObject();
                    rpvo.setAttributeName(localResultProbe.getProbeName().getAttributeName());
                    rpvo.setAttributeValue(localResultProbe.getAttributeValue());
                    rpvo.setProbeIdentifier(localResultProbe.getProbeName().getProbeIdentifier());
                    rpvo.setId(localResultProbe.getId().toString());
                    rpvo.setTimestamp(localResultProbe.getTimestamp());
                    valueObjectResult.add(rpvo);
                }
                rvo.setResultProbes(valueObjectResult);

                result.add(rvo);

            }

            System.out.println("End time "+System.currentTimeMillis());
            session.close();
        }


        return result;
    }
                */

    public Collection monitorRule(String ruleset, String rule) {
        Collection result = new ArrayList();
        try {
            Context initalContext = new InitialContext();

            LocalResultHome lrh = (LocalResultHome) initalContext.lookup("ResultEJBHome_L");

            Collection col = lrh.findAllRule(ruleset, rule);
            for (Iterator iterator = col.iterator(); iterator.hasNext();) {
                LocalResult localResult = (LocalResult) iterator.next();

                ResultValueObject rvo = new ResultValueObject();
                rvo.setLevel(localResult.getLevel());
                rvo.setResultMessage(localResult.getResultMessage());
                rvo.setTimeStamp(localResult.getTimeStamp());
                rvo.setId(localResult.getId());
                rvo.setRule(localResult.getRule());
                rvo.setRuleSet(localResult.getRuleSet());


                Collection resultProbes = localResult.getResult1();
                Collection valueObjectResult = new ArrayList();
                for (Iterator iterator1 = resultProbes.iterator(); iterator1.hasNext();) {
                    LocalResultProbe localResultProbe = (LocalResultProbe) iterator1.next();
                    ResultProbeValueObject rpvo = new ResultProbeValueObject();
                    rpvo.setAttributeName(localResultProbe.getResultProbeName().getAttributeName());
                    rpvo.setAttributeValue(localResultProbe.getAttributeValue());
                    rpvo.setProbeIdentifier(localResultProbe.getResultProbeName().getProbeIdentifier());
                    rpvo.setId(localResultProbe.getId());
                    rpvo.setTimestamp(localResultProbe.getTimestamp());
                    valueObjectResult.add(rpvo);
                }
                rvo.setResultProbes(valueObjectResult);

                result.add(rvo);
            }
        } catch (NamingException e) {
            e.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
        } catch (FinderException e) {
            e.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
        }
        return result;  //To change body of implemented methods use File | Settings | File Templates.
    }

    public Collection findAllAttributes(String attributeName, String probeIdentifier) {
        Context initalContext;
        Collection result = new ArrayList();
        try {
            initalContext = new InitialContext();
            LocalResultProbeHome lrh = (LocalResultProbeHome) initalContext.lookup("ResultProbeEJBHome_L");

            Collection col = lrh.findAllAttributes(attributeName, probeIdentifier);

            for (Iterator iterator = col.iterator(); iterator.hasNext();) {
                LocalResultProbe localResultProbe = (LocalResultProbe) iterator.next();

                ResultProbeValueObject rpvo = new ResultProbeValueObject();
                rpvo.setAttributeName(localResultProbe.getResultProbeName().getAttributeName());
                rpvo.setAttributeValue(localResultProbe.getAttributeValue());
                rpvo.setId(localResultProbe.getId());
                rpvo.setProbeIdentifier(localResultProbe.getResultProbeName().getProbeIdentifier());
                rpvo.setTimestamp(localResultProbe.getTimestamp());
                result.add(rpvo);
            }
        } catch (NamingException e) {
            e.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
        } catch (FinderException e) {
            e.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
        }
        return result;  //To change body of implemented methods use File | Settings | File Templates.
    }

    public Collection findAllAttributes(String attributeName, String probeIdentifier, String timestamp) {
        Context initalContext;
        Collection result = new ArrayList();
        try {
            initalContext = new InitialContext();
            LocalResultProbeHome lrh = (LocalResultProbeHome) initalContext.lookup("ResultProbeEJBHome_L");

            Collection col = lrh.findAllAttributes(attributeName, probeIdentifier, timestamp);

            for (Iterator iterator = col.iterator(); iterator.hasNext();) {
                LocalResultProbe localResultProbe = (LocalResultProbe) iterator.next();

                ResultProbeValueObject rpvo = new ResultProbeValueObject();
                rpvo.setAttributeName(localResultProbe.getResultProbeName().getAttributeName());
                rpvo.setAttributeValue(localResultProbe.getAttributeValue());
                rpvo.setId(localResultProbe.getId());
                rpvo.setProbeIdentifier(localResultProbe.getResultProbeName().getProbeIdentifier());
                rpvo.setTimestamp(localResultProbe.getTimestamp());
                result.add(rpvo);
            }
        } catch (NamingException e) {
            e.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
        } catch (FinderException e) {
            e.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
        }
        return result;  //To change body of implemented methods use File | Settings | File Templates.
    }

    public Collection findRule() {
        Context initalContext;
        Collection result = new ArrayList();
        try {
            initalContext = new InitialContext();
            LocalResultHome lrh = (LocalResultHome) initalContext.lookup("ResultEJBHome_L");
            result = lrh.selectRule();
        } catch (NamingException e) {
            e.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
        } catch (FinderException e) {
            e.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
        }
        return result;  //To change body of implemented methods use File | Settings | File Templates.
    }

    public Collection findAttributesForARule(String rulename) {
        Context initalContext;
        Collection result = new ArrayList();
        try {
            initalContext = new InitialContext();
            LocalResultProbeHome lrh = (LocalResultProbeHome) initalContext.lookup("ResultProbeEJBHome_L");
            result = lrh.selectAllAttributesOfARule(rulename);
        } catch (NamingException e) {
            e.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
        } catch (FinderException e) {
            e.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
        }
        return result;  //To change body of implemented methods use File | Settings | File Templates.
    }


    public Collection findAllDistinctProbeIdentifier() {
        Context initalContext;
        Collection result = new ArrayList();
        try {
            initalContext = new InitialContext();
            //LocalResultProbeHome lrh = (LocalResultProbeHome) initalContext.lookup("ResultProbeEJBHome_L");
            //result = lrh.selectAllDistinctProbes();
            LocalResultProbeNameHome lrh = (LocalResultProbeNameHome) initalContext.lookup("ResultProbeNameEJBHome_L");
            result = lrh.selectAllDistinctProbes();
        } catch (NamingException e) {
            e.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
        } catch (FinderException e) {
            e.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
        }
        return result;  //To change body of implemented methods use File | Settings | File Templates.    }
    }

    public Collection findAllDistinctProbeAttributes(String probeName) {
        Context initalContext;
        Collection result = new ArrayList();
        try {
            initalContext = new InitialContext();
            //LocalResultProbeHome lrh = (LocalResultProbeHome) initalContext.lookup("ResultProbeEJBHome_L");
            //result = lrh.selectAllDistinctProbeAttributes(probeName);
            LocalResultProbeNameHome lrh = (LocalResultProbeNameHome) initalContext.lookup("ResultProbeNameEJBHome_L");
            result = lrh.selectAllDistinctProbeAttributes(probeName);
        } catch (NamingException e) {
            e.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
        } catch (FinderException e) {
            e.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
        }
        return result;  //To change body of implemented methods use File | Settings | File Templates.    }
    }
    /*
    // This methode uses cmp2 to add the cluster status. It does the same operation
    // as the next methode but less effective.
    // If you want to delete hibernate, uncomment this methode and comment the following one
public void addClusterStatus(ClusterStatus clusterStatus) {
  if (clusterStatus != null) {
      Context initalContext;
      try {
          initalContext = new InitialContext();

          LocalClusterStatusHome lcsm = (LocalClusterStatusHome) initalContext.lookup("ClusterStatusEJBHome_L");
          LocalHostStatusHome lhsm = (LocalHostStatusHome) initalContext.lookup("HostStatusEJBHome_L");
          LocalDomainStatusHome ldsm = (LocalDomainStatusHome) initalContext.lookup("DomainStatusEJBHome_L");
          LocalPumpStatusHome lpumpsm = (LocalPumpStatusHome) initalContext.lookup("PumpStatusEJBHome_L");
          LocalInstanceStatusHome lism = (LocalInstanceStatusHome) initalContext.lookup("InstanceStatusEJBHome_L");
          LocalProbeStatusHome lprobesm = (LocalProbeStatusHome) initalContext.lookup("ProbeStatusEJBHome_L");

          String timestamp = Long.toString(clusterStatus.getTimestamp());
          LocalClusterStatus lcs = lcsm.create();
          String id = lcs.getPrimaryKey().toString();
          lcs.setTimestamp(timestamp);
          lcs.setId(id);
          lcs.setName(clusterStatus.getIdentifier());
          lcs.setStatus(clusterStatus.getStatus());

          Collection hosts = clusterStatus.getHosts().values();
          Collection localHosts = new ArrayList();
          for (Iterator iterator = hosts.iterator(); iterator.hasNext();) {
              HostStatus hostStatus = (HostStatus) iterator.next();

              System.out.println("Trying to create a host with id = " + timestamp);
              LocalHostStatus localHostStatus = lhsm.create(timestamp, hostStatus.getIdentifier());
              localHostStatus.setCluster(lcs);
              System.out.println("hostStatus.getIdentifier() = " + hostStatus.getIdentifier());
              //localHostStatus.setIdentifier(hostStatus.getIdentifier());
              localHostStatus.setStatus(hostStatus.getStatus());

              PumpStatus pumpStatus = hostStatus.getPumpStatus();
              LocalPumpStatus localPumpStatus = lpumpsm.create(timestamp, pumpStatus.getIdentifier());
              localPumpStatus.setHost(localHostStatus);
              //localPumpStatus.setIdentifier(pumpStatus.getIdentifier());
              localPumpStatus.setStatus(pumpStatus.getStatus());

              Collection probes = pumpStatus.getProbes().values();
              System.out.println(" ------------ Number of probes in pump object : " + probes.size());
              Collection localProbes = new ArrayList();
              for (Iterator iterator1 = probes.iterator(); iterator1.hasNext();) {
                  ProbeStatus probeStatus = (ProbeStatus) iterator1.next();
                  LocalProbeStatus lprobeStatus = lprobesm.create(timestamp, probeStatus.getIdentifier());
                  //lprobeStatus.setIdentifier(probeStatus.getIdentifier());
                  lprobeStatus.setPump(localPumpStatus);
                  lprobeStatus.setStatus(probeStatus.getStatus());
                  localProbes.add(lprobeStatus);
              }
              System.out.println(" ------------ Number of probes in pump object 2222 : " + localProbes.size());
              localPumpStatus.setProbes(localProbes);

              localHostStatus.setPump(localPumpStatus);

              // domain
              Collection domains = hostStatus.getDomains().values();
              Collection localDomains = new ArrayList();
              for (Iterator iterator1 = domains.iterator(); iterator1.hasNext();) {
                  DomainStatus domainStatus = (DomainStatus) iterator1.next();
                  LocalDomainStatus localDomainStatus = ldsm.create(timestamp, domainStatus.getIdentifier());
                  localDomainStatus.setHost(localHostStatus);
                  localDomainStatus.setStatus(domainStatus.getStatus());
                  //localDomainStatus.setIdentifier(domainStatus.getIdentifier());

                  Collection instances = domainStatus.getInstances().values();
                  Collection localInstances = new ArrayList();
                  for (Iterator iterator2 = instances.iterator(); iterator2.hasNext();) {
                      InstanceStatus instanceStatus = (InstanceStatus) iterator2.next();

                      LocalInstanceStatus localInstanceStatus = lism.create(timestamp, instanceStatus.getIdentifier());
                      localInstanceStatus.setDomain(localDomainStatus);
                      //localInstanceStatus.setIdentifier(instanceStatus.getIdentifier());
                      localInstanceStatus.setStatus(instanceStatus.getStatus());

                      probes = instanceStatus.getProbes().values();
                      Collection localProbes2 = new ArrayList();
                      for (Iterator iterator3 = probes.iterator(); iterator3.hasNext();) {
                          ProbeStatus probeStatus = (ProbeStatus) iterator3.next();
                          LocalProbeStatus lprobeStatus = lprobesm.create(timestamp, probeStatus.getIdentifier());
                          //lprobeStatus.setIdentifier(probeStatus.getIdentifier());
                          // -> otherwise he writes the probe alos in the pump : lprobeStatus.setPump(localPumpStatus);
                          lprobeStatus.setStatus(probeStatus.getStatus());
                          lprobeStatus.setInstance(localInstanceStatus);
                          localProbes2.add(lprobeStatus);
                      }
                      localInstanceStatus.setProbes(localProbes2);
                      localInstances.add(localInstanceStatus);
                  }

                  localDomainStatus.setInstances(localInstances);

                  localDomains.add(localDomainStatus);
              }
              localHostStatus.setDomain(localDomains);

              localHosts.add(localHostStatus);
          }
          lcs.setHosts(localHosts);

      } catch (NamingException e) {
          e.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
      } catch (CreateException e) {
          e.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
      }
  }
}
    */

    /**
     * Methode uses hibernate inorder to add a clusterstatus
     * Cmp2 is possible by uncommenting the previous methode in the source
     *
     * @param clusterStatus The cluster status to add
     */
    public void addClusterStatus(ClusterStatus clusterStatus) {

        Context initalContext = null;
        SessionFactory sf = null;
        try {
            initalContext = new InitialContext();
            sf = (SessionFactory) initalContext.lookup("hibernate");
        } catch (NamingException e) {
            e.printStackTrace();
        }

        if (sf != null) {
            Session session = sf.openSession();
            Transaction tx = session.beginTransaction();

            String timestamp = Long.toString(clusterStatus.getTimestamp());

            HClusterStatus lcs = new HClusterStatus();

            lcs.setTimestamp(timestamp);
            lcs.setName(clusterStatus.getIdentifier());
            lcs.setStatus(clusterStatus.getStatus());

            session.save(lcs);

            logger.log(Level.FINEST, "lcs.getJpk() = " + lcs.getJpk());

            lcs.setId(lcs.getJpk().toString());
            session.update(lcs);
            logger.log(Level.FINEST, "lcs.getId() = " + lcs.getId());

            Collection hosts = clusterStatus.getHosts().values();
            Set localHosts = new TreeSet();
            for (Iterator iterator = hosts.iterator(); iterator.hasNext();) {
                HostStatus hostStatus = (HostStatus) iterator.next();
                logger.log(Level.FINEST, "Trying to create a host with id = " + timestamp);

                HHostStatus localHostStatus = new HHostStatus();
                localHostStatus.setTimestamp(timestamp);
                localHostStatus.setIdentifier(hostStatus.getIdentifier());
                localHostStatus.setCluster(lcs);
                logger.log(Level.FINEST, "hostStatus.getIdentifier() = " + hostStatus.getIdentifier());

                //localHostStatus.setIdentifier(hostStatus.getIdentifier());
                localHostStatus.setStatus(hostStatus.getStatus());

                session.save(localHostStatus);

                PumpStatus pumpStatus = hostStatus.getPumpStatus();
                HPumpStatus localPumpStatus = new HPumpStatus();
                localPumpStatus.setTimestamp(timestamp);
                localPumpStatus.setIdentifier(pumpStatus.getIdentifier());
                localPumpStatus.setHost(localHostStatus);
                localPumpStatus.setStatus(pumpStatus.getStatus());

                session.save(localPumpStatus);

                Collection probes = pumpStatus.getProbes().values();
                Set localProbes = new TreeSet();
                for (Iterator iterator1 = probes.iterator(); iterator1.hasNext();) {
                    ProbeStatus probeStatus = (ProbeStatus) iterator1.next();

                    HProbeStatus lprobeStatus = new HProbeStatus();
                    lprobeStatus.setTimestamp(timestamp);
                    lprobeStatus.setIdentifier(probeStatus.getIdentifier());
                    lprobeStatus.setPump(localPumpStatus);
                    lprobeStatus.setStatus(probeStatus.getStatus());
                    session.save(lprobeStatus);

                    localProbes.add(lprobeStatus);
                }

                logger.log(Level.FINEST, "------------ Number of probes in pump object 2222 : " + localProbes.size());
                localPumpStatus.setProbes(localProbes);
                session.update(localPumpStatus);

                localHostStatus.setPump(localPumpStatus);
                session.update(localHostStatus);

                // domain
                Collection domains = hostStatus.getDomains().values();
                Set localDomains = new TreeSet();
                for (Iterator iterator1 = domains.iterator(); iterator1.hasNext();) {
                    DomainStatus domainStatus = (DomainStatus) iterator1.next();

                    HDomainStatus localDomainStatus = new HDomainStatus();
                    localDomainStatus.setTimestamp(timestamp);
                    localDomainStatus.setIdentifier(domainStatus.getIdentifier());
                    localDomainStatus.setHost(localHostStatus);
                    localDomainStatus.setStatus(domainStatus.getStatus());
                    session.save(localDomainStatus);

                    Collection instances = domainStatus.getInstances().values();
                    Set localInstances = new TreeSet();
                    for (Iterator iterator2 = instances.iterator(); iterator2.hasNext();) {
                        InstanceStatus instanceStatus = (InstanceStatus) iterator2.next();

                        HInstanceStatus localInstanceStatus = new HInstanceStatus();
                        localInstanceStatus.setTimestamp(timestamp);
                        localInstanceStatus.setIdentifier(instanceStatus.getIdentifier());
                        localInstanceStatus.setDomain(localDomainStatus);
                        //localInstanceStatus.setIdentifier(instanceStatus.getIdentifier());
                        localInstanceStatus.setStatus(instanceStatus.getStatus());
                        session.save(localInstanceStatus);

                        probes = instanceStatus.getProbes().values();
                        Set localProbes2 = new TreeSet();
                        for (Iterator iterator3 = probes.iterator(); iterator3.hasNext();) {
                            ProbeStatus probeStatus = (ProbeStatus) iterator3.next();

                            HProbeStatus lprobeStatus = new HProbeStatus();
                            lprobeStatus.setTimestamp(timestamp);
                            lprobeStatus.setIdentifier(probeStatus.getIdentifier());
                            lprobeStatus.setStatus(probeStatus.getStatus());
                            lprobeStatus.setInstance(localInstanceStatus);
                            localProbes2.add(lprobeStatus);

                            session.save(lprobeStatus);
                        }
                        localInstanceStatus.setProbes(localProbes2);
                        localInstances.add(localInstanceStatus);

                        session.update(localInstanceStatus);
                    }
                    localDomainStatus.setInstances(localInstances);
                    session.update(localDomainStatus);

                    localDomains.add(localDomainStatus);
                }
                localHostStatus.setDomain(localDomains);
                session.update(localHostStatus);

                localHosts.add(localHostStatus);
            }
            lcs.setHosts(localHosts);
            session.update(lcs);

            tx.commit();

            /*
           // setting the id -> to stay cmp2 compatible
           tx = session.beginTransaction();
           session.refresh(lcs);
           lcs.setId(lcs.getJpk().toString());
           tx.commit();
            */

            session.close();
        }
    }


    public org.objectweb.jimys.commons.clusterstatus.ClusterStatus getLastClusterStatus(String identifier) {
        Context initalContext;
        try {
            initalContext = new InitialContext();
            LocalClusterStatusHome lcsh = (LocalClusterStatusHome) initalContext.lookup("ClusterStatusEJBHome_L");
            logger.log(Level.FINEST, "searching max");
            String max = lcsh.selectMaxTimeStamp(identifier);
            logger.log(Level.FINEST, "max found = " + max);

            if (max != null) {
                logger.log(Level.FINEST, "entering if in cluster status ");

                LocalClusterStatus lcs = lcsh.findClusterStatus(identifier, max);
                long timestamp = Long.parseLong(lcs.getTimestamp());
                org.objectweb.jimys.commons.clusterstatus.ClusterStatus cs = new org.objectweb.jimys.commons.clusterstatus.ClusterStatus(lcs.getName(), lcs.getStatus(), timestamp);

                Collection hosts = lcs.getHosts();
                for (Iterator iterator = hosts.iterator(); iterator.hasNext();) {
                    LocalHostStatus localHostStatus = (LocalHostStatus) iterator.next();
                    HostStatus hostStatus = new HostStatus(localHostStatus.getIdentifier(), localHostStatus.getStatus(), timestamp);
                    cs.addHostStatus(hostStatus);

                    LocalPumpStatus lpumpStatus = localHostStatus.getPump();
                    PumpStatus pumpStatus = new PumpStatus(lpumpStatus.getIdentifier(), lpumpStatus.getStatus(), timestamp);

                    Collection probes = lpumpStatus.getProbes();
                    for (Iterator iterator1 = probes.iterator(); iterator1.hasNext();) {
                        LocalProbeStatus localProbeStatus = (LocalProbeStatus) iterator1.next();

                        ProbeStatus probeStatus = new ProbeStatus(localProbeStatus.getIdentifier(), localProbeStatus.getStatus(), timestamp);
                        pumpStatus.addProbeStatus(probeStatus);
                    }

                    Collection domains = localHostStatus.getDomain();
                    for (Iterator iterator1 = domains.iterator(); iterator1.hasNext();) {
                        LocalDomainStatus localDomainStatus = (LocalDomainStatus) iterator1.next();
                        DomainStatus domainStatus = new DomainStatus(localDomainStatus.getIdentifier(), localDomainStatus.getStatus(), timestamp);
                        hostStatus.addDomainStatus(domainStatus);

                        Collection instances = localDomainStatus.getInstances();
                        for (Iterator iterator2 = instances.iterator(); iterator2.hasNext();) {
                            LocalInstanceStatus localInstanceStatus = (LocalInstanceStatus) iterator2.next();
                            InstanceStatus instanceStatus = new InstanceStatus(localInstanceStatus.getIdentifier(), localInstanceStatus.getStatus(), timestamp);
                            domainStatus.addInstanceStatus(instanceStatus);

                            probes = localInstanceStatus.getProbes();
                            for (Iterator iterator3 = probes.iterator(); iterator3.hasNext();) {
                                LocalProbeStatus localProbeStatus = (LocalProbeStatus) iterator3.next();
                                ProbeStatus probeStatus = new ProbeStatus(localProbeStatus.getIdentifier(), localProbeStatus.getStatus(), timestamp);
                                instanceStatus.addProbeStatus(probeStatus);
                            }
                        }
                    }
                    hostStatus.setPumpStatus(pumpStatus);
                    cs.addHostStatus(hostStatus);
                }
                logger.log(Level.FINEST, "Finished cluster status : " + cs);
                return cs;
            }

        } catch (NamingException e) {
            e.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
        } catch (FinderException e) {
            e.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
        }
        return null;  //To change body of implemented methods use File | Settings | File Templates.    }

    }

    public Collection getClusterNames() {
        Context initalContext;
        Collection result = new ArrayList();
        try {
            initalContext = new InitialContext();
            LocalClusterStatusHome lch = (LocalClusterStatusHome) initalContext.lookup("ClusterStatusEJBHome_L");
            result = lch.selectDistinctClusterName();
        } catch (NamingException e) {
            e.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
        } catch (FinderException e) {
            e.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
        }
        return result;  //To change body of implemented methods use File | Settings | File Templates.    }
    }

    public Collection findAllResults(Integer beginRowNumber) {
        Collection result = new ArrayList();
        try {
            Context initalContext = new InitialContext();

            LocalResultHome lrh = (LocalResultHome) initalContext.lookup("ResultEJBHome_L");

            Collection col = lrh.findAllOptimized(beginRowNumber);
            for (Iterator iterator = col.iterator(); iterator.hasNext();) {
                LocalResult localResult = (LocalResult) iterator.next();

                ResultValueObject rvo = new ResultValueObject();
                rvo.setLevel(localResult.getLevel());
                rvo.setResultMessage(localResult.getResultMessage());
                rvo.setTimeStamp(localResult.getTimeStamp());
                rvo.setId(localResult.getId());
                rvo.setRule(localResult.getRule());
                rvo.setRuleSet(localResult.getRuleSet());


                Collection resultProbes = localResult.getResult1();
                Collection valueObjectResult = new ArrayList();
                for (Iterator iterator1 = resultProbes.iterator(); iterator1.hasNext();) {
                    LocalResultProbe localResultProbe = (LocalResultProbe) iterator1.next();
                    ResultProbeValueObject rpvo = new ResultProbeValueObject();
                    rpvo.setAttributeName(localResultProbe.getResultProbeName().getAttributeName());
                    rpvo.setAttributeValue(localResultProbe.getAttributeValue());
                    rpvo.setProbeIdentifier(localResultProbe.getResultProbeName().getProbeIdentifier());
                    rpvo.setId(localResultProbe.getId());
                    rpvo.setTimestamp(localResultProbe.getTimestamp());
                    valueObjectResult.add(rpvo);
                }

                rvo.setResultProbes(valueObjectResult);

                result.add(rvo);
            }
        } catch (NamingException e) {
            e.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
        } catch (FinderException e) {
            e.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
        }
        return result;  //To change body of implemented methods use File | Settings | File Templates.
    }

    public Collection findAllResultsWithLevel(String level) {
        Collection result = new ArrayList();
        try {
            Context initalContext = new InitialContext();

            LocalResultHome lrh = (LocalResultHome) initalContext.lookup("ResultEJBHome_L");

            Collection col = lrh.findAllResultWithLevel(level);
            for (Iterator iterator = col.iterator(); iterator.hasNext();) {
                LocalResult localResult = (LocalResult) iterator.next();

                ResultValueObject rvo = new ResultValueObject();
                rvo.setLevel(localResult.getLevel());
                rvo.setResultMessage(localResult.getResultMessage());
                rvo.setTimeStamp(localResult.getTimeStamp());
                rvo.setId(localResult.getId());
                rvo.setRule(localResult.getRule());
                rvo.setRuleSet(localResult.getRuleSet());

                /*   // Will be too slow if filling also the relation !!!
Collection resultProbes = localResult.getResult1();
Collection valueObjectResult = new ArrayList();
for (Iterator iterator1 = resultProbes.iterator(); iterator1.hasNext();) {
LocalResultProbe localResultProbe = (LocalResultProbe) iterator1.next();
ResultProbeValueObject rpvo = new ResultProbeValueObject();
rpvo.setAttributeName(localResultProbe.getResultProbeName().getAttributeName());
rpvo.setAttributeValue(localResultProbe.getAttributeValue());
rpvo.setProbeIdentifier(localResultProbe.getResultProbeName().getProbeIdentifier());
rpvo.setId(localResultProbe.getId());
rpvo.setTimestamp(localResultProbe.getTimestamp());
valueObjectResult.add(rpvo);
}

rvo.setResultProbes(valueObjectResult);
                */
                result.add(rvo);
            }
        } catch (NamingException e) {
            e.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
        } catch (FinderException e) {
            e.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
        }
        return result;  //To change body of implemented methods use File | Settings | File Templates.
    }


    public void changeLevel(String resultId, String level) {
        try {
            Context initalContext = new InitialContext();

            LocalResultHome lrh = (LocalResultHome) initalContext.lookup("ResultEJBHome_L");
            Integer pk = new Integer(Integer.parseInt(resultId));
            LocalResult lr = lrh.findByPrimaryKey(pk);

            lr.setLevel(level);
        } catch (NamingException e) {
            e.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
        } catch (FinderException e) {
            e.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
        }
    }

    /**
     * Methode uses hibernate to delete all the data from the table.
     * If you want to delete hibernate, you have to implement it using pure JDBC or
     * cmp2. But cmp2 will be very slow, because there is no bulk delete
     */
    public void clearAllTables() {

        Context initalContext = null;
        SessionFactory sf = null;
        try {
            initalContext = new InitialContext();
            sf = (SessionFactory) initalContext.lookup("hibernate");
        } catch (NamingException e) {
            e.printStackTrace();
        }

        if (sf != null) {
            Session session = sf.openSession();
            Transaction tx = session.beginTransaction();

            session.createQuery("delete from org.objectweb.jimys.dbConnection.hibernate.beans.HClusterStatus").executeUpdate();
            session.createQuery("delete from org.objectweb.jimys.dbConnection.hibernate.beans.HHostStatus").executeUpdate();
            session.createQuery("delete from org.objectweb.jimys.dbConnection.hibernate.beans.HDomainStatus").executeUpdate();
            session.createQuery("delete from org.objectweb.jimys.dbConnection.hibernate.beans.HInstanceStatus").executeUpdate();
            session.createQuery("delete from org.objectweb.jimys.dbConnection.hibernate.beans.HProbeStatus").executeUpdate();
            session.createQuery("delete from org.objectweb.jimys.dbConnection.hibernate.beans.HPumpStatus").executeUpdate();
            session.createQuery("delete from org.objectweb.jimys.dbConnection.hibernate.beans.ResultProbe").executeUpdate();
            session.createQuery("delete from org.objectweb.jimys.dbConnection.hibernate.beans.Result").executeUpdate();
            session.createQuery("delete from org.objectweb.jimys.dbConnection.hibernate.beans.ProbeName").executeUpdate();
            session.createQuery("delete from org.objectweb.jimys.dbConnection.hibernate.beans.HCartography").executeUpdate();

            tx.commit();
            session.close();
        }

        // Pure JDBC implementation of the methode
        //   Savepoint sp = null;
        // TODO Weak -> use hibernate instead
        /*
        Connection conn = null;
        try {
            Context initalContext = new InitialContext();

            DataSource datasource = (DataSource) initalContext.lookup("jdbc_1");

            conn = datasource.getConnection();
            conn.setAutoCommit(false);
            //         sp = conn.setSavepoint();

            PreparedStatement ps = conn.prepareStatement("truncate table resultprobeejb_");
            ps.executeUpdate();
            ps.close();
            ps = conn.prepareStatement("truncate table resultejb_");
            ps.executeUpdate();
            ps.close();
            ps = conn.prepareStatement("truncate table clusterstatusejb_");
            ps.executeUpdate();
            ps.close();
            ps = conn.prepareStatement("truncate table domainstatusejb_");
            ps.executeUpdate();
            ps.close();
            ps = conn.prepareStatement("truncate table hoststatusejb_");
            ps.executeUpdate();
            ps.close();
            ps = conn.prepareStatement("truncate table instancestatusejb_");
            ps.executeUpdate();
            ps.close();
            ps = conn.prepareStatement("truncate table probestatusejb_");
            ps.executeUpdate();
            ps.close();
            ps = conn.prepareStatement("truncate table pumpstatusejb_");
            ps.executeUpdate();
            ps.close();

            conn.commit();
            conn.close();
        } catch (NamingException e) {
            e.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
        } catch (SQLException e) {
            e.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
        }
           */
    }


    /**
     * This methode uses hibernate inorder to delete the data between the time slice
     * Example of pure jdbc is commented in the methode
     *
     * @param start begin time
     * @param end   end time
     */
    public void clearTimePeriod(long start, long end) {

        Context initalContext = null;
        SessionFactory sf = null;
        try {
            initalContext = new InitialContext();
            sf = (SessionFactory) initalContext.lookup("hibernate");
        } catch (NamingException e) {
            e.printStackTrace();
        }

        if (sf != null) {
            Session session = sf.openSession();
            Transaction tx = session.beginTransaction();

            System.out.println("Executing clearTimePeriod : start " + start + " end " + end);

            Query q = session.createQuery("delete from org.objectweb.jimys.dbConnection.hibernate.beans.HClusterStatus where timestamp > :start and timestamp < :end");
            q.setString("start", Long.toString(start));
            q.setString("end", Long.toString(end));
            q.executeUpdate();

            q = session.createQuery("delete from org.objectweb.jimys.dbConnection.hibernate.beans.HHostStatus where timestamp > :start and timestamp < :end");
            q.setString("start", Long.toString(start));
            q.setString("end", Long.toString(end));
            q.executeUpdate();

            q = session.createQuery("delete from org.objectweb.jimys.dbConnection.hibernate.beans.HDomainStatus where timestamp > :start and timestamp < :end");
            q.setString("start", Long.toString(start));
            q.setString("end", Long.toString(end));
            q.executeUpdate();

            q = session.createQuery("delete from org.objectweb.jimys.dbConnection.hibernate.beans.HInstanceStatus where timestamp > :start and timestamp < :end");
            q.setString("start", Long.toString(start));
            q.setString("end", Long.toString(end));
            q.executeUpdate();

            q = session.createQuery("delete from org.objectweb.jimys.dbConnection.hibernate.beans.HProbeStatus where timestamp > :start and timestamp < :end");
            q.setString("start", Long.toString(start));
            q.setString("end", Long.toString(end));
            q.executeUpdate();

            q = session.createQuery("delete from org.objectweb.jimys.dbConnection.hibernate.beans.HPumpStatus where timestamp > :start and timestamp < :end");
            q.setString("start", Long.toString(start));
            q.setString("end", Long.toString(end));
            q.executeUpdate();

            // timstamp is a long
            q = session.createQuery("delete from org.objectweb.jimys.dbConnection.hibernate.beans.ResultProbe where timestamp > :start and timestamp < :end");
            q.setLong("start", start);
            q.setLong("end", end);
            q.executeUpdate();

            q = session.createQuery("delete from org.objectweb.jimys.dbConnection.hibernate.beans.Result where timestamp > :start and timestamp < :end");
            q.setLong("start", start);
            q.setLong("end", end);
            q.executeUpdate();

            q = session.createQuery("delete from org.objectweb.jimys.dbConnection.hibernate.beans.HCartography where time > :start and time < :end");
            q.setLong("start", start);
            q.setLong("end", end);
            q.executeUpdate();

//                session.createQuery("delete from org.objectweb.jimys.dbConnection.hibernate.beans.ProbeName").executeUpdate();

            tx.commit();
            session.close();
        }

        // Pure JDBC to do the same thing
        //   Savepoint sp = null;
        /*
       Connection conn = null;
       try {
           Context initalContext = new InitialContext();

           DataSource datasource = (DataSource) initalContext.lookup("jdbc_1");

           conn = datasource.getConnection();
           conn.setAutoCommit(false);
           //         sp = conn.setSavepoint();

           PreparedStatement ps = conn.prepareStatement("delete * from resultprobeejb_ where timeStamp >= "+start+" and timeStamp <="+end);
           ps.executeUpdate();
           ps.close();
           ps = conn.prepareStatement("delete * from resultejb_ where timeStamp >= "+start+" and timeStamp <="+end);
           ps.executeUpdate();
           ps.close();
           ps = conn.prepareStatement("delete * from clusterstatusejb_ where timeStamp >= "+start+" and timeStamp <="+end);
           ps.executeUpdate();
           ps.close();
           ps = conn.prepareStatement("delete * from domainstatusejb_ where timeStamp >= "+start+" and timeStamp <="+end);
           ps.executeUpdate();
           ps.close();
           ps = conn.prepareStatement("delete * from hoststatusejb_ where timeStamp >= "+start+" and timeStamp <="+end);
           ps.executeUpdate();
           ps.close();
           ps = conn.prepareStatement("delete * from instancestatusejb_ where timeStamp >= "+start+" and timeStamp <="+end);
           ps.executeUpdate();
           ps.close();
           ps = conn.prepareStatement("delete * from probestatusejb_ where timeStamp >= "+start+" and timeStamp <="+end);
           ps.executeUpdate();
           ps.close();
           ps = conn.prepareStatement("delete * from pumpstatusejb_ where timeStamp >= "+start+" and timeStamp <="+end);
           ps.executeUpdate();
           ps.close();

           conn.commit();
           conn.close();
       } catch (NamingException e) {
           e.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
       } catch (SQLException e) {
           e.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.

       }
        */
    }

    /**
     * Adding the carthography to the database. This methode uses hibernate.
     * The same operation may be done with cmp2 by uncommenting the following methode
     *
     * @param carthography The carthography to add
     */
    public void addCartography(Cartography carthography) {
        Context initalContext = null;
        SessionFactory sf = null;
        try {
            initalContext = new InitialContext();
            sf = (SessionFactory) initalContext.lookup("hibernate");
        } catch (NamingException e) {
            e.printStackTrace();
        }

        if (sf != null) {
            Session session = sf.openSession();
            Transaction tx = session.beginTransaction();


            String hostname = carthography.getIdentifierHostname();
            Long time = new Long(carthography.getTimeStamp());

            Hashtable controllers = carthography.getAttributes();
            Enumeration enumeration = controllers.keys();

            logger.log(Level.FINEST, "CARTOGRAPHY : ENUMHASNEXT " + enumeration.hasMoreElements());

            while (enumeration.hasMoreElements()) {
                String key = (String) enumeration.nextElement();
                Collection probes = (Collection) controllers.get(key);

                logger.log(Level.FINEST, "CARTOGRAPHY : COLLECTION SIZE " + probes.size());
                for (Iterator iterator = probes.iterator(); iterator.hasNext();) {
                    AbstractProbe abstractProbe = (AbstractProbe) iterator.next();

                    Hashtable subProbes = abstractProbe.getAttributes();
                    Enumeration enumSubProbes = subProbes.keys();
                    int deviceNbr = 0;
                    logger.log(Level.FINEST, "CARTOGRAPHY : enumSubProbes ENUMHASNEXT " + enumSubProbes.hasMoreElements());
                    while (enumSubProbes.hasMoreElements()) {
                        //for example carthocontroller
                        String attributeName = (String) enumSubProbes.nextElement();
                        String attributeValue = (String) subProbes.get(attributeName);

                        HCartography lc = new HCartography();
                        lc.setAttributeName(attributeName);
                        lc.setAttributeValue(attributeValue);
                        lc.setHostID(hostname);
                        lc.setProbeID(abstractProbe.getIdentifier());
                        lc.setTime(time);
                        lc.setDeviceNbr(new Integer(deviceNbr++));

                        session.save(lc);

                    }


                }

            }
            tx.commit();
            session.close();
        }

    }

    /*
      CMP2 version of the methode
        public void addCartography(Cartography carthography) {
            try {
                Context initalContext = new InitialContext();

                LocalCartographyHome lch = (LocalCartographyHome) initalContext.lookup("CartographyEJBHome_L");

                String hostname = carthography.getIdentifierHostname();
                Long time = new Long(carthography.getTimeStamp());

                Hashtable controllers = carthography.getAttributes();
                Enumeration enumeration = controllers.keys();

                System.out.println("CARTOGRAPHY : ENUMHASNEXT " + enumeration.hasMoreElements());

                while (enumeration.hasMoreElements()) {
                    String key = (String) enumeration.nextElement();
                    Collection probes = (Collection) controllers.get(key);

                    System.out.println("CARTOGRAPHY : COLLECTION SIZE " + probes.size());
                    for (Iterator iterator = probes.iterator(); iterator.hasNext();) {
                        AbstractProbe abstractProbe = (AbstractProbe) iterator.next();

                        Hashtable subProbes = abstractProbe.getAttributes();
                        Enumeration enumSubProbes = subProbes.keys();
                        int deviceNbr = 0;
                        System.out.println("CARTOGRAPHY : enumSubProbes ENUMHASNEXT " + enumSubProbes.hasMoreElements());
                        while (enumSubProbes.hasMoreElements()) {
                            //for example carthocontroller
                            String attributeName = (String) enumSubProbes.nextElement();
                            String attributeValue = (String) subProbes.get(attributeName);

                            LocalCartography lc = lch.create();
                            lc.setAttributeName(attributeName);
                            lc.setAttributeValue(attributeValue);
                            lc.setHostID(hostname);
                            lc.setProbeID(abstractProbe.getIdentifier());
                            lc.setTime(time);
                            lc.setDeviceNbr(new Integer(deviceNbr++));
                        }


                    }

                }

            } catch (NamingException e) {
                e.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
            } catch (CreateException e) {
                e.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
            }
        }
    */

    public Collection getCartography(String hostname) {
        Collection attributes = new ArrayList();
        try {
            Context initalContext = new InitialContext();

            LocalCartographyHome lch = (LocalCartographyHome) initalContext.lookup("CartographyEJBHome_L");

            Long maxTimeStamp = lch.selectMaxTimeStamp(hostname);

            Collection clc = lch.findCartography(hostname, maxTimeStamp);

            String probeID = null;
            Cartography cart = null;

            for (Iterator iterator = clc.iterator(); iterator.hasNext();) {
                LocalCartography cartography = (LocalCartography) iterator.next();
                if (probeID == null) {
                    probeID = cartography.getProbeID();
                    cart = new Cartography(maxTimeStamp.longValue(), probeID, new Hashtable());
                    cart.getAttributes().put(cartography.getAttributeName(), cartography.getAttributeValue());
                } else {
                    if (!probeID.equals(cartography.getProbeID())) {
                        attributes.add(cart);
                        probeID = cartography.getProbeID();
                        cart = new Cartography(maxTimeStamp.longValue(), probeID, new Hashtable());
                    }

                    cart.getAttributes().put(cartography.getAttributeName(), cartography.getAttributeValue());
                }

            }

        } catch (NamingException e) {
            e.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
        } catch (FinderException e) {
            e.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
        }
        return attributes;  //To change body of implemented methods use File | Settings | File Templates.
    }

    public Collection findAllResultsWithLevel(String level, String probeidentifier) {
        Collection result = new ArrayList();
        try {
            Context initalContext = new InitialContext();

            LocalResultHome lrh = (LocalResultHome) initalContext.lookup("ResultEJBHome_L");

            logger.log(Level.FINEST, "Level : " + level + " probeidentifier " + probeidentifier);

            Collection col = lrh.findAllResultWithLevel(level, probeidentifier);
            for (Iterator iterator = col.iterator(); iterator.hasNext();) {
                LocalResult localResult = (LocalResult) iterator.next();

                ResultValueObject rvo = new ResultValueObject();
                rvo.setLevel(localResult.getLevel());
                rvo.setResultMessage(localResult.getResultMessage());
                rvo.setTimeStamp(localResult.getTimeStamp());
                rvo.setId(localResult.getId());
                rvo.setRule(localResult.getRule());
                rvo.setRuleSet(localResult.getRuleSet());


                Collection resultProbes = localResult.getResult1();
                Collection valueObjectResult = new ArrayList();
                for (Iterator iterator1 = resultProbes.iterator(); iterator1.hasNext();) {
                    LocalResultProbe localResultProbe = (LocalResultProbe) iterator1.next();
                    ResultProbeValueObject rpvo = new ResultProbeValueObject();
                    rpvo.setAttributeName(localResultProbe.getResultProbeName().getAttributeName());
                    rpvo.setAttributeValue(localResultProbe.getAttributeValue());
                    rpvo.setProbeIdentifier(localResultProbe.getResultProbeName().getProbeIdentifier());
                    rpvo.setId(localResultProbe.getId());
                    rpvo.setTimestamp(localResultProbe.getTimestamp());
                    valueObjectResult.add(rpvo);
                }

                rvo.setResultProbes(valueObjectResult);

                result.add(rvo);
            }
        } catch (NamingException e) {
            e.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
        } catch (FinderException e) {
            e.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
        }
        return result;  //To change body of implemented methods use File | Settings | File Templates.
    }

    /**
     * Methode to get a JFreeChart object filled with the data to display.
     * This methode uses hibernate and is optimized for speed.
     * The same operation can be done using pure JDBC by uncommenting the following methode.
     * Attention : Using cmp2 for this operation will cause problems because it does to many
     * SQL queries and does not performe well. You will probably encounter Transaction
     * Rollback operations.
     *
     * @param ruleset   The ruleset
     * @param rule      The rule
     * @param attribute The attribute
     * @param begintime the begintime
     * @return The jfreechart object
     */
    public XYSeriesCollection getMonitorRuleChart(String ruleset, String rule, String attribute, String begintime) {
        Context initalContext = null;
        SessionFactory sf = null;
        try {
            initalContext = new InitialContext();
            sf = (SessionFactory) initalContext.lookup("hibernate");
        } catch (NamingException e) {
            e.printStackTrace();
        }

        if (sf != null) {
            Session session = sf.openSession();

            long begin = System.currentTimeMillis();
            logger.log(Level.FINEST, "Time Before query execution ...");
            // sql : select r2.timestamp as time, r2.probeName.probeIdentifier as pi, r2.attributeValue as av from org.objectweb.jimys.dbConnection.hibernate.beans.Result as r1, org.objectweb.jimys.dbConnection.hibernate.beans.ResultProbe as r2 where r1.rule = :rule and r1.ruleSet = :ruleset and r1 = r2.result and r2.timestamp > :timestamp and r2.probeName.attributeName = :attributename order by r2.probeName.probeIdentifier
            Query query = session.createQuery("select r2.timestamp as time, r2.probeName.probeIdentifier as pi, r2.attributeValue as av " +
                    "from org.objectweb.jimys.dbConnection.hibernate.beans.Result as r1, org.objectweb.jimys.dbConnection.hibernate.beans.ResultProbe as r2 " +
                    "where r1.rule = :rule and r1.ruleSet = :ruleset and r1 = r2.result " +
                    "and r2.timestamp > :timestamp and r2.probeName.attributeName = :attributename " +
                    "order by r2.probeName.probeIdentifier");

            query.setString("ruleset", ruleset);
            query.setString("rule", rule);
            query.setLong("timestamp", Long.parseLong(begintime));
            query.setString("attributename", attribute);

            List list = query.list();

            logger.log(Level.FINEST, "Time After query execution  .... " + (System.currentTimeMillis() - begin));
            XYSeriesCollection xyDSet = new XYSeriesCollection();
            String pi = null;
            XYSeries series = null;
            logger.log(Level.FINEST, "size : " + list.size());
            for (int i = 0; i < list.size(); i++) {
                Object[] objects = (Object[]) list.get(i);

                Long rTimestamp = (Long) objects[0];
                String rProbeIdentifier = (String) objects[1];
                String rValue = (String) objects[2];


                if ((pi == null) || !(pi.equals(rProbeIdentifier))) {
                    pi = rProbeIdentifier;
                    if (series != null) {
                        logger.log(Level.FINEST, "adding serie ...");
                        xyDSet.addSeries(series);
                    }
                    series = new XYSeries(pi);
                }

                series.add(rTimestamp.longValue(), Double.parseDouble(rValue));

            }
            // Adding the last series
            if (series != null) {
                xyDSet.addSeries(series);
            }

            session.close();
            return xyDSet;

        }
        return new XYSeriesCollection();
    }

    /*  PURE JDBC (Not good because hardcoded tables)
    public XYSeriesCollection getMonitorRuleChart(String ruleset, String rule, String attribute, String begintime) {


        Connection conn = null;
        PreparedStatement ps = null;
        try {
            Context initalContext = new InitialContext();

            DataSource datasource = (DataSource) initalContext.lookup("jdbc_1");

            conn = datasource.getConnection();
            conn.setAutoCommit(false);

            ps = conn.prepareStatement("select r2.timeStamp_ as time, r2.ResultProbeNameEJB_probeIdentifier_ as pi, r2.attributeValue_ as av " +
                    "from RESULTEJB_ as r1, RESULTPROBEEJB_ as r2 " +
                    "where r1.rule_ = ? and r1.ruleSet_ = ? and r1.JPK_ = r2.ResultEJB_JPK_ " +
                    "and r2.timeStamp_ < ? and r2.ResultProbeNameEJB_attributeName_ = ? " +
                    "order by r2.ResultProbeNameEJB_probeIdentifier_");

            ps.setString(1, rule);
            ps.setString(2, ruleset);
            ps.setLong(3, Long.parseLong(begintime));
            ps.setString(4, attribute);

            ResultSet rs = ps.executeQuery();

            XYSeriesCollection xyDSet = new XYSeriesCollection();
            String pi = null;
            while (rs.next()) {
                long rTimeStamp = rs.getLong("time");
                String rProbeIdentifier = rs.getString("pi");
                String rAttValue = rs.getString("av");

                XYSeries series = null;
                if ((pi == null) || !(pi.equals(rProbeIdentifier))) {
                    pi = rProbeIdentifier;
                    if (series != null) {
                        xyDSet.addSeries(series);
                    }
                    series = new XYSeries(pi);
                }

                series.add(rTimeStamp, Double.parseDouble(rAttValue));
            }
            rs.close();
            ps.close();
            conn.close();

            return xyDSet;

        } catch (SQLException e ){
            e.printStackTrace();
        } catch (NamingException e) {
            e.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
        } finally {
            if (ps != null) {
                try {
                    ps.close();
                } catch (SQLException e) {
                    e.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
                }
            }
            if (conn != null) {
                try {
                    conn.close();
                } catch (SQLException e) {
                    e.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
                }
            }
        }

        return null;
    }
    */

    // this methode uses hibernate because it performs faster than cmp2
    // It is used to know if there are arlarms for a certain probe
    public Integer findResultsWithLevelCount(String level, String probeidentifier) {
        Context initalContext = null;
        SessionFactory sf = null;
        try {
            initalContext = new InitialContext();
            sf = (SessionFactory) initalContext.lookup("hibernate");
        } catch (NamingException e) {
            e.printStackTrace();
        }

        if (sf != null) {
            Session session = sf.openSession();

            Query query = session.createQuery("select count(r1) from  org.objectweb.jimys.dbConnection.hibernate.beans.Result as r1 where r1.level = :Level" +
                    " and r1.resultProbes.probeName.probeIdentifier like :ProbeIdentifier");
            query.setParameter("Level", level);
            query.setParameter("ProbeIdentifier", probeidentifier + "%");

            List l = query.list();
            Integer count = new Integer(-1);
            if (l.size() == 1) {
                count = (Integer) l.get(0);
            }
            session.close();
            return count;
        }
        return null;  //To change body of implemented methods use File | Settings | File Templates.
    }

    public ResultValueObject loadProbes(ResultValueObject rvo) {
        Context initalContext = null;
        try {
            initalContext = new InitialContext();
            LocalResultHome lrh = (LocalResultHome) initalContext.lookup("ResultEJBHome_L");

            LocalResult lr = lrh.findByPrimaryKey(Integer.valueOf(rvo.getId()));
            Collection lrp = lr.getResult1();
            Collection rp = new ArrayList();
            for (Iterator iterator = lrp.iterator(); iterator.hasNext();) {
                LocalResultProbe localResultProbe = (LocalResultProbe) iterator.next();
                ResultProbeValueObject rpvo = new ResultProbeValueObject();
                rpvo.setAttributeName(localResultProbe.getResultProbeName().getAttributeName());
                rpvo.setAttributeValue(localResultProbe.getAttributeValue());
                rpvo.setId(localResultProbe.getId());
                rpvo.setProbeIdentifier(localResultProbe.getResultProbeName().getProbeIdentifier());
                rpvo.setTimestamp(localResultProbe.getTimestamp());
                rp.add(rpvo);
            }
            rvo.setResultProbes(rp);
        } catch (NamingException e) {
            e.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
        } catch (FinderException e) {
            e.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
        }
        return rvo;  //To change body of implemented methods use File | Settings | File Templates.
    }

    public Collection findProbeWithStatus(String query, String level) {
        Collection col = new ArrayList();
        try {
            Context initalContext = new InitialContext();

            LocalResultProbeHome lrph = (LocalResultProbeHome) initalContext.lookup("ResultProbeEJBHome_L");
            Collection probes = lrph.findAllProbesWithStatus(query + "%", level);
            for (Iterator iterator = probes.iterator(); iterator.hasNext();) {
                LocalResultProbe localResultProbe = (LocalResultProbe) iterator.next();
                ResultProbeValueObject rpvo = new ResultProbeValueObject();
                rpvo.setAttributeName(localResultProbe.getResultProbeName().getAttributeName());
                rpvo.setAttributeValue(localResultProbe.getAttributeValue());
                rpvo.setId(localResultProbe.getId());
                rpvo.setProbeIdentifier(localResultProbe.getResultProbeName().getProbeIdentifier());
                rpvo.setTimestamp(localResultProbe.getTimestamp());
                col.add(rpvo);
            }


        } catch (NamingException e) {
            e.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
        } catch (FinderException e) {
            e.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
        }
        return col;
    }

    public void cancelAllAlarms(String level) {
        Context initalContext = null;
        SessionFactory sf = null;
        try {
            initalContext = new InitialContext();
            sf = (SessionFactory) initalContext.lookup("hibernate");
        } catch (NamingException e) {
            e.printStackTrace();
        }

        if (sf != null) {
            Session session = sf.openSession();
            Transaction tx = session.beginTransaction();

            String hqlUpdate = "update org.objectweb.jimys.dbConnection.hibernate.beans.Result set level = :newLevel where level = :oldLevel";
            int updatedEntities = session.createQuery(hqlUpdate).setString("newLevel", LevelHelper.CANCEL).setString("oldLevel", level).executeUpdate();
            tx.commit();
            session.close();
        }
    }

}


