/**
 * Copyright (C) 2005 - Bull S.A.
 *
 * Jimys - J2EE is monitoring your system
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id:  $
 * --------------------------------------------------------------------------
 */
package org.objectweb.jimys.dbConnection.status.entity.cluster;

import javax.ejb.CreateException;
import javax.ejb.EJBLocalHome;
import javax.ejb.FinderException;
import java.util.Collection;

/**
 *
 * LocalClusterStatusHome
 *
 * @author Georges Goebel, Marc Calvisi
 */
public interface LocalClusterStatusHome extends EJBLocalHome {

    /**
     * EJB find PK : The PK is a java.lang.Integer and created by the container
     * @param key The PK
     * @return The LocalClusterStatus related to the PK
     * @throws FinderException
     */
    public LocalClusterStatus findByPrimaryKey(Object key) throws FinderException;

    /**
     * Find the clusterstatus of a given cluster name and timestamp
     * Nasty hack inorder to find the last (max) clusterstatus of a given cluster
     * @param name The cluster name
     * @param timestamp The timestamp
     * @return The clusterstatus
     * @throws FinderException
     */
    public LocalClusterStatus findClusterStatus(String name, String timestamp) throws FinderException;

    /**
     * Select the maximum time of a Clusterstatus for a cluster name
     * @param name the cluster name
     * @return The clusterstatus
     * @throws FinderException
     */
    public String selectMaxTimeStamp(String name) throws FinderException;

    /**
     * Select all the different clusternames
     * @return A Collection of all the different clusternames (string)
     * @throws FinderException
     */
    public Collection selectDistinctClusterName() throws FinderException;

    /**
     * EJB create : creates a new clusterstatus with a PK created by the conatiner
     * @return
     * @throws CreateException
     */
    public LocalClusterStatus create() throws CreateException;
}
