/**
* Copyright (C) 2005 - Bull S.A.
*
* Jimys - J2EE is monitoring your system
*
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2.1 of the License, or any later version.
*
* This library is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
* Lesser General Public License for more details.
*
* You should have received a copy of the GNU Lesser General Public
* License along with this library; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
* USA
*
* --------------------------------------------------------------------------
* $Id:  $
* --------------------------------------------------------------------------
*/


package org.objectweb.jimys.commons.cartography;

import org.objectweb.jimys.commons.probe.AbstractProbe;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Hashtable;
import java.util.Iterator;

/**
*
* Represents a cartography
*
* @author Georges Goebel, Marc Calvisi
*/
public class Cartography extends AbstractProbe {
    /**
     * Constructor
     * @param timeStamp timestamp of the probe
     * @param identifier identifier of the probe
     * @param attributes all the attributes if this probe
     */
    public Cartography(long timeStamp, String identifier, Hashtable attributes) {
        super(timeStamp, identifier, attributes);
    }

    /**
     * Returns an attribute of this probe
     * @param key attributeName
     * @return attributeValue
     */
    public Object getAttributeByKey(String key) {
        Collection value = (Collection) this.attributes.get(key);
        if (value != null) {
            return value;
        }
        return new ArrayList();
    }

    /**
     * Returns the value of a specific component
     * @param componentGroup the group the component belongs to : Controller, DiskPartition ...
     * @param componentName the name of the component
     * @param attributeName the attribute of the component to look at
     * @return the attribute value or "not_found" if the attribute is not found
     */
    public String getComponentValue(String componentGroup, String componentName, String attributeName) {
        Collection col = (Collection) getAttributeByKey(componentGroup);
        if (col != null) {
            Iterator it = col.iterator();
            while (it.hasNext()) {
                AbstractProbe ap = (AbstractProbe) it.next();
                if (ap.getIdentifier().indexOf(componentName) > -1) {
                    String value = (String) ap.getAttributeByKey(attributeName);
                    if (value != null) {
                        return value;
                    }
                }
            }
        }
        return "not_found";
    }
}
