/**
* Copyright (C) 2005 - Bull S.A.
*
* Jimys - J2EE is monitoring your system
*
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2.1 of the License, or any later version.
*
* This library is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
* Lesser General Public License for more details.
*
* You should have received a copy of the GNU Lesser General Public
* License along with this library; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
* USA
*
* --------------------------------------------------------------------------
* $Id:  $
* --------------------------------------------------------------------------
*/

/**
*
*
*
* @author Georges Goebel, Marc Calvisi
*/
package org.objectweb.jimys.commons.clusterstatus;

import java.util.Hashtable;
/**
*
*  ClusterStatus
*
* @author Georges Goebel, Marc Calvisi
*/
public class ClusterStatus extends org.objectweb.jimys.commons.clusterstatus.AbstractStatus {
    /**
     * contains all the hosts being part of the cluster
     */
    private Hashtable hosts = null;

    /**
     * Constructor
     * @param identifier the name of the cluster
     */
    public ClusterStatus(String identifier) {
        super(identifier, "", -1);
        hosts = new Hashtable();
    }

    /**
     * Constructor
     * @param identifier name of the cluster
     * @param status actual status
     * @param timestamp actual timestamp
     */
    public ClusterStatus(String identifier, String status, long timestamp) {
        super(identifier, status, timestamp);
        hosts = new Hashtable();
    }

    /**
     * returns the name of the cluster ans its statis
     * @return name and status
     */
    public String getNameAndStatus() {
        return "Cluster " + getIdentifier() + " : " + getHtmlStatus();
    }

    /**
     * returns the status of a specific host
     * @param hostname the name of the host being to look at
     * @return the status of that host
     */
    public HostStatus getHostStatus(String hostname) {
        if (this.hosts.containsKey(hostname)) {
            return (HostStatus) this.hosts.get(hostname);
        }
        return null;
    }

    /**
     * modify the status of a host of the cluster
     * @param ds the new status of that host
     */
    public void modifyHostStatus(HostStatus ds) {
        this.hosts.put(ds.getIdentifier(), ds);
    }

    /**
     * adds a new host to the clusterstatus
     * @param ds the new host to add
     */
    public void addHostStatus(HostStatus ds) {
        this.hosts.put(ds.getIdentifier(), ds);
    }

    /**
     * get all the host being part of the cluster
     * @return the hashtable with all the hosts
     */
    public Hashtable getHosts() {
        return this.hosts;
    }

    /**
     * set an new table with host
     * @param hosts the new hosts table
     */
    public void setHosts(Hashtable hosts) {
        this.hosts = hosts;
    }

    /**
     * removes a host from the clusterstatus
     * @param hostname the host to remove
     */
    public void removeHost(String hostname){
        this.hosts.remove(hostname);
    }

    /**
     * returns the cluster name and its status
     * @return cluster name and its colored status
     */
    public String toString() {
        return "Cluster " + getIdentifier() + " : " + getHtmlStatus();
    }

}
