/**
* Copyright (C) 2005 - Bull S.A.
*
* Jimys - J2EE is monitoring your system
*
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2.1 of the License, or any later version.
*
* This library is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
* Lesser General Public License for more details.
*
* You should have received a copy of the GNU Lesser General Public
* License along with this library; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
* USA
*
* --------------------------------------------------------------------------
* $Id:  $
* --------------------------------------------------------------------------
*/


package org.objectweb.jimys.commons.clusterstatus;

import java.util.Hashtable;
/**
*
*  DomainStatus
*
* @author Georges Goebel, Marc Calvisi
*/
public class DomainStatus extends org.objectweb.jimys.commons.clusterstatus.AbstractStatus {
    /**
     * contains all instances of this domain
     */
    private Hashtable instances = null;

    /**
     * Constructor
     * @param identifier identifier of the domain
     * @param status status of the domain
     * @param timestamp timestamp of the domain
     */
    public DomainStatus(String identifier, String status, long timestamp) {
        super(identifier, status, timestamp);
        this.instances = new Hashtable();
    }

    /**
     * returns an instance status to check
     * @param identifier the name of the instance status
     * @return the instance status or "null" if not found
     */
    public InstanceStatus getInstanceStatus(String identifier) {
        if (this.instances.containsKey(identifier)) {
            return (InstanceStatus) this.instances.get(identifier);
        }
        return null;
    }

    /**
     * changes the status of an instance
     * @param is the new instance status
     */
    public void modifyInstanceStatus(InstanceStatus is) {
        this.instances.put(is.getIdentifier(), is);
    }

    /**
     * add a mew instance to the domain
     * @param is the new instance
     */
    public void addInstanceStatus(InstanceStatus is) {
        this.instances.put(is.getIdentifier(), is);
    }

    /**
     * get all instances of this domain
     * @return all instances
     */
    public Hashtable getInstances() {
        return instances;
    }

    /**
     * set the value of the instances to a new value
     * @param instances table with all instances
     */
    public void setInstances(Hashtable instances) {
        this.instances = instances;
    }

    /**
     * get the name and the status of this domain
     * @return the name and the colored status of the domain
     */
    public String getNameAndStatus() {
        return getIdentifier().substring(getIdentifier().lastIndexOf(";") + 1) + " : " + getHtmlStatus();
    }

}
