/**
* Copyright (C) 2005 - Bull S.A.
*
* Jimys - J2EE is monitoring your system
*
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2.1 of the License, or any later version.
*
* This library is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
* Lesser General Public License for more details.
*
* You should have received a copy of the GNU Lesser General Public
* License along with this library; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
* USA
*
* --------------------------------------------------------------------------
* $Id:  $
* --------------------------------------------------------------------------
*/


package org.objectweb.jimys.commons.clusterstatus;

import java.util.Hashtable;
/**
*
* HostStatus
*
* @author Georges Goebel, Marc Calvisi
*/
public class HostStatus extends org.objectweb.jimys.commons.clusterstatus.AbstractStatus {
    /**
     * contains all domains on this host
     */
    private Hashtable domains = null;
    /**
     * the reference to the pump on this host
     */
    private PumpStatus pumpStatus = null;

    /**
     * constructor
     * @param identifier hostname
     * @param status status of the host
     * @param timestamp timestamp of the status
     */
    public HostStatus(String identifier, String status, long timestamp) {
        super(identifier, status, timestamp);
        domains = new Hashtable();
    }

    /**
     * get a domain of this host
     * @param identifier the name of the domain
     * @return the domain status and "null" if not found
     */
    public DomainStatus getDomainStatus(String identifier) {
        if (this.domains.containsKey(identifier)) {
            return (DomainStatus) this.domains.get(identifier);
        }
        return null;
    }

    /**
     * modifies the value of a domain status
     * @param ds the new value of a domain status
     */
    public void modifyDomainStatus(DomainStatus ds) {
        this.domains.put(ds.getIdentifier(), ds);
    }

    /**
     * add a new domain to this host status
      * @param ds the new domainstatus
     */
    public void addDomainStatus(DomainStatus ds) {
        this.domains.put(ds.getIdentifier(), ds);
    }

    /**
     * gets the status of the pump on this hist
     * @return the pump status
     */
    public PumpStatus getPumpStatus() {
        return this.pumpStatus;
    }

    /**
     * sets the pump status of this host to a new one
     * @param status the new pump status
     */
    public void setPumpStatus(PumpStatus status) {
        this.pumpStatus = status;
    }

    /**
     * returns a hashtable with all the domains of this host
     * @return hashtable with all domains
     */
    public Hashtable getDomains() {
        return domains;
    }

    /**
     * set the value of the domains to a new one
     * @param domains the new domains
     */
    public void setDomains(Hashtable domains) {
        this.domains = domains;
    }

    /**
     * get the string value of the identifier
     * @return the value of the identifier
     */
    public String toString() {
        return "toString -> " + getIdentifier();
    }

    /**
     * get the name and status of this host
     * @return the name and colored status
     */
    public String getNameAndStatus() {
        return getIdentifier() + " : " + getHtmlStatus();
    }

}
