/**
* Copyright (C) 2005 - Bull S.A.
*
* Jimys - J2EE is monitoring your system
*
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2.1 of the License, or any later version.
*
* This library is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
* Lesser General Public License for more details.
*
* You should have received a copy of the GNU Lesser General Public
* License along with this library; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
* USA
*
* --------------------------------------------------------------------------
* $Id:  $
* --------------------------------------------------------------------------
*/


package org.objectweb.jimys.commons.clusterstatus;

import java.util.Hashtable;
/**
*
* InstanceStatus
*
* @author Georges Goebel, Marc Calvisi
*/
public class InstanceStatus extends org.objectweb.jimys.commons.clusterstatus.AbstractStatus {
    /**
     * contains all the probes monitoring this instance
     */
    private Hashtable probes = null;

    /**
     * constructor
     * @param identifier identifier of the instance
     * @param status actual status of the instance
     * @param timestamp timestamp of the status
     */
    public InstanceStatus(String identifier, String status, long timestamp) {
        super(identifier, status, timestamp);
        this.probes = new Hashtable();
    }

    /**
     * get the status of a probe that belongs to this instance
     * @param identifier the identifier of the probe
     * @return the probe status
     */
    public ProbeStatus getProbeStatus(String identifier) {
        return (ProbeStatus) this.probes.get(identifier);
    }

    /**
     * modify the status of a probe
     * @param ps the new probe status
     */
    public void modifyProbeStatus(ProbeStatus ps) {
        this.probes.put(ps.getIdentifier(), ps);
    }

    /**
     * add a probe status to this instance
     * @param ps the new probe status
     */
    public void addProbeStatus(ProbeStatus ps) {
        this.probes.put(ps.getIdentifier(), ps);
    }

    /**
     * get all the probes of this instance
     * @return the probes
     */
    public Hashtable getProbes() {
        return probes;
    }

    /**
     * set the hashtable of the probes to a new value
     * @param probes the new hashtable
     */
    public void setProbes(Hashtable probes) {
        this.probes = probes;
    }

    /**
     * gets the identifier of this instance
     * @return the identifier
     */
    public String toString() {
        return getIdentifier();
    }

    /**
     * returns the name and the actual status of this instance
     * @return the name and colored status of the instance
     */
    public String getNameAndStatus() {
        return getIdentifier().substring(getIdentifier().lastIndexOf(";") + 1) + " : " + getHtmlStatus();
    }


}
