/**
* Copyright (C) 2005 - Bull S.A.
*
* Jimys - J2EE is monitoring your system
*
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2.1 of the License, or any later version.
*
* This library is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
* Lesser General Public License for more details.
*
* You should have received a copy of the GNU Lesser General Public
* License along with this library; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
* USA
*
* --------------------------------------------------------------------------
* $Id:  $
* --------------------------------------------------------------------------
*/


package org.objectweb.jimys.commons.clusterstatus;

import java.util.Hashtable;
 /**
*
* PumpStatus
*
* @author Georges Goebel, Marc Calvisi
*/
public class PumpStatus extends org.objectweb.jimys.commons.clusterstatus.AbstractStatus {
    /**
     * the probes that belong to the pump on a host
     */
    private Hashtable probes = null;

    /**
     * constructor
     * @param identifier the identifier of the pump
     * @param status the status of the pump
     * @param timestamp the timestamp of the pump
     */
    public PumpStatus(String identifier, String status, long timestamp) {
        super(identifier, status, timestamp);
        this.probes = new Hashtable();
    }

    /**
     * get the status of a probe
     * @param identifier the name of the probe
     * @return the status of the probe
     */
    public ProbeStatus getProbeStatus(String identifier) {
        if (this.probes.containsKey(identifier)) {
            return (ProbeStatus) this.probes.get(identifier);
        }
        return null;
    }

    /**
     * modify the status of a probe
     * @param ps the new probestatus
     */
    public void modifyProbeStatus(ProbeStatus ps) {
        this.probes.put(ps.getIdentifier(), ps);
    }

    /**
     * add a probe status
     * @param ps a new probe status
     */
    public void addProbeStatus(ProbeStatus ps) {
        this.probes.put(ps.getIdentifier(), ps);
    }

    /**
     * get all the probe status of this pump
     * @return the probe status
     */
    public Hashtable getProbes() {
        return probes;
    }

    /**
     * set the probe status of this pump to new values
     * @param probes the new probe status
     */
    public void setProbes(Hashtable probes) {
        this.probes = probes;
    }

    /**
     * get the name of the pump
     * @return the identifier of the pump
     */
    public String toString() {
        return getIdentifier();
    }

    /**
     * the name and status of the pump
     * @return name and colored status of the pump
     */
    public String getNameAndStatus() {
        return "pump" + " : " + getHtmlStatus();
    }
}
