/**
* Copyright (C) 2005 - Bull S.A.
*
* Jimys - J2EE is monitoring your system
*
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2.1 of the License, or any later version.
*
* This library is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
* Lesser General Public License for more details.
*
* You should have received a copy of the GNU Lesser General Public
* License along with this library; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
* USA
*
* --------------------------------------------------------------------------
* $Id:  $
* --------------------------------------------------------------------------
*/


package org.objectweb.jimys.commons.clusterstatus;

import java.io.Serializable;
import java.util.StringTokenizer;
/**
*
* Status
*
* @author Georges Goebel, Marc Calvisi
*/
public class Status implements Serializable {
    /**
     * the entire identifier of the status
     */
    private String identifier = "";
    /**
     * the host name of the status
     */
    private String identifierHostname = "";
    /**
     * the domain name of the status
     */
    private String identifierDomain = "";
    /**
     * the instance name of the status
     */
    private String identifierInstance = "";
    /**
     * the probe name of the status
     */
    private String identifierProbe = "";
    /**
     * the status value
     */
    private String status = "";
    /**
     * the timestamp of the status
     */
    private long timestamp = -1;

    /**
     * Constructor
     * @param identifier the entire identifier of the status
     * @param status the value of the status
     * @param timestamp the timestamp of the status
     */
    public Status(String identifier, String status, long timestamp) {
        this.identifier = identifier;
        this.status = status;
        this.timestamp = timestamp;
        this.splitIdentifier(identifier);
    }

    /**
     * splits the identifier into the different identifier's
     * @param identifier the entire identifier
     */
    private void splitIdentifier(String identifier) {
        StringTokenizer st = new StringTokenizer(identifier, ";");
        this.identifierHostname = st.nextToken();
        this.identifierDomain = st.nextToken();
        this.identifierInstance = st.nextToken();
        st.nextToken();
        this.identifierProbe = st.nextToken();
    }

    /**
     * get the identifier containing the host name
     * @return the host name
     */
    public String getIdentifierHostname() {
        return this.identifierHostname;
    }

    /**
     * get the identifier containing the domain name
     * @return the domain name
     */
    public String getIdentifierDomain() {
        return this.identifierDomain;
    }

    /**
     * get the identifier containing the instance name
     * @return the instance name
     */
    public String getIdentifierInstance() {
        return this.identifierInstance;
    }


    /**
     * get the identifier containing the probe name
     * @return the probe name
     */
    public String getIdentifierProbe() {
        return this.identifierProbe;
    }

    /**
     * get the entiree identifier
     * @return the identifier
     */
    public String getIdentifier() {
        return this.identifier;
    }

    /**
     * get the status
     * @return the status
     */
    public String getStatus() {
        return this.status;
    }

    /**
     * the timestamp
     * @return the timestamp
     */
    public long getTimestamp() {
        return this.timestamp;
    }

}
