/**
* Copyright (C) 2005 - Bull S.A.
*
* Jimys - J2EE is monitoring your system
*
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2.1 of the License, or any later version.
*
* This library is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
* Lesser General Public License for more details.
*
* You should have received a copy of the GNU Lesser General Public
* License along with this library; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
* USA
*
* --------------------------------------------------------------------------
* $Id:  $
* --------------------------------------------------------------------------
*/

package org.objectweb.jimys.commons.configuration;

import java.io.Serializable;
import java.util.Hashtable;

/**
*
* Configuration
*
* @author Georges Goebel, Marc Calvisi
*/
public class Configuration implements Serializable {
    /**
     * the fireRuleAfter value of the engine configuration
     */
    private String fireRuleAfter = "";
    /**
     * the timeinterval value of the engine configuration
     */
    private String timeInterval = "";
    /**
     * the other values of the engine configuration that cannot be changed
     */
    private Hashtable values = null;

    /**
     * constructor
     */
    public Configuration() {
        values = new Hashtable();
    }

    /**
     * gets the value of the fireRuleAfter config
     * @return the value of the config
     */
    public String getFireRuleAfter() {
        return fireRuleAfter;
    }

    /**
     * set the fireRuleAfter to a new value
     * @param fireRuleAfter the new value
     */
    public void setFireRuleAfter(String fireRuleAfter) {
        this.fireRuleAfter = fireRuleAfter;
    }

    /**
     * gets all the non modifiable values of the configuration
     * @return all the values
     */
    public Hashtable getValues() {
        return this.values;
    }

    /**
     * sets the value to new ones
     * @param values the new values
     */
    public void setValues(Hashtable values) {
        this.values = values;
    }

    /**
     * adds a configuration value to the values hashtable
     * @param key the key of the value
     * @param value the value
     */
    public void addValue(String key, String value) {
        this.values.put(key, value);
    }
    /**
     * returns the actual timeinterval between 2 fireAll rules actions
     * @return the timeinterval
     */
    public String getTimeInterval() {
        return timeInterval;
    }
    /**
     * sets the timeinterval to a new value
     * @param timeInterval the new value
     */
    public void setTimeInterval(String timeInterval) {
        this.timeInterval = timeInterval;
    }
}