/**
* Copyright (C) 2005 - Bull S.A.
*
* Jimys - J2EE is monitoring your system
*
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2.1 of the License, or any later version.
*
* This library is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
* Lesser General Public License for more details.
*
* You should have received a copy of the GNU Lesser General Public
* License along with this library; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
* USA
*
* --------------------------------------------------------------------------
* $Id:  $
* --------------------------------------------------------------------------
*/


package org.objectweb.jimys.commons.configuration;

import java.io.Serializable;

/**
*
* RuleActivator
*
* @author Georges Goebel, Marc Calvisi
*/
public class RuleActivator implements Serializable {
    /**
     * the name of the rule to activate or disactivate
     */
    private String rulename = "";
    /**
     * the name of the ruleset the rule belongs to
     */
    private String ruleset = "";
    /**
     * if this rule is active or inactive
     */
    boolean active = true;

    /**
     * constructor
     * @param rulename the rule name
     * @param ruleset  the ruleset name
     */
    public RuleActivator(String rulename, String ruleset) {
        this.rulename = rulename;
        this.ruleset = ruleset;
    }

    /**
     * constructor
     * @param rulename the rule name
     * @param ruleset the ruleset name
     * @param active true if active otherwise false
     */
    public RuleActivator(String rulename, String ruleset, boolean active) {
        this.rulename = rulename;
        this.ruleset = ruleset;
        this.active = active;
    }


    /**
     * get the name of the rule
     * @return rulename
     */
    public String getRulename() {
        return rulename;
    }

    /**
     * set the name of the rule to a new value
     * @param rulename the new value for the name
     */
    public void setRulename(String rulename) {
        this.rulename = rulename;
    }

    /**
     * get the name of the ruleset
     * @return ruleset name
     */
    public String getRuleset() {
        return ruleset;
    }

    /**
     * set the name of the ruleset to a new one
     * @param ruleset the new value of the ruleset name
     */
    public void setRuleset(String ruleset) {
        this.ruleset = ruleset;
    }

    /**
     * changes the state of the rule
     * @param active true if active otherwise false
     */
    public void setActive(boolean active) {
        this.active = active;
    }

    /**
     * check wether the rule is active or inactive
     * @return the status of the rule
     */
    public boolean isActive() {
        return active;
    }

    /**
     * get the value of status of the rule
     * @return the value of the rule
     */
    public boolean getActive() {
        return active;
    }

}
