/**
* Copyright (C) 2005 - Bull S.A.
*
* Jimys - J2EE is monitoring your system
*
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2.1 of the License, or any later version.
*
* This library is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
* Lesser General Public License for more details.
*
* You should have received a copy of the GNU Lesser General Public
* License along with this library; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
* USA
*
* --------------------------------------------------------------------------
* $Id:  $
* --------------------------------------------------------------------------
*/


package org.objectweb.jimys.commons.probe;

import java.io.Serializable;
import java.util.Hashtable;
import java.util.StringTokenizer;

/**
*
* The abstract class to implement by each probe
*
* @author Georges Goebel, Marc Calvisi
*/
public abstract class AbstractProbe implements Serializable {
    /**
     * the timestamp
     */
    private long timeStamp = 0;
    /**
     * the entire identifier of the probe
     */
    private String identifier = "";
    /**
     * all the attributes of the probe
     */
    protected Hashtable attributes = null;
    /**
     * the host name of the probe
     */
    private String identifierHostname = "";
    /**
     * the domain name of the probe
     */
    private String identifierDomain = "";
    /**
     * the instance name of the probe
     */
    private String identifierInstance = "";
    /**
     * the probe name of the probe
     */
    private String identifierProbe = "";
    /**
     * a subidentifier of the probe
     */
    private String subIdentifierProbe = "";

    /**
     * constructor
     * @param timeStamp the timestamp
     * @param identifier the identifier of the probe
     */
    public AbstractProbe(long timeStamp, String identifier) {
        this.timeStamp = timeStamp;
        this.identifier = identifier;
        this.splitIdentifier(identifier);
    }

    /**
     * constructor
     * @param timeStamp the timestamp
     * @param identifier the identifier of the probe
     * @param attributes all the attributes of the probe
     */
    public AbstractProbe(long timeStamp, String identifier, Hashtable attributes) {
        this.timeStamp = timeStamp;
        this.identifier = identifier;
        this.attributes = attributes;
        this.splitIdentifier(identifier);
    }

    /**
     * Constructor
     */
    public AbstractProbe() {
    }

    /**
     * splits the identifier into the different names
     * @param identifier the entire identifier
     */
    private void splitIdentifier(String identifier) {
        StringTokenizer st = new StringTokenizer(identifier, ";");
        this.identifierHostname = st.nextToken();
        this.identifierDomain = st.nextToken();
        this.identifierInstance = st.nextToken();
        String probe = st.nextToken();
        this.identifierProbe = st.nextToken();
        if (st.hasMoreTokens()){
            this.subIdentifierProbe = st.nextToken();
        }
    }

    /**
     * set the attributes of the probe
     * @param attributes the new attributes
     */
    public void setAttributes(Hashtable attributes) {
        this.attributes = attributes;
    }

    /**
     * get the attributes of the probe
     * @return all attributes
     */
    public Hashtable getAttributes() {
        return this.attributes;
    }

    /**
     * select an attribute by his key
     * @param key the key of an attribute
     * @return the value of that attribute
     */
    public Object getAttributeByKey(String key) {
        Integer value = (Integer) this.attributes.get(key);
        if (value != null) {
            return value;
        }
        return new Integer(-1);
    }

    /**
     * the entire identifier of the probe
     * @return the identifier
     */
    public String getIdentifier() {
        return identifier;
    }

    /**
     * the timestamp of the probe
     * @return timestamp
     */
    public long getTimeStamp() {
        return timeStamp;
    }

    /**
     * the host name of the probe
     * @return host name
     */
    public String getIdentifierHostname() {
        return this.identifierHostname;
    }

    /**
     * the domain name of the probe
     * @return domain name
     */
    public String getIdentifierDomain() {
        return this.identifierDomain;
    }

    /**
     * the instance name of the probe
     * @return instance name
     */
    public String getIdentifierInstance() {
        return this.identifierInstance;
    }

    /**
     * the probe name of the probe
     * @return the probe name
     */
    public String getIdentifierProbe() {
        return this.identifierProbe;
    }

    /**
     * the sub identifier of the probe
     * @return sub identifier
     */
    public String getSubIdentifierProbe() {
        return subIdentifierProbe;
    }

    /**
     * set the sub identifier of a probe
     * @param subIdentifierProbe the new value
     */
    public void setSubIdentifierProbe(String subIdentifierProbe) {
        this.subIdentifierProbe = subIdentifierProbe;
    }
}
