/**
* Copyright (C) 2005 - Bull S.A.
*
* Jimys - J2EE is monitoring your system
*
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2.1 of the License, or any later version.
*
* This library is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
* Lesser General Public License for more details.
*
* You should have received a copy of the GNU Lesser General Public
* License along with this library; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
* USA
*
* --------------------------------------------------------------------------
* $Id:  $
* --------------------------------------------------------------------------
*/


package org.objectweb.jimys.commons.rule;


import java.io.Serializable;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;

/**
*
* RuleHelper
*
* @author Georges Goebel, Marc Calvisi
*/
public class RuleHelper implements Serializable {
    /**
     * the name of the rule
     */
    private String rulename = "";
    /**
     * an array with all the conditions of the rule
     */
    private Collection conditions = new ArrayList();
    /**
     * the consequence of the rule
     */
    private String consequence = "";

    /**
     * constructor
     */
    public RuleHelper() {
    }

    /***
     * get the name of this rule
     * @return the rulename
     */
    public String getRulename() {
        return rulename;
    }

    /**
     * set the name of the rule
     * @param rulename the new name
     */
    public void setRulename(String rulename) {
        this.rulename = rulename;
    }

    /**
     * get all the conditions of the rule
     * @return all conditions
     */
    public Collection getConditions() {
        return conditions;
    }

    /**
     * set the conditions to a new value
     * @param conditions the new value
     */
    public void setConditions(Collection conditions) {
        this.conditions = conditions;
    }

    /**
     * add a condition to the conditions of this rule
     * @param condition a new conditions
     */
    public void addCondition(String condition) {
        this.conditions.add(condition);
    }

    /**
     * get the consequence of this rule
     * @return the consequence
     */
    public String getConsequence() {
        return consequence;
    }

    /**
     * set the consequence to a new value
     * @param consequence the new value
     */
    public void setConsequence(String consequence) {
        this.consequence = consequence;
    }

    /**
     * returns a sting containing the rulename, the conditions and the consequence
     * @return a formatted string
     */
    public String toString() {
        String output = this.rulename + "\n";
        Iterator it = this.conditions.iterator();
        while (it.hasNext()) {
            output += it.next() + "\n";
        }
        output += this.consequence;
        return output;
    }

    /**
     * get a formatted strng with all informations of the rule
     * @return the formatted string
     */
    public String getRule() {
        return this.toString();
    }
}
