/**
 * Copyright (C) 2005 - Bull S.A.
 *
 * Jimys - J2EE is monitoring your system
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id:  $
 * --------------------------------------------------------------------------
 */
package org.objectweb.jimys.ads.mediator.communication;

import org.objectweb.jimys.commons.probe.AbstractProbe;

import javax.naming.Context;
import javax.naming.InitialContext;
import javax.naming.NamingException;
import javax.jms.*;
import java.util.Properties;
import java.io.IOException;

/**
 * Connector Mediator to rule engine
 *
 * @author Georges Goebel, Marc Calvisi
 */
public class EngineConnector {
    /**
     * the queue to use to send the message
     */
    private static Queue queue = null;
    /**
     * the queue connection factory
     */
    private static QueueConnectionFactory qcf = null;

    /**
     * constructor
     */
    public EngineConnector(){
        if (qcf == null) {
            Properties props = new Properties();
            try {
                props.load(Thread.currentThread().getContextClassLoader().getResourceAsStream("mediator.properties"));
            } catch (IOException e1) {
                e1.printStackTrace();
            }
            String factory = "";
            String url = "";
            if (props.getProperty("engine.java.naming.factory.initial") != null) {
                factory = props.getProperty("engine.java.naming.factory.initial");
            }
            if (props.getProperty("engine.java.naming.provider.url") != null) {
                url = props.getProperty("engine.java.naming.provider.url");
            }
            Properties properties = new Properties();
            properties.put("engine.java.naming.factory.initial", factory);
            properties.put("engine.java.naming.provider.url", url);
            try {
                Context ictx = new InitialContext(properties);
                String queuename = "";
                if (props.getProperty("engine.mdb.queue.name") != null) {
                    queuename = props.getProperty("engine.mdb.queue.name");
                }
                queue = (Queue) ictx.lookup(queuename);
                String queuefactory = "";
                if (props.getProperty("engine.mdb.queue.connection.factory") != null) {
                    queuefactory = props.getProperty("engine.mdb.queue.connection.factory");
                }
                qcf = (QueueConnectionFactory) ictx.lookup(queuefactory);
                ictx.close();
            } catch (NamingException e) {
                e.printStackTrace();
            }
        }
    }

    /**
     * send a probe to the rule engine
     * @param ap the probe to send
     */
    public void sendProbeToRuleEngine(AbstractProbe ap) {
        try {
            QueueConnection qc = qcf.createQueueConnection();
            QueueSession qs = qc.createQueueSession(true, 0);
            QueueSender qsend = qs.createSender(queue);
            ObjectMessage msg = qs.createObjectMessage();
            msg.setObject(ap);
            qsend.send(msg);
            qsend.close();
            qsend = null;
            qs.commit();
            qs.close();
            qs = null;
            qc.close();
            qc = null;
        } catch (JMSException e) {
            e.printStackTrace();
        }
    }
}
