/**
 * Copyright (C) 2005 - Bull S.A.
 *
 * Jimys - J2EE is monitoring your system
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id:  $
 * --------------------------------------------------------------------------
 */
package org.objectweb.jimys.ads.mediator.communication;


import org.objectweb.jimys.commons.exceptions.ProblemException;

import javax.ejb.EJBException;
import javax.ejb.EJBLocalObject;
import java.util.Collection;

/**
 *  interface to the pumpConnector
 *  bean that connects to a pumps
 *
 * @author Georges Goebel, Marc Calvisi
 */
public interface PumpConnector extends EJBLocalObject {
    /**
     * connect to a pump on a host
     * @param host the hostname
     * @param port the port to connect to
     * @throws EJBException
     * @throws ProblemException
     */
    public void run(String host, String port) throws EJBException, ProblemException;

    /**
     * stop the connection to this pump
     * @throws EJBException
     */
    public void stopConnection() throws EJBException, ProblemException;

    /**
     * check the node
     * @return
     * @throws EJBException
     * @throws ProblemException
     */
    public String checkNode() throws EJBException, ProblemException;

    /**
     * get all the registered probes of this pump
     * @return all probes
     * @throws EJBException
     * @throws ProblemException
     */
    public Collection getRegisteredProbesFromPump() throws EJBException, ProblemException;

    /**
     * get the samplerate of a specific probe
     * @param probename the probe name
     * @return the sample rate
     * @throws EJBException
     * @throws ProblemException
     */
    public Long getProbeSampleRate(String probename) throws EJBException, ProblemException;

    /**
     * set the samplerate of a specific probe
     * @param probename the probe name
     * @param sampleRate the new sample rate
     * @throws EJBException
     * @throws ProblemException
     */
    public void setProbeSampleRate(String probename, Long sampleRate) throws EJBException, ProblemException;

}
