/**
 * Copyright (C) 2005 - Bull S.A.
 *
 * Jimys - J2EE is monitoring your system
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id:  $
 * --------------------------------------------------------------------------
 */
package org.objectweb.jimys.ads.mediator.communication;

import org.objectweb.jimys.commons.exceptions.ProblemException;

import javax.ejb.EJBException;
import javax.ejb.SessionBean;
import javax.ejb.SessionContext;
import javax.jms.*;
import javax.management.*;
import javax.management.remote.JMXConnector;
import javax.management.remote.JMXConnectorFactory;
import javax.management.remote.JMXServiceURL;
import javax.naming.Context;
import javax.naming.InitialContext;
import javax.naming.NamingException;
import java.io.IOException;
import java.io.InputStream;
import java.io.ByteArrayOutputStream;
import java.net.MalformedURLException;
import java.util.Collection;
import java.util.Properties;
import java.util.zip.Inflater;
import java.util.zip.DataFormatException;
 /**
 *   PumpConnectorBean
 *
 * @author Georges Goebel, Marc Calvisi
 */
public class PumpConnectorBean implements SessionBean {

    private MBeanServerConnection connection = null;

    private ObjectName delegateName = null;

    private NotificationListener listener = null;

    private static Properties props = new Properties();

    private String pumpname = "";

    private static Topic topic = null;

    private static TopicConnectionFactory topicConnFactory = null;
    private String jmxhost = "";
    public PumpConnectorBean() {
        if (topicConnFactory == null) {
            InputStream is = Thread.currentThread().getContextClassLoader().getResourceAsStream("mediator.properties");
            try {
                props.load(is);
            } catch (IOException e1) {
                e1.printStackTrace();
            }
            String factory = "";
            String url = "";
            if (props.getProperty("mediator.java.naming.factory.initial") != null) {
                factory = props.getProperty("mediator.java.naming.factory.initial");
            }

            if (props.getProperty("mediator.java.naming.provider.url") != null) {
                url = props.getProperty("mediator.java.naming.provider.url");
            }
            Properties properties = new Properties();
            properties.put("java.naming.factory.initial", factory);
            properties.put("java.naming.provider.url", url);
            try {
                Context ictx = new InitialContext(properties);
                String topicname = "";
                if (props.getProperty("mediator.mdb.topic.name") != null) {
                    topicname = props.getProperty("mediator.mdb.topic.name");
                }
                String topicFactory = "";
                if (props.getProperty("mediator.mdb.topic.connection.factory") != null) {
                    topicFactory = props.getProperty("mediator.mdb.topic.connection.factory");
                }
                topic = (Topic) ictx.lookup(topicname);
                topicConnFactory = (TopicConnectionFactory) ictx.lookup(topicFactory);
                ictx.close();
            } catch (NamingException e) {
                e.printStackTrace();
            }
        }
    }


    public void setSessionContext(SessionContext arg0) throws EJBException {
        System.out.println("PumpConnector : setSessionContext()");

    }


    public void ejbRemove() throws EJBException {
        System.out.println("PumpConnector : ejbRemove()");
    }

    public void ejbActivate() throws EJBException {
        System.out.println("PumpConnector : ejbActivate()");

    }

    public void ejbPassivate() throws EJBException {
        System.out.println("PumpConnector : ejbPassivate()");

    }

    public void ejbCreate() throws EJBException {
        System.out.println("PumpConnector : ejbCreate()");

    }

    public void run(String host, String port) throws EJBException, ProblemException {
        jmxhost = host;
         try {
            JMXServiceURL address = new JMXServiceURL("service:jmx:rmi://" + host + "/jndi/rmi://" + host + ":" + port + "/jmx");
            System.out.println("Address is : " + address);

            JMXConnector connector = JMXConnectorFactory.connect(address, null);

            connection = connector.getMBeanServerConnection();
            System.out.println("connection : " + connection);

            System.out.println("MBeans : " + connection.getMBeanCount().intValue());
             if (props.getProperty("mediator.connect.pump") != null) {
                pumpname = props.getProperty("mediator.connect.pump");
            }
            delegateName = ObjectName.getInstance(pumpname);
            System.out.println("MBean : " + delegateName);
           /* listener = new NotificationListener() {
                public void handleNotification(Notification arg0, Object arg1) {
                    if (arg0.getUserData() instanceof byte[]){
                        sendProbeToParser(decompress((byte[]) arg0.getUserData()));
                    }else{
                        sendProbeToParser((String)arg0.getUserData());
                    }
                }
            };    */
            listener = new PumpConnectorListener(this);
            connection.addNotificationListener(delegateName, listener, null, null);
            System.out.println("Done");
        } catch (MalformedURLException e) {
            e.printStackTrace();
            // System.out.println("MalformedURLException");
            throw new ProblemException("MalformedURLException");
            //e.printStackTrace();
        } catch (IOException e) {
            e.printStackTrace();
            //System.out.println("IOException");
            throw new ProblemException("IOException");
            // e.printStackTrace();
        } catch (MalformedObjectNameException e) {
            e.printStackTrace();
            //System.out.println("MalformedObjectNameException");
            throw new ProblemException("MalformedObjectNameException");
            //e.printStackTrace();
        } catch (InstanceNotFoundException e) {
            e.printStackTrace();
            //System.out.println("InstanceNotFoundException");
            throw new ProblemException("InstanceNotFoundException");
            // e.printStackTrace();
        }
    }

    public Collection getRegisteredProbesFromPump() throws EJBException, ProblemException {
        String attName = "";
        if (props.getProperty("mediator.attribute.registered.probes.name") != null) {
            attName = props.getProperty("mediator.attribute.registered.probes.name");
        }
        try {
            return (Collection) connection.getAttribute(ObjectName.getInstance(pumpname), attName);
        } catch (MBeanException e) {
            e.printStackTrace();
            throw new ProblemException("MBeanException");
        } catch (AttributeNotFoundException e) {
            e.printStackTrace();
            throw new ProblemException("AttributeNotFoundException");
        } catch (InstanceNotFoundException e) {
            e.printStackTrace();
            throw new ProblemException("InstanceNotFoundException");
        } catch (ReflectionException e) {
            e.printStackTrace();
            throw new ProblemException("ReflectionException");
        } catch (IOException e) {
            e.printStackTrace();
            throw new ProblemException("IOException");
        } catch (MalformedObjectNameException e) {
            e.printStackTrace();
            throw new ProblemException("MalformedObjectNameException");
        }
    }

    public String checkNode() throws EJBException, ProblemException {
        String attName = "";
        if (props.getProperty("mediator.attribute.livecheck.name") != null) {
            attName = props.getProperty("mediator.attribute.livecheck.name");
        }
        try {
            return (String) connection.getAttribute(ObjectName.getInstance(pumpname), attName);
        } catch (MBeanException e) {
            e.printStackTrace();
            throw new ProblemException("MBeanException");
        } catch (AttributeNotFoundException e) {
            e.printStackTrace();
            throw new ProblemException("AttributeNotFoundException");
        } catch (InstanceNotFoundException e) {
            e.printStackTrace();
            throw new ProblemException("InstanceNotFoundException");
        } catch (ReflectionException e) {
            e.printStackTrace();
            throw new ProblemException("ReflectionException");
        } catch (IOException e) {
            e.printStackTrace();
            throw new ProblemException("IOException");
        } catch (MalformedObjectNameException e) {
            e.printStackTrace();
            throw new ProblemException("MalformedObjectNameException");
        } 
    }

    public void stopConnection() throws EJBException {
        try {
            System.out.println("PumpConnectorBean : stopConnection() " + jmxhost);
            System.out.println("Connection : " + connection);
            connection.removeNotificationListener(delegateName, listener);
        } catch (InstanceNotFoundException e) {
            e.printStackTrace();
        } catch (ListenerNotFoundException e) {
            e.printStackTrace();
        } catch (IOException e) {
            e.printStackTrace();
        }
    }

    public void sendProbeToParser(String message) {
        try {
            String username = "";
            if (props.getProperty("mediator.mdb.topic.connection.username") != null) {
                username = props.getProperty("mediator.mdb.topic.connection.username");
            }
            String password = "";
            if (props.getProperty("mediator.mdb.topic.connection.password") != null) {
                password = props.getProperty("mediator.mdb.topic.connection.password");
            }
            TopicConnection topicConnection = topicConnFactory.createTopicConnection(username, password);
            topicConnection.start();
            TopicSession topicSession = topicConnection.createTopicSession(false, Session.AUTO_ACKNOWLEDGE);
            TextMessage msg = topicSession.createTextMessage();
            msg.setText(message);
            TopicPublisher pub = topicSession.createPublisher(topic);
            pub.publish(msg);
            pub.close();
            pub = null;
            topicSession.close();
            topicSession = null;
            topicConnection.stop();
            topicConnection.close();
            topicConnection = null;
        } catch (JMSException e) {
            e.printStackTrace();
        }
    }

    public Long getProbeSampleRate(String probename) throws EJBException, ProblemException {
        String attName = "";
        if (props.getProperty("mediator.attribute.get.probe.sample.time.name") != null) {
            attName = props.getProperty("mediator.attribute.get.probe.sample.time.name");
        }
        try {
            return (Long) connection.invoke(ObjectName.getInstance(pumpname), attName, new Object[]{probename}, new String[]{"java.lang.String"});
        } catch (InstanceNotFoundException e) {
            e.printStackTrace();
            throw new ProblemException("InstanceNotFoundException");
        } catch (MBeanException e) {
            e.printStackTrace();
            throw new ProblemException("MBeanException");
        } catch (ReflectionException e) {
            e.printStackTrace();
            throw new ProblemException("ReflectionException");
        } catch (IOException e) {
            e.printStackTrace();
            throw new ProblemException("IOException");
        } catch (MalformedObjectNameException e) {
            e.printStackTrace();
            throw new ProblemException("MalformedObjectNameException");
        }
    }

    public void setProbeSampleRate(String probename, Long sampleRate) throws EJBException, ProblemException {
        String attName = "";
        if (props.getProperty("mediator.attribute.set.probe.sample.time.name") != null) {
            attName = props.getProperty("mediator.attribute.set.probe.sample.time.name");
        }
        try {
            connection.invoke(ObjectName.getInstance(pumpname), attName, new Object[]{probename, sampleRate}, new String[]{"java.lang.String", "java.lang.Long"});
        } catch (InstanceNotFoundException e) {
            e.printStackTrace();
            throw new ProblemException("InstanceNotFoundException");
        } catch (MBeanException e) {
            e.printStackTrace();
            throw new ProblemException("MBeanException");
        } catch (ReflectionException e) {
            e.printStackTrace();
            throw new ProblemException("ReflectionException");
        } catch (IOException e) {
            e.printStackTrace();
            throw new ProblemException("IOException");
        } catch (MalformedObjectNameException e) {
            e.printStackTrace();
            throw new ProblemException("MalformedObjectNameException");
        }
    }

    /**
    * Decompress a byte array to a string
    * @param data The compressed data
    * @return The decompressed data in a string
    */
   public String decompress(byte[] data) {
       if (data == null) return null;

       Inflater decompressor = new Inflater();
       decompressor.setInput(data);

       // Create an expandable byte array to hold the decompressed data
       ByteArrayOutputStream bos2 = new ByteArrayOutputStream(data.length);

       // Decompress the data
       byte[] buf2 = new byte[1024];
       while (!decompressor.finished()) {
           try {
               int count = decompressor.inflate(buf2);
               bos2.write(buf2, 0, count);
           } catch (DataFormatException e) {
              // logger.log(Level.SEVERE, e.getMessage());
               e.getMessage()  ;
           }
       }
       try {
           bos2.close();
       } catch (IOException e) {
          // logger.log(Level.SEVERE, e.getMessage());
           e.getMessage()     ;
       }

       // Get the decompressed data
       byte[] decompressedData = bos2.toByteArray();
       return new String(decompressedData);
   }

}
