/**
 * Copyright (C) 2005 - Bull S.A.
 *
 * Jimys - J2EE is monitoring your system
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id:  $
 * --------------------------------------------------------------------------
 */
package org.objectweb.jimys.ads.mediator.parser;

import org.objectweb.jimys.commons.probe.AbstractProbe;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

import java.util.Hashtable;

/**
 *  Abstract class which needs to be implemented by each parser
 *
 * @author Georges Goebel, Marc Calvisi
 */
public abstract class AbstractProbeParser {
    /**
     * the node of the xml string that represents this probe
     */
    private Node node = null;
    /**
     * the id of the probe
     */
    private String id = "";

    /**
     * constructor
     * @param node the xml node that represents this probe
     * @param id the id of the probe
     */
    public AbstractProbeParser(Node node, String id) {
        this.node = node;
        this.id = id;
    }

    /**
     * get the probe
     * @return the probe object
     */
    public abstract AbstractProbe getAbstractProbe();

    /**
     * get all attributes of the node
     * @param node the node with the attributes
     * @return all attributes
     */
    public Hashtable getAttributes(Node node) {
        Hashtable attr = new Hashtable();
        NodeList nl = this.node.getChildNodes();
        for (int i = 0; i < nl.getLength(); i++) {
            Element e = (Element) nl.item(i);
            String name = e.getAttribute("name");
            NodeList childValue = e.getChildNodes();
            if (childValue.item(0).getNodeType() == Node.TEXT_NODE) {
                String value = childValue.item(0).getNodeValue();
                attr.put(name, value);
            }
        }
        return attr;
    }

    /**
     * get the node
     * @return the node
     */
    public Node getNode() {
        return this.node;
    }

    /**
     * get the id
     * @return id
     */
    public String getId() {
        return this.id;
    }

}
