/**
 * Copyright (C) 2005 - Bull S.A.
 *
 * Jimys - J2EE is monitoring your system
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id:  $
 * --------------------------------------------------------------------------
 */
package org.objectweb.jimys.ads.mediator.parser;

import org.objectweb.jimys.commons.clusterstatus.Status;
import org.objectweb.jimys.commons.probe.AbstractProbe;
import org.objectweb.jimys.commons.probe.Admin;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

import java.util.Hashtable;

/**
 * admin parser
 *
 * @author Georges Goebel, Marc Calvisi
 */
public class AdminParser extends AbstractProbeParser {
    /**
     * constructor
     * @param node the xml node containing this probe
     * @param id the id of the probe
     */
    public AdminParser(Node node, String id) {
        super(node, id);
    }

    /**
     * get the probe object
     * @return probe object
     */
    public AbstractProbe getAbstractProbe() {
        String timeStamp = ((Element) getNode()).getAttribute("createTime");
         return new Admin(Long.parseLong(timeStamp), getId(), getAttributes(getNode()));
    }

    /**
     * split the node into the different attributes
     * @param node the node with the attributes
     * @return all attributes
     */
    public Hashtable getAttributes(Node node) {
        Hashtable attr = new Hashtable();
        NodeList nl = node.getChildNodes();
        for (int i = 0; i < nl.getLength(); i++) {
            Element e = (Element) nl.item(i);
            String identifier = e.getAttribute("fullIdentifier");
            String probe = e.getAttribute("probe");
            String status = e.getAttribute("status");
            String time = e.getAttribute("time");
            Status ps = new Status(identifier + ";" + probe, status, (new Long(time)).longValue());
            attr.put(probe, ps);
        }
        return attr;
    }
}
