/**
 * Copyright (C) 2005 - Bull S.A.
 *
 * Jimys - J2EE is monitoring your system
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id:  $
 * --------------------------------------------------------------------------
 */
package org.objectweb.jimys.ads.mediator.parser;

import org.apache.xerces.parsers.DOMParser;
import org.objectweb.jimys.ads.mediator.communication.EngineConnector;
import org.objectweb.jimys.commons.probe.AbstractProbe;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.xml.sax.InputSource;

import javax.ejb.EJBException;
import javax.ejb.MessageDrivenBean;
import javax.ejb.MessageDrivenContext;
import javax.jms.*;
import java.io.*;
import java.util.*;

/**
 * Message driven bean which parse the xml-stream received by a pump
 *
 * @author Georges Goebel, Marc Calvisi
 */
public class ParseXMLBean implements MessageDrivenBean, MessageListener {
    /**
     * the node of an xml string to split into objects
     */
    private Node node = null;
    /**
     * the connection to the rule engine
     */
    private EngineConnector ec = null;

    /**
     * constructor
     */
    public ParseXMLBean() {
        ec = new EngineConnector();
    }

    /**
     * parse the string into the DOMParser
     * @param s the xml string
     * @return a DOM xml node
     */
    private Node parseXml(String s) {
        DOMParser d = new DOMParser();
        try {
            StringReader sr = new StringReader(s);
            InputSource is = new InputSource(sr);
            d.parse(is);
            return d.getDocument().getFirstChild();
        } catch (Exception e) {
            System.out.println("No XML Format " + s);
        }
        return null;
    }


    public void setMessageDrivenContext(MessageDrivenContext arg0) throws EJBException {

    }


    public void ejbRemove() throws EJBException {

    }

    public void ejbCreate() throws EJBException {
    }

    /**
     * the message containing the xml-string is split into the different objects that represent the probes
     * @param arg0
     */
    public void onMessage(Message arg0) {

        try {
            this.node = parseXml(((TextMessage) arg0).getText());
        } catch (JMSException e) {
            e.printStackTrace();
        }
        if (node == null) {
            return;
        }
        Collection events = new ArrayList();
        if (node.getNodeName().equals("admin")) {
            String adminId = ((Element) this.node).getAttribute("adminId");
            AdminParser ap = new AdminParser(node, adminId);
            events.add(ap.getAbstractProbe());
        } else if (node.getNodeName().equals("pump")) {
            NodeList nl = this.node.getChildNodes();
            String pumpId = ((Element) this.node).getAttribute("pumpID");
            String id = pumpId.substring(0, pumpId.indexOf("pump"));
            for (int i = 0; i < nl.getLength(); i++) {
                Node n = nl.item(i);
                if (n.getNodeName().equals("Cpu")) {
                    CpuProbeParser cpp = new CpuProbeParser(n, id);
                    events.add(cpp.getAbstractProbe());
                } else if (n.getNodeName().equals("Memory")) {
                    MemoryProbeParser mpp = new MemoryProbeParser(n, id);
                    events.add(mpp.getAbstractProbe());
                } else if (n.getNodeName().equals("Network")) {
                    NetworkProbeParser npp = new NetworkProbeParser(n, id);
                    events.add(npp.getAbstractProbe());
                } else if (n.getNodeName().equals("FileSystem")) {
                    FilesystemProbeParser fpp = new FilesystemProbeParser(n, id);
                    events.add(fpp.getAbstractProbe());
                } else if (n.getNodeName().equals("JonasLog")) {
                    JonasLogProbeParser jpp = new JonasLogProbeParser(n, id);
                    events.add(jpp.getAbstractProbe());
                } else if (n.getNodeName().equals("Cjdbc")) {
                    CjdbcProbeParser cpp = new CjdbcProbeParser(n, id);
                    events.add(cpp.getAbstractProbe());
                } else if (n.getNodeName().equals("Jonas")) {
                    NodeList mbeanlist = n.getChildNodes();
                    String identifier = ((Element) n).getAttribute("id");
                    for (int y = 0; y < mbeanlist.getLength(); y++) {
                        Node mbnode = mbeanlist.item(y);
                        JonasMBeanProbeParser jmbpp = new JonasMBeanProbeParser(mbnode, identifier);
                        events.add(jmbpp.getAbstractProbe());
                    }
                } else if (n.getNodeName().equals("Cartography")) {
                    CartographyProbeParser cpp = new CartographyProbeParser(n, id);
                    events.add(cpp.getAbstractProbe());
                } else if (n.getNodeName().equals("Joram")) {
                    NodeList mbeanlist = n.getChildNodes();
                    String identifier = ((Element) n).getAttribute("id");
                    for (int y = 0; y < mbeanlist.getLength(); y++) {
                        Node mbnode = mbeanlist.item(y);
                        JoramProbeParser jpp = new JoramProbeParser(mbnode, identifier);
                        events.add(jpp.getAbstractProbe());
                    }
                }
            }
        }
        Iterator it = events.iterator();
        while (it.hasNext()) {
            sendProbeToRuleEngine((AbstractProbe) it.next());
        }
        events.clear();
        events = null;

    }

    /**
     * send the probe to the rule engine
     * @param ap the probe to send to the rule engine
     */
    public void sendProbeToRuleEngine(AbstractProbe ap) {
        ec.sendProbeToRuleEngine(ap);
    }

}


