/**
 * Copyright (C) 2005 - Bull S.A.
 *
 * Jimys - J2EE is monitoring your system
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id:  $
 * --------------------------------------------------------------------------
 */
package org.objectweb.jimys.ads.mediator.web;

import org.objectweb.jimys.commons.exceptions.ProblemException;

import javax.ejb.EJBObject;
import java.rmi.RemoteException;
import java.util.Collection;
import java.util.Properties;

/**
 * interface to the MediatorWebBean
 * used by the web application to communicate with the mediator
 *
 * @author Georges Goebel, Marc Calvisi
 */

public interface MediatorWeb extends EJBObject {

    public void connectToPumps(String[] hosts) throws RemoteException, ProblemException;

    public void connectToPump(String host, int port) throws RemoteException, ProblemException;

    public void disconnectFromPump(String host) throws RemoteException, ProblemException;

    public void disconnectFromPumps() throws RemoteException, ProblemException;

    public void disconnectFromPumps(String[] hosts) throws RemoteException, ProblemException;

    public Collection getConnectedPumps() throws RemoteException;

    public Collection getAvailablePumps() throws RemoteException;

    public Collection checkCluster() throws RemoteException;

    public Collection getRegisteredProbesFromPump(String hostname) throws RemoteException, ProblemException;

    public Long getProbeSampleRate(String hostname, String probename) throws RemoteException, ProblemException;

    public String getPumpName() throws RemoteException, ProblemException;

    public void setProbeSampleRate(String hostname, String probename, Long sampleRate) throws RemoteException, ProblemException;

    public void connectToAll() throws RemoteException, ProblemException;

    public Properties getConfiguration()  throws RemoteException;
}
