/**
 * Copyright (C) 2005 - Bull S.A.
 *
 * Jimys - J2EE is monitoring your system
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id:  $
 * --------------------------------------------------------------------------
 */
package org.objectweb.jimys.webapp.ads.enginewebapp;

import org.apache.struts.action.ActionError;
import org.apache.struts.action.ActionErrors;
import org.apache.struts.action.ActionMapping;
import org.objectweb.jimys.ads.adengine.web.EngineWeb;
import org.objectweb.jimys.commons.configuration.Configuration;
import org.objectweb.jimys.webapp.base.BaseForm;

import javax.ejb.Handle;
import javax.servlet.http.HttpServletRequest;
import java.rmi.RemoteException;

/**
 *
 * ModifyConfigurationForm
 *
 * @author Georges Goebel, Marc Calvisi
 */
public class ModifyConfigurationForm extends BaseForm {
    private String fireRuleAfter = "";
    private String ruleFireTimer = "";
    private String modifyValue = "";

    public String getFireRuleAfter() {
        return fireRuleAfter;
    }

    public void setFireRuleAfter(String fireRuleAfter) {
        this.fireRuleAfter = fireRuleAfter;
    }

    public String getRuleFireTimer() {
        return ruleFireTimer;
    }

    public void setRuleFireTimer(String ruleFireTimer) {
        this.ruleFireTimer = ruleFireTimer;
    }

    public String getModifyValue() {
        return modifyValue;
    }

    public void setModifyValue(String modifyValue) {
        this.modifyValue = modifyValue;
    }

    public ActionErrors validate(ActionMapping mapping, HttpServletRequest request) {
        ActionErrors ae = new ActionErrors();
        if (modifyValue.equals("fireRuleAfter")) {
            if (!isBlank(getFireRuleAfter())) {
                if (!isNumeric(getFireRuleAfter()) && !(Long.parseLong(getRuleFireTimer()) < 0)) {
                    ae.add("error.jimys.engine.configuration.fire.after.wrong.value", new ActionError("error.jimys.engine.configuration.fire.after.wrong.value"));
                }
            } else {
                ae.add("error.jimys.engine.configuration.fire.after.wrong.value", new ActionError("error.jimys.engine.configuration.fire.after.wrong.value"));
            }
        } else if (modifyValue.equals("ruleFireTimer")) {
            if (!isBlank(getRuleFireTimer())) {
                if (!isNumeric(getRuleFireTimer()) && !(Long.parseLong(getRuleFireTimer()) < 0)) {
                    ae.add("error.jimys.engine.configuration.fire.timer.wrong.value", new ActionError("error.jimys.engine.configuration.fire.timer.wrong.value"));
                }
            } else {
                ae.add("error.jimys.engine.configuration.fire.timer.wrong.value", new ActionError("error.jimys.engine.configuration.fire.timer.wrong.value"));
            }
        }
        return ae;
    }

    public void reset(ActionMapping mapping, HttpServletRequest request) {
        Handle handle = (Handle) request.getSession().getAttribute("engineweb");

        EngineWeb ew = null;
        try {
            if (handle == null) {
                ew = this.createEngineWeb();
                if (ew != null) {
                    request.getSession().setAttribute("engineweb", ew.getHandle());
                }
            } else {
                ew = (EngineWeb) handle.getEJBObject();

            }
        } catch (RemoteException e) {
            e.printStackTrace();
        }

        if (ew != null) {
            try {
                Configuration c = ew.getConfiguration();
                fireRuleAfter = c.getFireRuleAfter();
                ruleFireTimer = c.getTimeInterval();
            } catch (RemoteException e) {
                e.printStackTrace();
            }
        } else {
            fireRuleAfter = "";
            ruleFireTimer = "";
        }
    }


}
