/**
 * Copyright (C) 2005 - Bull S.A.
 *
 * Jimys - J2EE is monitoring your system
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id:  $
 * --------------------------------------------------------------------------
 */
package org.objectweb.jimys.webapp.ads.mediatorwebapp;

import org.apache.struts.action.ActionForm;
import org.apache.struts.action.ActionForward;
import org.apache.struts.action.ActionMapping;
import org.objectweb.jimys.ads.mediator.web.MediatorWeb;
import org.objectweb.jimys.commons.exceptions.ProblemException;
import org.objectweb.jimys.webapp.base.BaseAction;

import javax.ejb.Handle;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;

/**
 *
 * MediatorDetailConfigAction
 *
 * @author Georges Goebel, Marc Calvisi
 */
public class MediatorDetailConfigAction extends BaseAction {
    public ActionForward execute(ActionMapping mapping, ActionForm form, HttpServletRequest request, HttpServletResponse response)
            throws IOException, ServletException {

        Handle handle = (Handle) request.getSession().getAttribute("mediator");
        MediatorWeb mediator = null;
        if (handle == null) {
            mediator = this.createMediator();
            if (mediator == null) {
                return mapping.findForward("failure");
            }
            request.getSession().setAttribute("mediator", mediator.getHandle());
        } else {
            mediator = (MediatorWeb) handle.getEJBObject();
        }

        // get the probes
        if (request.getParameter("main") != null) {
            request.getSession().removeAttribute("connectedHosts");
            request.getSession().removeAttribute("hostProbes");
            return mapping.findForward("success");
        }

        if (request.getParameter("detail") != null) {
            Collection hostnames = mediator.getConnectedPumps();
            request.getSession().setAttribute("connectedHosts", hostnames);
            request.getSession().removeAttribute("hostProbes");
            return mapping.findForward("detailConfig");
        }

        if (request.getParameter("change") != null) {
            //System.out.println("------------------- changing ----------------");
            MediatorDetailConfigForm mdcf = (MediatorDetailConfigForm) form;
            if (mdcf.getHostName() != null && mdcf.getNewSampleTime() != null && mdcf.getProbeName() != null) {
                //  System.out.println("New Sample time : " + mdcf.getNewSampleTime());
                Long sample = new Long(Long.parseLong(mdcf.getNewSampleTime()));
                try {
                    mediator.setProbeSampleRate(mdcf.getHostName(), mdcf.getProbeName(), sample);
                    Collection col = mediator.getRegisteredProbesFromPump(mdcf.getHostName());

                    Collection result = new ArrayList();
                    for (Iterator iterator = col.iterator(); iterator.hasNext();) {
                        String s = (String) iterator.next();
                        Long sampleTime = mediator.getProbeSampleRate(mdcf.getHostName(), s);
                        ProbeHelper ph = new ProbeHelper();
                        ph.setProbeName(s);
                        ph.setSampleTime(sampleTime);
                        ph.setHostName(request.getParameter("hostName"));
                        result.add(ph);
                    }
                    mdcf.reset(mapping, request);
                    request.getSession().setAttribute("hostProbes", result);
                } catch (ProblemException e) {
                    e.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
                }
                return mapping.findForward("detailConfig");
            }
        }

        if ((request.getParameter("hostName") != null)) {
            try {
                Collection col = mediator.getRegisteredProbesFromPump(request.getParameter("hostName"));

                Collection result = new ArrayList();
                for (Iterator iterator = col.iterator(); iterator.hasNext();) {
                    String s = (String) iterator.next();
                    Long sampleTime = mediator.getProbeSampleRate(request.getParameter("hostName"), s);
                    ProbeHelper ph = new ProbeHelper();
                    ph.setProbeName(s);
                    ph.setSampleTime(sampleTime);
                    ph.setHostName(request.getParameter("hostName"));
                    result.add(ph);
                }

                request.getSession().setAttribute("hostProbes", result);
                return mapping.findForward("detailConfig");
            } catch (ProblemException e) {
                e.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
            }
        }


        return mapping.findForward("failure");
    }


}
