/**
 * Copyright (C) 2005 - Bull S.A.
 *
 * Jimys - J2EE is monitoring your system
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id:  $
 * --------------------------------------------------------------------------
 */
package org.objectweb.jimys.webapp.base;

import org.objectweb.jimys.dbConnection.session.DBManager;
import org.objectweb.jimys.dbConnection.session.DBManagerHome;
import org.apache.struts.action.Action;
import org.objectweb.jimys.ads.adengine.timer.RuleFireTimer;
import org.objectweb.jimys.ads.adengine.timer.RuleFireTimerHome;
import org.objectweb.jimys.ads.adengine.web.EngineWeb;
import org.objectweb.jimys.ads.adengine.web.EngineWebHome;
import org.objectweb.jimys.ads.mediator.web.MediatorWeb;
import org.objectweb.jimys.ads.mediator.web.MediatorWebHome;

import javax.ejb.CreateException;
import javax.naming.Context;
import javax.naming.InitialContext;
import javax.naming.NamingException;
import javax.rmi.PortableRemoteObject;
import java.rmi.RemoteException;
import java.util.Properties;
import java.io.IOException;
import java.io.InputStream;

/**
 *
 * BaseAction
 *
 * @author Georges Goebel, Marc Calvisi
 */
public class BaseAction extends Action {

    public EngineWeb createEngineWeb() {
        InputStream config = Thread.currentThread().getContextClassLoader().getResourceAsStream("dbmanager.properties");
        Properties props = new Properties();
        try {
            props.load(config);
        } catch (IOException e1) {
            e1.printStackTrace();
        }
        String factory = "";
        String url = "";
        if (props.getProperty("engine.java.naming.factory.initial") != null) {
            factory = props.getProperty("engine.java.naming.factory.initial");
        }

        if (props.getProperty("engine.java.naming.provider.url") != null) {
            url = props.getProperty("engine.java.naming.provider.url");
        }
        Properties properties = new Properties();
        properties.put("java.naming.factory.initial", factory);
        properties.put("java.naming.provider.url", url);
        try {
            Context initialContext = new InitialContext(properties);
            String ejbname = "";
            if (props.getProperty("engine.web.session.name") != null) {
                ejbname = props.getProperty("engine.web.session.name");
            }
            EngineWebHome ewhome = (EngineWebHome) PortableRemoteObject.narrow(initialContext.lookup(ejbname), EngineWebHome.class);
            return ewhome.create();
        } catch (NamingException e) {
            e.printStackTrace();
        } catch (CreateException e) {
            e.printStackTrace();
        } catch (RemoteException e) {
            e.printStackTrace();
        }
        return null;
    }

    public MediatorWeb createMediator() {
        InputStream config = Thread.currentThread().getContextClassLoader().getResourceAsStream("dbmanager.properties");
        Properties props = new Properties();
        try {
            props.load(config);
        } catch (IOException e1) {
            e1.printStackTrace();
        }
        String factory = "";
        String url = "";
        if (props.getProperty("mediator.java.naming.factory.initial") != null) {
            factory = props.getProperty("mediator.java.naming.factory.initial");
        }

        if (props.getProperty("mediator.java.naming.provider.url") != null) {
            url = props.getProperty("mediator.java.naming.provider.url");
        }
        Properties properties = new Properties();
        properties.put("java.naming.factory.initial", factory);
        properties.put("java.naming.provider.url", url);
        try {
            Context initialContext = new InitialContext(properties);
            String ejbname = "";
            if (props.getProperty("mediator.web.session.name") != null) {
                ejbname = props.getProperty("mediator.web.session.name");
            }
            MediatorWebHome mwhome = (MediatorWebHome) PortableRemoteObject.narrow(initialContext.lookup(ejbname), MediatorWebHome.class);
            return mwhome.create();
        } catch (NamingException e) {
            e.printStackTrace();
        } catch (CreateException e) {
            e.printStackTrace();
        } catch (RemoteException e) {
            e.printStackTrace();
        }
        return null;
    }

    public RuleFireTimer createRuleFireTimer() {

        InputStream config = Thread.currentThread().getContextClassLoader().getResourceAsStream("dbmanager.properties");
        Properties props = new Properties();
        try {
            props.load(config);
        } catch (IOException e1) {
            e1.printStackTrace();
        }
        String factory = "";
        String url = "";
        if (props.getProperty("engine.java.naming.factory.initial") != null) {
            factory = props.getProperty("engine.java.naming.factory.initial");
        }

        if (props.getProperty("engine.java.naming.provider.url") != null) {
            url = props.getProperty("engine.java.naming.provider.url");
        }
        Properties properties = new Properties();
        properties.put("java.naming.factory.initial", factory);
        properties.put("java.naming.provider.url", url);
        try {
            Context initialContext = new InitialContext(properties);
            String ejbname = "";
            if (props.getProperty("engine.timer.session.name") != null) {
                ejbname = props.getProperty("engine.timer.session.name");
            }
            RuleFireTimerHome rfth = (RuleFireTimerHome) PortableRemoteObject.narrow(initialContext.lookup(ejbname), RuleFireTimerHome.class);
            return rfth.create();
        } catch (NamingException e) {
            e.printStackTrace();
        } catch (CreateException e) {
            e.printStackTrace();
        } catch (RemoteException e) {
            e.printStackTrace();
        }
        return null;
    }


    public DBManager createDBManager() {
        InputStream config = Thread.currentThread().getContextClassLoader().getResourceAsStream("dbmanager.properties");
        Properties props = new Properties();
        try {
            props.load(config);
        } catch (IOException e1) {
            e1.printStackTrace();
        }
        String factory = "";
        String url = "";
        if (props.getProperty("database.java.naming.factory.initial") != null) {
            factory = props.getProperty("database.java.naming.factory.initial");
        }

        if (props.getProperty("database.java.naming.provider.url") != null) {
            url = props.getProperty("database.java.naming.provider.url");
        }
        Properties properties = new Properties();
        properties.put("java.naming.factory.initial", factory);
        properties.put("java.naming.provider.url", url);
        try {
            Context ictx = new InitialContext(properties);
            String ejbname = "";
            if (props.getProperty("database.remote.home") != null) {
                ejbname = props.getProperty("database.remote.home");
            }
            System.out.println("ejbname : "+ejbname);
            DBManagerHome dbHome = (DBManagerHome) PortableRemoteObject.narrow(ictx.lookup(ejbname), DBManagerHome.class);
            System.out.println("dbHome : "+dbHome.getClass());
            DBManager dbm = dbHome.create();
            System.out.println("dbm : "+dbm.getClass());
            return dbm;
            //return dbHome.create();
        } catch (NamingException e) {
            e.printStackTrace();
        } catch (RemoteException e) {
            e.printStackTrace();
        } catch (CreateException e) {
            e.printStackTrace();
        }
        return null;

    }

}
