/**
 * Copyright (C) 2005 - Bull S.A.
 *
 * Jimys - J2EE is monitoring your system
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id:  $
 * --------------------------------------------------------------------------
 */
package org.objectweb.jimys.webapp.base;

import org.apache.struts.action.ActionForm;
import org.objectweb.jimys.ads.adengine.web.EngineWeb;
import org.objectweb.jimys.ads.adengine.web.EngineWebHome;

import javax.naming.Context;
import javax.naming.InitialContext;
import javax.naming.NamingException;
import javax.rmi.PortableRemoteObject;
import javax.ejb.CreateException;
import java.util.regex.Pattern;
import java.util.Properties;
import java.io.*;
import java.rmi.RemoteException;
import java.text.ParseException;
import java.text.Format;
import java.text.SimpleDateFormat;

/**
 *
 * BaseForm
 *
 * @author Georges Goebel, Marc Calvisi
 */
public class BaseForm extends ActionForm {
    public boolean isBlank(String str) {
        if (str == null) return true;
        return (str.length() == 0);
    }

    public boolean isBlank(String[] str) {
        if (str == null) return true;
        return (str.length == 0);
    }

    public boolean areHostsValid(String[] str) {
        for (int i = 0; i < str.length; i++) {
            if (isBlank(str[i]) || str[i].equals("")) return false;
        }
        return true;
    }

    public boolean isValidPort(String port) {
        return Pattern.matches("[0-9]*", port);
    }

    public boolean isNumeric(String value){
        return (Pattern.matches("[0-9]*", value));
    }

    public boolean isValidDate(String date){
        try {
            Format formatter = new SimpleDateFormat("HH:mm:ss dd/MM/yyyy");
            formatter.parseObject(date);
            return true;
        } catch (ParseException e) {
            return false;
        }
    }

     public EngineWeb createEngineWeb() {
        InputStream config = Thread.currentThread().getContextClassLoader().getResourceAsStream("dbmanager.properties");
        Properties props = new Properties();
        try {
            props.load(config);
        } catch (IOException e1) {
            e1.printStackTrace();
        }
        String factory = "";
        String url = "";
        if (props.getProperty("engine.java.naming.factory.initial") != null) {
            factory = props.getProperty("engine.java.naming.factory.initial");
        }

        if (props.getProperty("engine.java.naming.provider.url") != null) {
            url = props.getProperty("engine.java.naming.provider.url");
        }
        Properties properties = new Properties();
        properties.put("java.naming.factory.initial", factory);
        properties.put("java.naming.provider.url", url);
        try {
            Context initialContext = new InitialContext(properties);
            String ejbname = "";
            if (props.getProperty("engine.web.session.name") != null) {
                ejbname = props.getProperty("engine.web.session.name");
            }
            EngineWebHome ewhome = (EngineWebHome) PortableRemoteObject.narrow(initialContext.lookup(ejbname), EngineWebHome.class);
            return ewhome.create();
        } catch (NamingException e) {
            e.printStackTrace();
        } catch (CreateException e) {
            e.printStackTrace();
        } catch (RemoteException e) {
            e.printStackTrace();
        }
        return null;
    }
}
