/**
 * Copyright (C) 2005 - Bull S.A.
 *
 * Jimys - J2EE is monitoring your system
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id:  $
 * --------------------------------------------------------------------------
 */
package org.objectweb.jimys.webapp.visualizeDB.chart;

import org.objectweb.jimys.dbConnection.session.DBManager;
import org.objectweb.jimys.dbConnection.valueObjects.ResultProbeValueObject;
import org.apache.struts.action.ActionForm;
import org.apache.struts.action.ActionForward;
import org.apache.struts.action.ActionMapping;
import org.jfree.chart.ChartRenderingInfo;
import org.jfree.chart.ChartUtilities;
import org.jfree.chart.JFreeChart;
import org.jfree.chart.axis.DateAxis;
import org.jfree.chart.axis.NumberAxis;
import org.jfree.chart.entity.StandardEntityCollection;
import org.jfree.chart.labels.StandardXYToolTipGenerator;
import org.jfree.chart.plot.XYPlot;
import org.jfree.chart.renderer.xy.StackedXYAreaRenderer;
import org.jfree.chart.renderer.xy.XYAreaRenderer;
import org.jfree.chart.urls.TimeSeriesURLGenerator;
import org.jfree.data.xy.DefaultTableXYDataset;
import org.jfree.data.xy.XYSeries;
import org.objectweb.jimys.webapp.base.BaseAction;

import javax.ejb.Handle;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;
import java.awt.*;
import java.awt.geom.Ellipse2D;
import java.awt.geom.Rectangle2D;
import java.awt.image.BufferedImage;
import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.rmi.RemoteException;
import java.text.NumberFormat;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;
import java.util.Locale;

/**
 *
 * ChartAction
 *
 * @author Georges Goebel, Marc Calvisi
 */
public class ChartAction extends BaseAction {
    public ActionForward execute(ActionMapping mapping, ActionForm form, HttpServletRequest request, HttpServletResponse response)
            throws IOException, ServletException {

        System.out.println("ChartAction ......");

        String probeIdentifier = request.getParameter("probeIdentifier");
        if (probeIdentifier == null) {
            if (request.getSession().getAttribute("probeIdentifier") != null) {
                probeIdentifier = request.getSession().getAttribute("probeIdentifier").toString();
            }
        }

        String attribute = request.getParameter("attribute");
        if (attribute == null) {
            if (request.getSession().getAttribute("attribute") != null) {
                attribute = request.getSession().getAttribute("attribute").toString();
            }
        }

        String begintime = request.getParameter("begintime");
        if (begintime == null) {
            if (request.getSession().getAttribute("begintime") != null) {
                begintime = request.getSession().getAttribute("begintime").toString();
            }
        }

        if ((probeIdentifier != null) && (attribute != null)) {

            Handle handle = (Handle) request.getSession().getAttribute("dbmanager");
            DBManager dbManager = null;
            if (handle == null) {
                dbManager = this.createDBManager();
                if (dbManager == null) {
                    return mapping.findForward("failure");
                }
                request.getSession().setAttribute("dbmanager", dbManager.getHandle());

            } else {
                dbManager = (DBManager) handle.getEJBObject();
            }


            try {
                Collection col = null;
                if (begintime != null) {
                    col = dbManager.findAllAttributes(attribute, probeIdentifier, begintime);
                } else {
                    col = dbManager.findAllAttributes(attribute, probeIdentifier);
                }

                /*
                Collection resultProbes = new ArrayList();

                for (Iterator iterator = col.iterator(); iterator.hasNext();) {
                    ResultValueObject resultValueObject = (ResultValueObject) iterator.next();
                    Collection c = resultValueObject.getResultProbes();
                    for (Iterator iterator1 = c.iterator(); iterator1.hasNext();) {
                        ResultProbeValueObject o =  (ResultProbeValueObject) iterator1.next();
                        if ((o.getProbeIdentifier().equals(probeIdentifier)) && (o.getAttributeName().equals(attribute))){
                                resultProbes.add(o);
                        }
                    }
                }
                */

                String filename = generateXYAreaChart(request.getSession(), col, probeIdentifier);
                request.getSession().setAttribute("filename", filename);

            } catch (RemoteException e) {
                e.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
            }


        }

        return mapping.findForward("success");

    }


    private  synchronized String generateXYAreaChart(HttpSession session, Collection data, String title) {
        String filename = null;


        //WebHitDataSet whDataSet = new WebHitDataSet();
        //ArrayList sections = whDataSet.getSections();
        ArrayList sections = new ArrayList();
        sections.add("Section 1");
//        sections.add("Section 2");

        Iterator sectionIter = sections.iterator();
        DefaultTableXYDataset dataset = new DefaultTableXYDataset();

        int counter = 0;

        while (sectionIter.hasNext()) {
            System.out.println("Seciton ...");
            String section = (String) sectionIter.next();
            // data of section
            /*
            ArrayList list = new ArrayList();
            list.add(new Double(5 + counter++));
            list.add(new Double(5 + counter++));
            list.add(new Double(5 + counter++));
              */

            XYSeries dataSeries = new XYSeries(section, true, false);
            Iterator webHitIter = data.iterator();
            while (webHitIter.hasNext()) {
                ResultProbeValueObject webHit = (ResultProbeValueObject) webHitIter.next();
                // x y value
                dataSeries.add(Long.parseLong(webHit.getId()), Double.parseDouble(webHit.getAttributeValue()));
            }
            dataset.addSeries(dataSeries);
        }

        if (dataset.getItemCount() == 0) {
            System.out.println("No data has been found");
        }

        SimpleDateFormat sdf = new SimpleDateFormat("dd-MMM-yyyy", Locale.UK);
        StandardXYToolTipGenerator ttg = new StandardXYToolTipGenerator(StandardXYToolTipGenerator.DEFAULT_TOOL_TIP_FORMAT, sdf, NumberFormat.getInstance());
        TimeSeriesURLGenerator urlg = new TimeSeriesURLGenerator(sdf, "bar_chart.jsp", "series", "hitDate");
        // Create the X-Axis
        DateAxis xAxis = new DateAxis(null);
        xAxis.setLowerMargin(0.0);
        xAxis.setUpperMargin(0.0);
        // Create the Y-Axis
        NumberAxis yAxis = new NumberAxis(null);
        yAxis.setAutoRangeIncludesZero(true);
        // Create the renderer

        StackedXYAreaRenderer renderer = new StackedXYAreaRenderer(XYAreaRenderer.AREA_AND_SHAPES, ttg, urlg);
        renderer.setSeriesPaint(0, new Color(255, 255, 180));
        renderer.setSeriesPaint(1, new Color(206, 230, 255));
        renderer.setSeriesPaint(2, new Color(255, 230, 230));
        renderer.setSeriesPaint(3, new Color(206, 255, 206));
        renderer.setShapePaint(Color.gray);
        renderer.setShapeStroke(new BasicStroke(0.5f));
        renderer.setShape(new Ellipse2D.Double(-3, -3, 6, 6));
        renderer.setOutline(true);
        // Create the plot
        XYPlot plot = new XYPlot(dataset, xAxis, yAxis, renderer);
        plot.setForegroundAlpha(0.65f);
        // Reconfigure Y-Axis so the auto-range knows that the data is stacked
        yAxis.configure();
        // Create the chart
        JFreeChart chart = new JFreeChart(null, JFreeChart.DEFAULT_TITLE_FONT, plot, true);
        chart.setTitle(title);
        chart.setBackgroundPaint(java.awt.Color.white);
        // Write the chart image to the temporary directory
        ChartRenderingInfo info = new ChartRenderingInfo(new StandardEntityCollection());


        try {
            //    filename = ServletUtilities.saveChartAsPNG(chart, 500, 300, info, session);
//            filename = ServletUtilities.saveChartAsPNG(chart, 640, 480, info, session);

            BufferedImage g = new BufferedImage(640, 480, BufferedImage.TYPE_INT_RGB);
            BufferedImage gg = g.getSubimage(0,0,640,240);
            BufferedImage gg2 = g.getSubimage(0,240,640,240);



            Graphics2D g2d = gg.createGraphics();
            Rectangle2D r2D = new Rectangle2D.Double(0.0D, 0.0D, 640, 240);

            Graphics2D g22d = gg2.createGraphics();
            Rectangle2D r22D = new Rectangle2D.Double(0.0D, 0, 640, 240);


            chart.draw(g2d, r2D);
            chart.draw(g22d, r22D);

            byte[] imageByte = ChartUtilities.encodeAsPNG(g);

            System.out.println("imageByte = " + imageByte);

            FileOutputStream fout = new FileOutputStream(new File("/tmp/blabla.png"));
            fout.write(imageByte);
            fout.flush();
            fout.close();

            /*
          ChartUtilities.writeImageMap(pw, filename, info);
          pw.flush();
            */
        } catch (IOException e) {
            e.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
            filename = "public_error_500x300.png";
        }
        // Write the image map to the PrintWriter
        return filename;
    }
}
