/**
 * Copyright (C) 2005 - Bull S.A.
 *
 * Jimys - J2EE is monitoring your system
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id:  $
 * --------------------------------------------------------------------------
 */
package org.objectweb.jimys.webapp.visualizeDB.chart;

import org.objectweb.jimys.dbConnection.session.DBManager;
import org.objectweb.jimys.dbConnection.valueObjects.ResultProbeValueObject;
import org.apache.struts.action.ActionForm;
import org.apache.struts.action.ActionForward;
import org.apache.struts.action.ActionMapping;
import org.jfree.chart.ChartFactory;
import org.jfree.chart.ChartRenderingInfo;
import org.jfree.chart.JFreeChart;
import org.jfree.chart.axis.DateAxis;
import org.jfree.chart.axis.NumberAxis;
import org.jfree.chart.entity.StandardEntityCollection;
import org.jfree.chart.plot.PlotOrientation;
import org.jfree.chart.servlet.ServletUtilities;
import org.jfree.data.xy.XYSeries;
import org.jfree.data.xy.XYSeriesCollection;
import org.objectweb.jimys.webapp.base.BaseAction;

import javax.ejb.Handle;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.rmi.RemoteException;
import java.util.Collection;
import java.util.Iterator;

/**
 *
 * ChartMoreParametersAction
 *
 * @author Georges Goebel, Marc Calvisi
 */
public class ChartMoreParametersAction extends BaseAction {
    public ActionForward execute(ActionMapping mapping, ActionForm form, HttpServletRequest request, HttpServletResponse response)
            throws IOException, ServletException {

        /*
        String[] probeIdentifier = request.getParameterValues("probeIdentifier");
        String[] parameter = request.getParameterValues("parameter");
        String begintime = request.getParameter("begintime");
          */

        Collection selectedProbeList = (Collection) request.getSession().getAttribute("selectedProbeList");
        Collection selectedProbeAttributeList = (Collection) request.getSession().getAttribute("selectedProbeAttributeList");
        String begintime = (String) request.getSession().getAttribute("begintime");

        if ((selectedProbeList != null) && (selectedProbeAttributeList != null) && (selectedProbeList.size() == selectedProbeAttributeList.size())) {

            try {

                Handle handle = (Handle) request.getSession().getAttribute("dbmanager");
                DBManager dbManager = null;
                if (handle == null) {
                    dbManager = this.createDBManager();
                    if (dbManager == null) {
                        return mapping.findForward("failure");
                    }
                    request.getSession().setAttribute("dbmanager", dbManager.getHandle());

                } else {
                    dbManager = (DBManager) handle.getEJBObject();
                }

                XYSeriesCollection xyDSet = new XYSeriesCollection();

                Iterator it1 = selectedProbeList.iterator();
                Iterator it2 = selectedProbeAttributeList.iterator();

                while (it1.hasNext() && it2.hasNext()) {
                    String para = (String) it2.next();
                    String probeId = (String) it1.next();

                    Collection col = null;
                    if (begintime != null) {
                        col = dbManager.findAllAttributes(para, probeId, begintime);
                    } else {
                        col = dbManager.findAllAttributes(para, probeId);
                    }

                    XYSeries series = new XYSeries(probeId);

                    for (Iterator iterator = col.iterator(); iterator.hasNext();) {
                        ResultProbeValueObject resultProbeValueObject = (ResultProbeValueObject) iterator.next();
                        series.add(resultProbeValueObject.getTimestamp(), Double.valueOf(resultProbeValueObject.getAttributeValue()));
                    }
                    xyDSet.addSeries(series);
                }
                DateAxis xAxis = new DateAxis(null);
                xAxis.setLowerMargin(0.0);
                xAxis.setUpperMargin(0.0);
                // Create the Y-Axis
                NumberAxis yAxis = new NumberAxis(null);
                yAxis.setAutoRangeIncludesZero(true);


                JFreeChart chart = ChartFactory.createXYLineChart("Title", "x-axis", "y-axis", xyDSet, PlotOrientation.VERTICAL, true, true, false);
                chart.setTitle("More Parameters");
                chart.getXYPlot().setDomainAxis(xAxis);
                chart.getXYPlot().setRangeAxis(yAxis);

                String filename = null;
                ChartRenderingInfo info = new ChartRenderingInfo(new StandardEntityCollection());
                try {
                    filename = ServletUtilities.saveChartAsPNG(chart, 700, 500, info, request.getSession());
                } catch (IOException e) {
                    e.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
                    filename = "public_error_500x300.png";
                }

                request.getSession().setAttribute("filename", filename);


                return mapping.findForward("success");

            }catch (RemoteException e) {
                e.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
            }

        }

        return mapping.findForward("error");

    }


}
