/**
 * Copyright (C) 2005 - Bull S.A.
 *
 * Jimys - J2EE is monitoring your system
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id:  $
 * --------------------------------------------------------------------------
 */
package org.objectweb.jimys.webapp.visualizeDB.clusterStatus;

import org.objectweb.jimys.dbConnection.session.DBManager;
import fr.improve.struts.taglib.layout.menu.MenuComponent;
import fr.improve.struts.taglib.layout.util.LayoutUtils;
import org.apache.struts.action.ActionForm;
import org.apache.struts.action.ActionForward;
import org.apache.struts.action.ActionMapping;
import org.objectweb.jimys.webapp.base.BaseAction;
import org.objectweb.jimys.commons.clusterstatus.*;
import org.objectweb.jimys.commons.level.LevelHelper;

import javax.ejb.CreateException;
import javax.ejb.Handle;
import javax.naming.NamingException;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.rmi.RemoteException;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;

/**
 *
 * ClusterStatusAction
 *
 * @author Georges Goebel, Marc Calvisi
 */
public class ClusterStatusAction extends BaseAction {
    public ActionForward execute(ActionMapping mapping, ActionForm form, HttpServletRequest request, HttpServletResponse response)
            throws IOException, ServletException {

        ClusterStatusForm csf = (ClusterStatusForm) form;
        if (request.getParameter("action") != null && request.getSession().getAttribute("clusterName") != null) {
            if (request.getParameter("action").equals("showData")) {
                Handle handle = (Handle) request.getSession().getAttribute("dbmanager");
                DBManager dbManager = null;
                if (handle == null) {
                    dbManager = this.createDBManager();
                    if (dbManager == null) {
                        return mapping.findForward("failure");
                    }
                    request.getSession().setAttribute("dbmanager", dbManager.getHandle());

                } else {
                    dbManager = (DBManager) handle.getEJBObject();
                }
                String level = request.getParameter("level");
                String query = request.getParameter("query");
                Collection alarms = new ArrayList();
                if (level != null && query != null) {
                    alarms = dbManager.findProbeWithStatus(query, level);
                }
                request.getSession().setAttribute("alarms", alarms);


            }

            if (request.getParameter("action").equals("changeLevel")) {
                //changing the level
                request.getSession().setAttribute("level", csf.getSelectedLevel());
                System.out.println("New Level : " + csf.getSelectedLevel());
                try {
                    buildTree(request.getSession().getAttribute("clusterName").toString(), request, csf.getSelectedLevel());
                    request.getSession().removeAttribute("alarms");
                } catch (NamingException e) {
                    e.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
                } catch (CreateException e) {
                    e.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
                }
            }
        } else {
            if (csf.getClusterName() != null) {
                request.getSession().setAttribute("clusterName", csf.getClusterName());
                String level = (String) request.getSession().getAttribute("level");
                if (level == null) {
                    level = LevelHelper.ALERT;
                }

                try {
                    buildTree(request.getSession().getAttribute("clusterName").toString(), request, level);

                    return mapping.findForward("success");

                } catch (NamingException e) {
                    e.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
                } catch (CreateException e) {
                    e.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
                }

            }else{
                request.getSession().removeAttribute("clusterName");
                request.getSession().removeAttribute("cs");

            }

        }


        return mapping.findForward("success");

    }

    private void buildTree(String clustername, HttpServletRequest request, String level) throws NamingException, RemoteException, CreateException {
        Handle handle = (Handle) request.getSession().getAttribute("dbmanager");
        DBManager dbManager = null;
        if (handle == null) {
            dbManager = this.createDBManager();
            request.getSession().setAttribute("dbmanager", dbManager.getHandle());

        } else {
            dbManager = (DBManager) handle.getEJBObject();
        }

        ClusterStatus cs = dbManager.getLastClusterStatus(clustername);

        MenuComponent lc_menu1 = new MenuComponent();
        lc_menu1.setName("clusterStatus");

        MenuComponent lc_menu11 = new MenuComponent();
        if (cs != null) {
            request.getSession().setAttribute("cs", cs);
            lc_menu11.setTitle(cs.toString());

            Collection hosts = cs.getHosts().values();
            for (Iterator iterator = hosts.iterator(); iterator.hasNext();) {
                HostStatus hostStatus = (HostStatus) iterator.next();
                MenuComponent lc_menu111 = new MenuComponent();
                lc_menu111.setTitle(hostStatus.getNameAndStatus());
                lc_menu111.setLocation(request.getRequestURI() + "?action=showData&node=host&level=" + level + "&query=" + hostStatus.getIdentifier());


                MenuComponent lc_menu1111 = new MenuComponent();
                lc_menu1111.setTitle("System");

                PumpStatus pumpStatus = hostStatus.getPumpStatus();
                MenuComponent lc_menu11111 = new MenuComponent();
                lc_menu11111.setTitle(pumpStatus.getNameAndStatus());
                lc_menu1111.addMenuComponent(lc_menu11111);

                Collection probes = pumpStatus.getProbes().values();
                for (Iterator iterator1 = probes.iterator(); iterator1.hasNext();) {
                    ProbeStatus probeStatus = (ProbeStatus) iterator1.next();
                    lc_menu11111 = new MenuComponent();

                    // modif -> hibernate
                    /*
                    Collection col = dbManager.findAllResultsWithLevel(LevelHelper.ALERT, probeStatus.getIdentifier());
                    if (col.size() > 0) {
                        probeStatus.setStatus(LevelHelper.ALERT);
                    }
                    */
                    Integer count = dbManager.findResultsWithLevelCount(LevelHelper.ALERT, probeStatus.getIdentifier());
                    if ((count == null) || (count.intValue() > 0)) {
                        probeStatus.setStatus(LevelHelper.ALERT);
                         
                        // modify the clusterstatus, hoststatus
                        hostStatus.setStatus(LevelHelper.ALERT);
                        lc_menu111.setTitle(hostStatus.getNameAndStatus());

                        cs.setStatus(LevelHelper.ALERT);
                        lc_menu11.setTitle(cs.toString());

                    }



                    lc_menu11111.setTitle(probeStatus.getNameAndStatus());
                    lc_menu1111.addMenuComponent(lc_menu11111);
                }

                Collection domains = hostStatus.getDomains().values();
                MenuComponent lc_menu1113 = new MenuComponent();
                lc_menu1113.setTitle("JOnAS");
                for (Iterator iterator1 = domains.iterator(); iterator1.hasNext();) {
                    DomainStatus domainStatus = (DomainStatus) iterator1.next();
                    MenuComponent lc_menu1112 = new MenuComponent();
                    lc_menu1112.setTitle(domainStatus.getNameAndStatus());

                    Collection instances = domainStatus.getInstances().values();
                    for (Iterator iterator2 = instances.iterator(); iterator2.hasNext();) {
                        InstanceStatus instanceStatus = (InstanceStatus) iterator2.next();
                        lc_menu11111 = new MenuComponent();
                        lc_menu11111.setTitle(instanceStatus.getNameAndStatus());

                        probes = instanceStatus.getProbes().values();
                        for (Iterator iterator3 = probes.iterator(); iterator3.hasNext();) {
                            ProbeStatus probeStatus = (ProbeStatus) iterator3.next();
                            MenuComponent lc_menu111111 = new MenuComponent();

                            // modif -> hibernate
                            /*
                            Collection col = dbManager.findAllResultsWithLevel(LevelHelper.ALERT, probeStatus.getIdentifier());
                            if (col.size() > 0) {
                                probeStatus.setStatus(LevelHelper.ALERT);
                            }
                            */
                            Integer count = dbManager.findResultsWithLevelCount(LevelHelper.ALERT, probeStatus.getIdentifier());
                            if ((count == null) || (count.intValue() > 0)) {
                                probeStatus.setStatus(LevelHelper.ALERT);

                                // modifying the status for the domain, instance, host, cluster 
                                instanceStatus.setStatus(LevelHelper.ALERT);
                                lc_menu11111.setTitle(instanceStatus.getNameAndStatus());

                                domainStatus.setStatus(LevelHelper.ALERT);
                                lc_menu1112.setTitle(domainStatus.getNameAndStatus());

                                hostStatus.setStatus(LevelHelper.ALERT);
                                lc_menu111.setTitle(hostStatus.getNameAndStatus());

                                cs.setStatus(LevelHelper.ALERT);
                                lc_menu11.setTitle(cs.toString());

                            }


                            lc_menu111111.setTitle(probeStatus.getNameAndStatus());
                            lc_menu11111.addMenuComponent(lc_menu111111);
                        }
                        lc_menu1112.addMenuComponent(lc_menu11111);
                    }
                    lc_menu1113.addMenuComponent(lc_menu1112);
                }
                lc_menu111.addMenuComponent(lc_menu1111);
                lc_menu11.addMenuComponent(lc_menu111);
                lc_menu111.addMenuComponent(lc_menu1113);
            }

        } else {
            lc_menu11.setTitle("NO CLuster Status Available");
        }
        lc_menu1.addMenuComponent(lc_menu11);
        LayoutUtils.addMenuIntoSession(request, lc_menu1);
    }

}
