/**
 * Copyright (C) 2005 - Bull S.A.
 *
 * Jimys - J2EE is monitoring your system
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id:  $
 * --------------------------------------------------------------------------
 */
package org.objectweb.jimys.webapp.visualizeDB.multipleChart;

import org.objectweb.jimys.dbConnection.session.DBManager;
import org.objectweb.jimys.dbConnection.session.DBManagerHome;
import org.objectweb.jimys.dbConnection.valueObjects.ResultProbeValueObject;
import org.jfree.chart.ChartFactory;
import org.jfree.chart.ChartUtilities;
import org.jfree.chart.JFreeChart;
import org.jfree.chart.axis.DateAxis;
import org.jfree.chart.axis.NumberAxis;
import org.jfree.chart.plot.PlotOrientation;
import org.jfree.data.xy.XYSeries;
import org.jfree.data.xy.XYSeriesCollection;

import javax.ejb.CreateException;
import javax.ejb.Handle;
import javax.naming.Context;
import javax.naming.InitialContext;
import javax.naming.NamingException;
import javax.rmi.PortableRemoteObject;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.awt.*;
import java.awt.geom.Rectangle2D;
import java.awt.image.BufferedImage;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.rmi.RemoteException;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;
import java.util.Properties;

/**
 *
 * ImageServlet
 *
 * @author Georges Goebel, Marc Calvisi
 */
public class ImageServlet extends HttpServlet {

    private static final String CONTENT_TYPE = "image/jpeg";

    public void doGet(HttpServletRequest request, HttpServletResponse response) throws ServletException, IOException {

        Collection selectedProbeList = (Collection) request.getSession().getAttribute("selectedProbeList0");
        Collection selectedProbeAttributeList = (Collection) request.getSession().getAttribute("selectedProbeAttributeList0");
        String begintime = (String) request.getSession().getAttribute("begintime");

        int counter = 1;

        Handle handle = (Handle) request.getSession().getAttribute("dbmanager");
        DBManager dbManager = null;
        if (handle == null) {
            dbManager = this.createDBManager();
            if (dbManager == null) {
                //return mapping.findForward("failure");
                System.out.println("ERROR CREATEING DBMANAGER");
                return;
            }
            request.getSession().setAttribute("dbmanager", dbManager.getHandle());

        } else {
            dbManager = (DBManager) handle.getEJBObject();
        }

        Collection xySeriesCollection = new ArrayList();

        while ((selectedProbeList != null) && (selectedProbeAttributeList != null) && (selectedProbeList.size() == selectedProbeAttributeList.size())) {

            XYSeriesCollection xyDSet = new XYSeriesCollection();

            Iterator it1 = selectedProbeList.iterator();
            Iterator it2 = selectedProbeAttributeList.iterator();

            while (it1.hasNext() && it2.hasNext()) {
                String para = (String) it2.next();
                String probeId = (String) it1.next();

                Collection col = null;
                if (begintime != null) {
                    col = dbManager.findAllAttributes(para, probeId, begintime);
                } else {
                    col = dbManager.findAllAttributes(para, probeId);
                }

                XYSeries series = new XYSeries(probeId+ " "+para);

                for (Iterator iterator = col.iterator(); iterator.hasNext();) {
                    ResultProbeValueObject resultProbeValueObject = (ResultProbeValueObject) iterator.next();
                    series.add(resultProbeValueObject.getTimestamp(), Double.valueOf(resultProbeValueObject.getAttributeValue()));
                }
                xyDSet.addSeries(series);
            }

            xySeriesCollection.add(xyDSet);

            selectedProbeList = (Collection) request.getSession().getAttribute("selectedProbeList"+counter);
            selectedProbeAttributeList = (Collection) request.getSession().getAttribute("selectedProbeAttributeList"+counter++);
        }


        int charts = xySeriesCollection.size();

        // limit to 4 charts
        BufferedImage g = new BufferedImage(640, 480, BufferedImage.TYPE_INT_RGB);
        switch (charts) {
            case 0 : {
                System.out.println("Error at least one chart ");
                return;
            }
            case 1 : {
                Rectangle2D r2D = new Rectangle2D.Double(0.0D, 0.0D, 640, 480);
                drawNewChartOnGraphic(0,g.createGraphics(), r2D, (XYSeriesCollection) xySeriesCollection.iterator().next());
                break;
            }
            case 2 : {
                Rectangle2D r2D = new Rectangle2D.Double(0.0D, 0.0D, 640, 240);
                BufferedImage gg = g.getSubimage(0,0,640,240);
                BufferedImage gg2 = g.getSubimage(0,240,640,240);
                Object[] series = xySeriesCollection.toArray();
                drawNewChartOnGraphic(0,gg.createGraphics(), r2D, (XYSeriesCollection) series[0]);
                drawNewChartOnGraphic(1,gg2.createGraphics(), r2D, (XYSeriesCollection) series[1]);
                break;
            }
            case 3 : {
                Rectangle2D r2D = new Rectangle2D.Double(0.0D, 0.0D, 640, 240);
                Rectangle2D r22D = new Rectangle2D.Double(0.0D, 0.0D, 320, 240);
                BufferedImage gg = g.getSubimage(0,0,640,240);
                BufferedImage gg2 = g.getSubimage(0,240,320,240);
                BufferedImage gg3 = g.getSubimage(320,240,320,240);
                Object[] series = xySeriesCollection.toArray();
                drawNewChartOnGraphic(0,gg.createGraphics(), r2D, (XYSeriesCollection) series[0]);
                drawNewChartOnGraphic(1,gg2.createGraphics(), r22D, (XYSeriesCollection) series[1]);
                drawNewChartOnGraphic(2,gg3.createGraphics(), r22D, (XYSeriesCollection) series[2]);
                break;
            }
            case 4 : {
                Rectangle2D r22D = new Rectangle2D.Double(0.0D, 0.0D, 320, 240);
                BufferedImage gg = g.getSubimage(0,0,320,240);
                BufferedImage gg2 = g.getSubimage(320,0,320,240);
                BufferedImage gg3 = g.getSubimage(0,240,320,240);
                BufferedImage gg4 = g.getSubimage(320,240,320,240);
                Object[] series = xySeriesCollection.toArray();
                drawNewChartOnGraphic(0,gg.createGraphics(), r22D, (XYSeriesCollection) series[0]);
                drawNewChartOnGraphic(1,gg2.createGraphics(), r22D, (XYSeriesCollection) series[1]);
                drawNewChartOnGraphic(2,gg3.createGraphics(), r22D, (XYSeriesCollection) series[2]);
                drawNewChartOnGraphic(3,gg4.createGraphics(), r22D, (XYSeriesCollection) series[3]);
                break;
            }
        }


        ByteArrayOutputStream baos = new ByteArrayOutputStream();
        ChartUtilities.writeBufferedImageAsJPEG(baos, g);
        //byte[] imageByte = ChartUtilities.encodeAsPNG(g);

        response.setContentType(CONTENT_TYPE);
        response.addHeader("Cache-Control", "No-cache; No-store; Must-revalidate; Proxy-revalidate");
        response.addDateHeader("Expires", 1L);
        response.addHeader("Pragma", "No-cache");

        OutputStream outstream = response.getOutputStream();
        outstream.write(baos.toByteArray());
        outstream.flush();
        outstream.close();

    }

    private void drawNewChartOnGraphic (int chartNumber,Graphics2D g2d, Rectangle2D r2D, XYSeriesCollection xyDSet) {
        DateAxis xAxis = new DateAxis(null);
        xAxis.setLowerMargin(0.0);
        xAxis.setUpperMargin(0.0);
        // Create the Y-Axis
        NumberAxis yAxis = new NumberAxis(null);
        yAxis.setAutoRangeIncludesZero(true);


        JFreeChart chart = ChartFactory.createXYLineChart("Title", "x-axis", "y-axis", xyDSet, PlotOrientation.VERTICAL, true, true, false);
        chart.setTitle("Chart"+chartNumber);
        chart.getXYPlot().setDomainAxis(xAxis);
        chart.getXYPlot().setRangeAxis(yAxis);

//        ChartRenderingInfo info = new ChartRenderingInfo(new StandardEntityCollection());

        chart.draw(g2d, r2D);
        chart.fireChartChanged();
    }




    private DBManager createDBManager() {
        InputStream config = Thread.currentThread().getContextClassLoader().getResourceAsStream("dbmanager.properties");
        Properties props = new Properties();
        try {
            props.load(config);
        } catch (IOException e1) {
            e1.printStackTrace();
        }
        String factory = "";
        String url = "";
        if (props.getProperty("database.java.naming.factory.initial") != null) {
            factory = props.getProperty("database.java.naming.factory.initial");
        }

        if (props.getProperty("database.java.naming.provider.url") != null) {
            url = props.getProperty("database.java.naming.provider.url");
        }
        Properties properties = new Properties();
        properties.put("java.naming.factory.initial", factory);
        properties.put("java.naming.provider.url", url);
        try {
            Context ictx = new InitialContext(properties);
            String ejbname = "";
            if (props.getProperty("database.remote.home") != null) {
                ejbname = props.getProperty("database.remote.home");
            }
            DBManagerHome dbHome = (DBManagerHome) PortableRemoteObject.narrow(ictx.lookup(ejbname), DBManagerHome.class);
            return dbHome.create();
        } catch (NamingException e) {
            e.printStackTrace();
        } catch (RemoteException e) {
            e.printStackTrace();
        } catch (CreateException e) {
            e.printStackTrace();
        }
        return null;

    }



}
