/**
 * Copyright (C) 2005 - Bull S.A.
 *
 * Jimys - J2EE is monitoring your system
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id:  $
 * --------------------------------------------------------------------------
 */
package org.objectweb.jimys.webapp.visualizeDB.ruleChart;

import org.objectweb.jimys.dbConnection.session.DBManager;
import org.objectweb.jimys.dbConnection.session.DBManagerHome;
import org.objectweb.jimys.dbConnection.valueObjects.ResultProbeValueObject;
import org.objectweb.jimys.dbConnection.valueObjects.ResultValueObject;
import org.jfree.chart.ChartFactory;
import org.jfree.chart.ChartUtilities;
import org.jfree.chart.JFreeChart;
import org.jfree.chart.ChartRenderingInfo;
import org.jfree.chart.entity.StandardEntityCollection;
import org.jfree.chart.axis.DateAxis;
import org.jfree.chart.axis.NumberAxis;
import org.jfree.chart.plot.PlotOrientation;
import org.jfree.data.xy.XYSeries;
import org.jfree.data.xy.XYSeriesCollection;

import javax.ejb.CreateException;
import javax.ejb.Handle;
import javax.naming.Context;
import javax.naming.InitialContext;
import javax.naming.NamingException;
import javax.rmi.PortableRemoteObject;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.awt.*;
import java.awt.geom.Rectangle2D;
import java.awt.image.BufferedImage;
import java.io.*;
import java.net.URL;
import java.rmi.RemoteException;
import java.util.*;

/**
 *
 * RuleImageServlet
 *
 * @author Georges Goebel, Marc Calvisi
 */
public class RuleImageServlet extends HttpServlet {
    private static final String CONTENT_TYPE = "image/jpeg";

    public void doGet(HttpServletRequest request, HttpServletResponse response) throws ServletException, IOException {

        String selectedRule = (String) request.getSession().getAttribute("selectedRule0");
        String selectedRuleAttribute = (String) request.getSession().getAttribute("selectedRuleAttribute0");
        String begintime = (String) request.getSession().getAttribute("ruleBegintime");

        int counter = 1;

        Handle handle = (Handle) request.getSession().getAttribute("dbmanager");
        DBManager dbManager = null;
        if (handle == null) {
            dbManager = this.createDBManager();
            if (dbManager == null) {
                //return mapping.findForward("failure");
                System.out.println("ERROR CREATEING DBMANAGER");
                return;
            }
            request.getSession().setAttribute("dbmanager", dbManager.getHandle());

        } else {
            dbManager = (DBManager) handle.getEJBObject();
        }

        Collection xySeriesCollection = new ArrayList();

        String[] names = new String[3];
        names[0] = "Chart0 : r1_"+selectedRule;

        while ((selectedRule != null) && (selectedRuleAttribute != null)) {

            XYSeriesCollection xyDSet = new XYSeriesCollection();

            String ruleSet = "r1";//selectedRule.substring(0, selectedRule.indexOf("_"));
            String ruleIdentifier = selectedRule;//selectedRule.substring((selectedRule.indexOf("_") + 1));

            String attribute = selectedRuleAttribute;

            Collection col = null;
            if (begintime != null) {
                System.out.println("Using new Methode ... ");
                //col = dbManager.monitorRule(ruleSet, ruleIdentifier, begintime);
                xyDSet = dbManager.getMonitorRuleChart(ruleSet, ruleIdentifier, attribute, begintime);

                xySeriesCollection.add(xyDSet);

            } else {
                col = dbManager.monitorRule(ruleSet, ruleIdentifier);

                //XYSeries series = new XYSeries(selectedRule + " " + attribute);
                Hashtable ht = new Hashtable();

                for (Iterator iterator = col.iterator(); iterator.hasNext();) {
                    ResultValueObject resultValueObject = (ResultValueObject) iterator.next();
                    Long timeStamp = resultValueObject.getTimeStamp();
                    Collection coll = resultValueObject.getResultProbes();
                    for (Iterator iterator1 = coll.iterator(); iterator1.hasNext();) {
                        ResultProbeValueObject resultProbeValueObject = (ResultProbeValueObject) iterator1.next();

                        String value = resultProbeValueObject.getAttributeValue();
                        String aName = resultProbeValueObject.getProbeIdentifier();

                        if (!attribute.equals(resultProbeValueObject.getAttributeName())) {
                            continue;
                        }

                        Hashtable values = (Hashtable) ht.get(aName);
                        if (values == null) values = new Hashtable();


                        values.put(resultProbeValueObject.getTimestamp(), value);

                        ht.put(aName, values);
                    }
                }

               // XYSeriesCollection xyDSet = new XYSeriesCollection();
                Enumeration en = ht.keys();
                while (en.hasMoreElements()) {
                    String key = (String) en.nextElement();
                    Hashtable values = (Hashtable) ht.get(key);

                    XYSeries series = new XYSeries(key);
                    Enumeration enu = values.keys();
                    while (enu.hasMoreElements()) {
                        Long vKey = (Long) enu.nextElement();
                        String value = (String) values.get(vKey);

                        series.add(vKey, Long.valueOf(value));
                    }
                    xyDSet.addSeries(series);
                }

                //xyDSet.addSeries(series);

                xySeriesCollection.add(xyDSet);


            }


            selectedRule = (String) request.getSession().getAttribute("selectedRule" + counter);
            names[counter] = "Chart"+counter+" : r1_"+(String) request.getSession().getAttribute("selectedRule"+counter);
            selectedRuleAttribute = (String) request.getSession().getAttribute("selectedRuleAttribute" + counter++);
        }


        int charts = xySeriesCollection.size();

        // limit to 4 charts
        BufferedImage g = new BufferedImage(640, 480, BufferedImage.TYPE_INT_RGB);
        switch (charts) {
            case 0 : {
                System.out.println("Error at least one chart ");
                return;
            }
            case 1 : {
                Rectangle2D r2D = new Rectangle2D.Double(0.0D, 0.0D, 640, 480);
                drawNewChartOnGraphic(names[0],g.createGraphics(), r2D, (XYSeriesCollection) xySeriesCollection.iterator().next());
                break;
            }
            case 2 : {
                Rectangle2D r2D = new Rectangle2D.Double(0.0D, 0.0D, 640, 240);
                BufferedImage gg = g.getSubimage(0, 0, 640, 240);
                BufferedImage gg2 = g.getSubimage(0, 240, 640, 240);
                Object[] series = xySeriesCollection.toArray();
                drawNewChartOnGraphic(names[0],gg.createGraphics(), r2D, (XYSeriesCollection) series[0]);
                drawNewChartOnGraphic(names[1],gg2.createGraphics(), r2D, (XYSeriesCollection) series[1]);
                break;
            }
            case 3 : {
                Rectangle2D r2D = new Rectangle2D.Double(0.0D, 0.0D, 640, 240);
                Rectangle2D r22D = new Rectangle2D.Double(0.0D, 0.0D, 320, 240);
                BufferedImage gg = g.getSubimage(0, 0, 640, 240);
                BufferedImage gg2 = g.getSubimage(0, 240, 320, 240);
                BufferedImage gg3 = g.getSubimage(320, 240, 320, 240);
                Object[] series = xySeriesCollection.toArray();
                drawNewChartOnGraphic(names[0],gg.createGraphics(), r2D, (XYSeriesCollection) series[0]);
                drawNewChartOnGraphic(names[1],gg2.createGraphics(), r22D, (XYSeriesCollection) series[1]);
                drawNewChartOnGraphic(names[2],gg3.createGraphics(), r22D, (XYSeriesCollection) series[2]);
                break;
            }
            case 4 : {
                Rectangle2D r22D = new Rectangle2D.Double(0.0D, 0.0D, 320, 240);
                BufferedImage gg = g.getSubimage(0, 0, 320, 240);
                BufferedImage gg2 = g.getSubimage(320, 0, 320, 240);
                BufferedImage gg3 = g.getSubimage(0, 240, 320, 240);
                BufferedImage gg4 = g.getSubimage(320, 240, 320, 240);
                Object[] series = xySeriesCollection.toArray();
                drawNewChartOnGraphic(names[0],gg.createGraphics(), r22D, (XYSeriesCollection) series[0]);
                drawNewChartOnGraphic(names[1],gg2.createGraphics(), r22D, (XYSeriesCollection) series[1]);
                drawNewChartOnGraphic(names[2],gg3.createGraphics(), r22D, (XYSeriesCollection) series[2]);
                drawNewChartOnGraphic(names[3],gg4.createGraphics(), r22D, (XYSeriesCollection) series[3]);
                break;
            }
        }


        ByteArrayOutputStream baos = new ByteArrayOutputStream();
        ChartUtilities.writeBufferedImageAsJPEG(baos, g);
        //byte[] imageByte = ChartUtilities.encodeAsPNG(g);

        response.setContentType(CONTENT_TYPE);
        response.addHeader("Cache-Control", "No-cache; No-store; Must-revalidate; Proxy-revalidate");
        response.addDateHeader("Expires", 1L);
        response.addHeader("Pragma", "No-cache");

        OutputStream outstream = response.getOutputStream();
        outstream.write(baos.toByteArray());
        outstream.flush();
        outstream.close();

    }

    private void drawNewChartOnGraphic(String title,Graphics2D g2d, Rectangle2D r2D, XYSeriesCollection xyDSet) {
        DateAxis xAxis = new DateAxis(null);
        xAxis.setLowerMargin(0.0);
        xAxis.setUpperMargin(0.0);
        // Create the Y-Axis
        NumberAxis yAxis = new NumberAxis(null);
        yAxis.setAutoRangeIncludesZero(true);


        JFreeChart chart = ChartFactory.createXYLineChart("Title", "x-axis", "y-axis", xyDSet, PlotOrientation.VERTICAL, true, true, false);
        chart.setTitle(title);
        chart.getXYPlot().setDomainAxis(xAxis);
        chart.getXYPlot().setRangeAxis(yAxis);

        // check if it works
        //ChartRenderingInfo info = new ChartRenderingInfo(new StandardEntityCollection());
        //info.setChartArea(r2D);
        //info.setPlotArea(r2D);

        //chart.draw(g2d, r2D, info);
        chart.draw(g2d, r2D);
    }


    private DBManager createDBManager() {
        URL config = Thread.currentThread().getContextClassLoader().getResource("dbmanager.properties");
        Properties props = new Properties();
        try {
            props.load(new FileInputStream(new File(config.getPath())));
        } catch (FileNotFoundException e1) {
            e1.printStackTrace();
        } catch (IOException e1) {
            e1.printStackTrace();
        }
        String factory = "";
        String url = "";
        if (props.getProperty("database.java.naming.factory.initial") != null) {
            factory = props.getProperty("database.java.naming.factory.initial");
        }

        if (props.getProperty("database.java.naming.provider.url") != null) {
            url = props.getProperty("database.java.naming.provider.url");
        }
        Properties properties = new Properties();
        properties.put("java.naming.factory.initial", factory);
        properties.put("java.naming.provider.url", url);
        try {
            Context ictx = new InitialContext(properties);
            String ejbname = "";
            if (props.getProperty("database.remote.home") != null) {
                ejbname = props.getProperty("database.remote.home");
            }
            DBManagerHome dbHome = (DBManagerHome) PortableRemoteObject.narrow(ictx.lookup(ejbname), DBManagerHome.class);
            return dbHome.create();
        } catch (NamingException e) {
            e.printStackTrace();
        } catch (RemoteException e) {
            e.printStackTrace();
        } catch (CreateException e) {
            e.printStackTrace();
        }
        return null;

    }


}
