/*
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 1999 Bull S.A.
 * Contact: jonas-team@objectweb.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: $
 * --------------------------------------------------------------------------
 */

package org.objectweb.jonas.webapp.admin;

import java.net.URLEncoder;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;

import org.apache.struts.action.Action;
import org.apache.struts.action.ActionForm;
import org.apache.struts.action.ActionForward;
import org.apache.struts.action.ActionMapping;
import org.objectweb.jonas.webapp.common.WebappContextSession;
import org.objectweb.jonas.webapp.taglib.TreeControl;
import org.objectweb.jonas.webapp.taglib.TreeControlNode;

/**
 * Test <code>Action</code> that handles events from the tree control test
 * page.
 *
 * @author Craig R. McClanahan
 * @author Michel-Ange ANTON
 */

public class ViewTreeAction extends Action {

// --------------------------------------------------------- Public Methods

    /**
     * Process the specified HTTP request, and create the corresponding HTTP
     * response (or forward to another web component that will create it).
     * Return an <code>ActionForward</code> instance describing where and how
     * control should be forwarded, or <code>null</code> if the response has
     * already been completed.
     *
     * @param mapping The ActionMapping used to select this instance
     * @param actionForm The optional ActionForm bean for this request (if any)
     * @param request The HTTP request we are processing
     * @param response The HTTP response we are creating
     *
     * @exception IOException if an input/output error occurs
     * @exception ServletException if a servlet exception occurs
     */
    public ActionForward execute(ActionMapping mapping, ActionForm form, HttpServletRequest request
        , HttpServletResponse response)
        throws Exception {

        String name = null;
        HttpSession session = request.getSession();
        WebappContextSession oWhere = (WebappContextSession) session.getAttribute(
            WebappContextSession.SESSION_NAME);
        // Detect if a WhereAreYou instance is present
        if (oWhere == null) {
            // Re-init
            getServlet().log("Instance WhereAreYou not found in session : re-init");
            return (mapping.findForward("Main Index"));
        }
        // Get the tree
        TreeControl control = oWhere.getTreeControl();
        // Handle a tree expand/contract event
        name = request.getParameter("tree");
        if (name != null) {
            getServlet().log("Tree expand/contract on " + name);
            TreeControlNode node = control.findNode(name);
            if (node != null) {
                getServlet().log("Found Node: " + name + "[" + node.getLabel() + "]");
                node.setExpanded(!node.isExpanded());
                // Add the anchor
                request.setAttribute("anchorToGo", URLEncoder.encode(name, "UTF-8"));
            }
        }
        else {
            getServlet().log("tree param is null");
        }

        // Handle a select item event
        name = request.getParameter("select");
        if (name != null) {
            getServlet().log("Select event on " + name);
            control.selectNode(name);
            // Add the anchor
            request.setAttribute("anchorToGo", URLEncoder.encode(name, "UTF-8"));
        }

        // Forward back to the test page
        return (mapping.findForward("Tree"));
    }
}
