/*
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 1999 Bull S.A.
 * Contact: jonas-team@objectweb.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: $
 * --------------------------------------------------------------------------
 */

package org.objectweb.jonas.webapp.admin.examples;

import org.apache.struts.Globals;
import org.apache.struts.action.ActionServlet;
import org.apache.struts.util.MessageResources;
import org.objectweb.jonas.webapp.taglib.TreeBuilder;
import org.objectweb.jonas.webapp.taglib.TreeControl;
import org.objectweb.jonas.webapp.taglib.TreeControlNode;

import javax.servlet.http.HttpServletRequest;

/**
 * Create all the node for the example's tree.
 *
 * @author Michel-Ange ANTON
 */
public class ExampleTreeBuilder implements TreeBuilder {

// ----------------------------------------------------- Instance Variables

// ---------------------------------------------------- TreeBuilder Methods

    /**
     * Add the required nodes to the specified <code>treeControl</code>
     * instance.
     *
     * @param p_TreeControl The <code>TreeControl</code> to which we should add our nodes
     * @param p_Servlet The controller servlet for the admin application
     * @param p_Request The servlet request we are processing
     */
    public void buildTree(TreeControl p_TreeControl, ActionServlet p_Servlet
        , HttpServletRequest p_Request) {
        try {
            TreeControlNode oHiddenRootNode = p_TreeControl.getRoot();
            MessageResources oResources = ((MessageResources) p_Request.getAttribute(Globals.
                MESSAGES_KEY));

            // Add children nodes
            addAdminBlankNodes(oHiddenRootNode, oResources, p_Request);
            //addAutomaticNodes(oHiddenRootNode, oResources, p_Request);
        }
        catch (Throwable t) {
            t.printStackTrace(System.out);
        }
    }

// ------------------------------------------------------ Protected Methods

    /**
     * Append nodes for the Admin Blank.
     *
     * @param p_RootNode Root node for the tree control
     * @param p_Resources The MessageResources for our localized messages
     * @param p_Request The servlet request we are processing
     *
     * @exception Exception if an exception occurs building the tree
     */
    protected void addAdminBlankNodes(TreeControlNode p_RootNode, MessageResources p_Resources
        , HttpServletRequest p_Request)
        throws Exception {

        String sName = "adminBlankNode";
        TreeControlNode adminBlankNode = new TreeControlNode(sName, "icon/war.gif"
            , "Administration", null, "content", true);
        p_RootNode.addChild(adminBlankNode);

        // Add children nodes
    //    addServerNodes(adminBlankNode, p_Resources, p_Request);
    //    addJvmPropertiesNodes(adminBlankNode, p_Resources, p_Request);
        addJimysPropertiesNodes(adminBlankNode, p_Resources, p_Request);
    }

    /**
     * Append nodes for the server.
     *
     * @param p_ParentNode Parent node where the new is adding
     * @param p_Resources The MessageResources for our localized messages
     * @param p_Request The servlet request we are processing
     *
     * @exception Exception if an exception occurs building the tree
     */
    protected void addServerNodes(TreeControlNode p_ParentNode, MessageResources p_Resources
        , HttpServletRequest p_Request)
        throws Exception {

        // Server
        String sName = "serverNode";
        TreeControlNode serverNode = new TreeControlNode(sName, "icon/BalJonasTree.gif"
            , p_Resources.getMessage(p_Request.getLocale(), "treenode.server"), "ViewServer.do"
            , "content", true);
        p_ParentNode.addChild(serverNode);

        // JMX
        sName = "serverJmxNode";
        TreeControlNode node = new TreeControlNode(sName, "icon/bean.gif"
            , p_Resources.getMessage(p_Request.getLocale(), "treenode.server.jmx")
            , "ViewServerJmx.do", "content", true);
        serverNode.addChild(node);

        // USER
        sName = "serverUserNode";
        node = new TreeControlNode(sName, "icon/bean.gif"
            , p_Resources.getMessage(p_Request.getLocale(), "treenode.server.user")
            , "ViewServerUser.do", "content", true);
        serverNode.addChild(node);


    }


    /**
     * Append nodes for the JVM Properties.
     *
     * @param p_ParentNode Parent node where the new is adding
     * @param p_Resources The MessageResources for our localized messages
     * @param p_Request The servlet request we are processing
     *
     * @exception Exception if an exception occurs building the tree
     */
    protected void addJimysPropertiesNodes(TreeControlNode p_ParentNode, MessageResources p_Resources
        , HttpServletRequest p_Request)
        throws Exception {

        String sName = "Jimys";
        TreeControlNode serverNode = new TreeControlNode(sName, "icon/BalJonasTree.gif"
            , p_Resources.getMessage(p_Request.getLocale(), "treenode.jimys"), "jimys.jsp"
            , "content", true);
        p_ParentNode.addChild(serverNode);


         // confguration
        sName = "configuration";
        TreeControlNode confnode = new TreeControlNode(sName, "icon/bean.gif"
            , p_Resources.getMessage(p_Request.getLocale(), "treenode.jimys.configuration")
            , "configuration.jsp", "content", true);
        serverNode.addChild(confnode);

        // Mediator
        sName = "mediator";
        TreeControlNode node = new TreeControlNode(sName, "icon/bean.gif"
            , p_Resources.getMessage(p_Request.getLocale(), "treenode.jimys.mediator")
            , "init.do", "content", true);
        confnode.addChild(node);

        // ADEngine
        sName = "ruleEngine";
        node = new TreeControlNode(sName, "icon/bean.gif"
            , p_Resources.getMessage(p_Request.getLocale(), "treenode.jimys.ruleEngine")
            , "engineInit.do", "content", true);
        confnode.addChild(node);

        // Monitoring
        sName = "monitoring";
        TreeControlNode mnode = new TreeControlNode(sName, "icon/bean.gif"
            , p_Resources.getMessage(p_Request.getLocale(), "treenode.jimys.monitoring")
            , "monitoring.jsp", "content", true);
        serverNode.addChild(mnode);

        sName = "monitoring.showAll";
        TreeControlNode cNode = new TreeControlNode(sName, "icon/bean.gif"
            , p_Resources.getMessage(p_Request.getLocale(), "treenode.jimys.monitoring.showAll")
            , "showAll.do", "content", true);
        mnode.addChild(cNode);

        sName = "monitoring.clusterStatus";
        cNode = new TreeControlNode(sName, "icon/bean.gif"
            , p_Resources.getMessage(p_Request.getLocale(), "treenode.jimys.monitoring.clusterStatus")
            , "clusterStatus.do", "content", true);
        mnode.addChild(cNode);

        sName = "monitoring.monitorRule";
        cNode = new TreeControlNode(sName, "icon/bean.gif"
            , p_Resources.getMessage(p_Request.getLocale(), "treenode.jimys.monitoring.monitorRule")
            , "selectMonitorRule.do", "content", true);
        mnode.addChild(cNode);

        /*
        sName = "monitoring.monitorRuleChart";
        cNode = new TreeControlNode(sName, "icon/bean.gif"
            , p_Resources.getMessage(p_Request.getLocale(), "treenode.jimys.monitoring.monitorRuleChart")
            , "selectRule.do", "content", true);
        mnode.addChild(cNode);

        sName = "monitoring.monitorProbeChart";
        cNode = new TreeControlNode(sName, "icon/bean.gif"
            , p_Resources.getMessage(p_Request.getLocale(), "treenode.jimys.monitoring.monitorProbeChart")
            , "selectProbe.do", "content", true);
        mnode.addChild(cNode);
          */
        sName = "monitoring.monitorProbeChart";
        cNode = new TreeControlNode(sName, "icon/bean.gif"
            , p_Resources.getMessage(p_Request.getLocale(), "treenode.jimys.monitoring.monitorProbeChart")
            , "getAvailableProbes.do", "content", true);
        mnode.addChild(cNode);

        sName = "monitoring.monitorRuleChart";
        cNode = new TreeControlNode(sName, "icon/bean.gif"
            , p_Resources.getMessage(p_Request.getLocale(), "treenode.jimys.monitoring.monitorRuleChart")
            , "getAvailableRules.do", "content", true);
        mnode.addChild(cNode);

        sName = "monitoring.liveclusterStatus";
        cNode = new TreeControlNode(sName, "icon/bean.gif"
            , p_Resources.getMessage(p_Request.getLocale(), "treenode.jimys.monitoring.liveclusterStatus")
            , "liveCheck.do", "content", true);
        mnode.addChild(cNode);

        sName = "monitoring.cleanDB";
        cNode = new TreeControlNode(sName, "icon/bean.gif"
            , p_Resources.getMessage(p_Request.getLocale(), "treenode.jimys.monitoring.cleanDB")
            , "cleanupDB.jsp", "content", true);
        mnode.addChild(cNode);

        // Alarm
        sName = "alarm";
        TreeControlNode anode = new TreeControlNode(sName, "icon/bean.gif"
            , p_Resources.getMessage(p_Request.getLocale(), "treenode.jimys.alarm")
            , "showAlarms.do", "content", true);
        serverNode.addChild(anode);

         //cartography
        sName = "cartography";
        TreeControlNode catnode = new TreeControlNode(sName, "icon/bean.gif"
            , p_Resources.getMessage(p_Request.getLocale(), "treenode.jimys.cartography")
            , "initCarto.do", "content", true);
        serverNode.addChild(catnode);

    }


    /**
     * Append nodes for the JVM Properties.
     *
     * @param p_ParentNode Parent node where the new is adding
     * @param p_Resources The MessageResources for our localized messages
     * @param p_Request The servlet request we are processing
     *
     * @exception Exception if an exception occurs building the tree
     */
    protected void addJvmPropertiesNodes(TreeControlNode p_ParentNode, MessageResources p_Resources
        , HttpServletRequest p_Request)
        throws Exception {

        String sName = "jvmPropertiesNode";
        TreeControlNode node = new TreeControlNode(sName, "icon/loggers.gif"
            , p_Resources.getMessage(p_Request.getLocale(), "treenode.jvm.properties")
            , "ListJvmProperties.do", "content", true);
        p_ParentNode.addChild(node);

    }

    /**
     * Append automatic nodes.
     *
     * @param p_ParentNode Parent node for the tree control
     * @param p_Resources The MessageResources for our localized messages
     * @param p_Request The servlet request we are processing
     *
     * @exception Exception if an exception occurs building the tree
     */
    protected void addAutomaticNodes(TreeControlNode p_ParentNode, MessageResources p_Resources
        , HttpServletRequest p_Request)
        throws Exception {

        // Automatic node
        TreeControlNode autoNode = new TreeControlNode("automatic", "icon/services.gif"
            , p_Resources.getMessage(p_Request.getLocale(), "treenode.automatic"), null, "content", true);
        p_ParentNode.addChild(autoNode);

        // Random node
        TreeControlNode randomNode = new TreeControlNode("random", "icon/deploy.gif"
            , p_Resources.getMessage(p_Request.getLocale(), "treenode.random"), "ViewBlank.do"
            , "content", true);
        autoNode.addChild(randomNode);

        // Add children Random node
        addRandomNodes(randomNode, 5, 3, 0);

    }

    /**
     * Append random nodes.
     *
     * @param p_ParentNode Parent node for the tree control
     * @param p_MaxLength The number maximum of children nodes in a parent node
     * @param p_MaxDepth The number maximum of nodes between the root node and the last leaf node
     * @param p_CurrentDepth Used to control the depth with the recursive call. At the beginning, it should be to 0 at the first call.
     *
     * @exception Exception if an exception occurs building the tree
     */
    protected void addRandomNodes(TreeControlNode p_ParentNode, int p_MaxLength, int p_MaxDepth
        , int p_CurrentDepth)
        throws Exception {

        String sName;
        String sLabel;
        String sIcon;
        TreeControlNode node;

        // Avoid overflow
        if (p_CurrentDepth < p_MaxDepth) {
            // Calculate the number of new nodes
            int iLength = (int) Math.round(Math.random() * p_MaxLength);
            // Loop to make the node
            for (int i = 0; i < iLength; i++) {
                // Build ID name and label
                sName = p_ParentNode.getName() + "_" + String.valueOf(i);
                sLabel = p_ParentNode.getLabel() + " " + String.valueOf(i);
                // Choice icon picture
                if ((i % 2) == 0) {
                    sIcon = "icon/deployed.gif";
                }
                else {
                    sIcon = "icon/deployed_not.gif";
                }
                // Make node
                node = new TreeControlNode(sName, sIcon, sLabel, "ViewBlank.do?node=" + sName
                    , "content", false);
                // Add node in tree
                p_ParentNode.addChild(node);
                // Add new children nodes
                addRandomNodes(node, p_MaxLength, p_MaxDepth, p_CurrentDepth + 1);
            }
        }
    }

    /**
     * Append nodes for Monitoring.
     *
     * @param p_ServerNode Server node for the tree control
     * @param p_Resources The MessageResources for our localized messages
     *  messages
     *
     * @exception Exception if an exception occurs building the tree
     */
    protected void getMonitoring(TreeControlNode p_ServerNode, MessageResources p_Resources)
        throws Exception {
        /*
                 // Monitoring server Service node
                 TreeControlNode node = new TreeControlNode(p_ServerNode.getName()
            + WhereAreYou.NODE_SEPARATOR + "monitoring", "icon/monitoring.gif"
            , p_Resources.getMessage("treenode.jonas.server.monitoring"), "EditMonitoring.do"
            , "content", false);
                 p_ServerNode.addChild(node);
         */
    }

    /**
     * Append nodes for Logging.
     *
     * @param p_ServerNode Server node for the tree control
     * @param p_Resources The MessageResources for our localized messages
     * @param p_Request The servlet request we are processing
     *
     * @exception Exception if an exception occurs building the tree
     */
    protected void getLogging(TreeControlNode p_ServerNode, MessageResources p_Resources
        , HttpServletRequest p_Request)
        throws Exception {
        /*
                 // Logging server Service node
                 TreeControlNode node = new TreeControlNode(p_ServerNode.getName()
            + WhereAreYou.NODE_SEPARATOR + "logging", "icon/loggers.gif"
             , p_Resources.getMessage("treenode.jonas.server.logging"), "ListLoggers.do", "content", false);
                 p_ServerNode.addChild(node);
                 // Add all loggers
                 getLoggers(node, p_Resources, p_Request);
         */
    }

    /**
     * Append nodes for each logger.
     *
     * @param p_LoggingNode Server node for the tree control
     * @param p_Resources The MessageResources for our localized messages
     * @param p_Request The servlet request we are processing
     *
     * @exception Exception if an exception occurs building the tree
     */
    public void getLoggers(TreeControlNode p_LoggingNode, MessageResources p_Resources
        , HttpServletRequest p_Request)
        throws Exception {
        /*
                 // Get WhereAreYou instance in session
             WhereAreYou oWhere = (WhereAreYou) p_Request.getSession().getAttribute(WhereAreYou.
            SESSION_NAME);
                 // Get list of loggers with Action name
                 ArrayList al = JonasAdminJmx.getLoggers(p_Resources, oWhere, true);
                 // Build the node of each logger
                 LoggerItem oLog;
                 String sIcon = "icon/loggers.gif";
                 for (int i = 0; i < al.size(); i++) {
            oLog = (LoggerItem) al.get(i);
            if (LoggerItem.LOGGER_JONAS.equals(oLog.getType()) == true) {
                sIcon = "icon/logger_jonas.gif";
            }
            else if (LoggerItem.LOGGER_CATALINA_ACCESS.equals(oLog.getType()) == true) {
                sIcon = "icon/logger_http.gif";
            }
            // Add a logger node
            TreeControlNode node = new TreeControlNode(p_LoggingNode.getName()
                + WhereAreYou.NODE_SEPARATOR + oLog.getType(), sIcon, oLog.getName()
                , oLog.getForward(), "content", false);
            p_LoggingNode.addChild(node);
                 }
         */
    }

}
