/*
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 1999 Bull S.A.
 * Contact: jonas-team@objectweb.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: $
 * --------------------------------------------------------------------------
 */

package org.objectweb.jonas.webapp.admin.mbean;

import java.net.URLEncoder;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Iterator;

import javax.management.MBeanAttributeInfo;
import javax.management.MBeanInfo;
import javax.management.MBeanOperationInfo;
import javax.management.MBeanServer;
import javax.management.ObjectName;
import javax.servlet.http.HttpServletRequest;

import org.apache.struts.Globals;
import org.apache.struts.action.ActionServlet;
import org.apache.struts.util.MessageResources;
import org.objectweb.jonas.webapp.common.BeanComparator;
import org.objectweb.jonas.webapp.common.WebappContextSession;
import org.objectweb.jonas.webapp.taglib.TreeBuilder;
import org.objectweb.jonas.webapp.taglib.TreeControl;
import org.objectweb.jonas.webapp.taglib.TreeControlNode;

/**
 * Create the tree to see all MBeans used in the current server.
 *
 * @author Michel-Ange ANTON
 */

public class MBeanTreeBuilder implements TreeBuilder {
// ----------------------------------------------------- Instance Variables

// ---------------------------------------------------- TreeBuilder Methods

    /**
     * Add the required nodes to the specified <code>treeControl</code>
     * instance.
     *
     * @param treeControl The <code>TreeControl</code> to which we should
     *  add our nodes
     * @param servlet The controller servlet for the admin application
     * @param request The servlet request we are processing
     */
    public void buildTree(TreeControl p_TreeControl, ActionServlet p_Servlet
        , HttpServletRequest p_Request) {
        try {
            TreeControlNode oRootNode = p_TreeControl.getRoot();
            MessageResources oResources = ((MessageResources) p_Request.getAttribute(Globals.
                MESSAGES_KEY));
            WebappContextSession oWhere = (WebappContextSession) p_Request.getSession().
                getAttribute(WebappContextSession.SESSION_NAME);
            MBeanServer oMBeanServer = oWhere.getMBeanServer();
            if (null != oMBeanServer) {
                getMBeans(oRootNode, oMBeanServer, oResources);
            }
        }
        catch (Throwable t) {
            t.printStackTrace(System.out);
        }
    }

// ------------------------------------------------------ Protected Methods

    /**
     * Append nodes for all defined MBeans.
     *
     * @param p_RootNode Root node for the tree control
     * @param p_MBeanServer The current MBean server
     * @param p_Resources The MessageResources for our localized messages
     *
     * @exception Exception if an exception occurs building the tree
     */
    public void getMBeans(TreeControlNode p_RootNode, MBeanServer p_MBeanServer
        , MessageResources p_Resources)
        throws Exception {

        TreeControlNode oNodeMBeans = new TreeControlNode("mbeans", "icon/mbeans.gif"
            , p_Resources.getMessage("treenode.allmbeans"), "ListMBeans.do", "content", false);
        p_RootNode.addChild(oNodeMBeans);

        ArrayList[] als = MBeanUtils.getFamiliesMBeanItemsLists(p_MBeanServer);
        if (als[MBeanItem.FAMILY_J2EE].size() > 0) {
            getJ2eeMBeans(oNodeMBeans, p_Resources, als[MBeanItem.FAMILY_J2EE]);
        }
        if (als[MBeanItem.FAMILY_OWNER].size() > 0) {
            getOwnerMBeans(oNodeMBeans, p_Resources, als[MBeanItem.FAMILY_OWNER]);
        }
        if (als[MBeanItem.FAMILY_UNKNOWN].size() > 0) {
            getUnknownMBeans(oNodeMBeans, p_Resources, als[MBeanItem.FAMILY_UNKNOWN]);
        }
    }

    /**
     * Append nodes for all defined Owner MBeans.
     *
     * @param p_ParentNode Parent node for the tree control
     * @param p_Resources The MessageResources for our localized messages
     * @param p_List The list of Mbeans
     *
     * @exception Exception if an exception occurs building the tree
     */
    public void getOwnerMBeans(TreeControlNode p_ParentNode, MessageResources p_Resources
        , ArrayList p_List)
        throws Exception {

        TreeControlNode nodeMBeans = new TreeControlNode("mbeans"
            + WebappContextSession.NODE_SEPARATOR + "owner", "icon/mbeans.gif"
            , p_Resources.getMessage("treenode.mbeans.owner"), "ListOwnerMBeans.do", "content", false);
        p_ParentNode.addChild(nodeMBeans);
        // Sort list
        Collections.sort(p_List, new BeanComparator(new String[] {
            "domain", "objectName"}));

        // Detail MBean node
        String sLastDomain = "";
        String sLastType = "";
        TreeControlNode nodeMBean = null;
        TreeControlNode nodeDomain = null;
        TreeControlNode nodeType = null;
        TreeControlNode nodeParent = null;
        OwnerMBeanItem oItem;

        Iterator it = p_List.iterator();
        while (it.hasNext()) {
            oItem = (OwnerMBeanItem) it.next();

            nodeParent = nodeMBeans;
            if (oItem.getDomain() != null) {
                // Domain node
                if (oItem.getDomain().equals(sLastDomain) == false) {
                    sLastDomain = oItem.getDomain();
                    nodeDomain = new TreeControlNode(nodeParent.getName()
                        + WebappContextSession.NODE_SEPARATOR + oItem.getDomain()
                        , "icon/mbeandomain.gif", oItem.getDomain(), null, "content", false);
                    nodeMBeans.addChild(nodeDomain);
                }
                nodeParent = nodeDomain;

                if (oItem.getType() != null) {
                    // Type node
                    if (oItem.getType().equals(sLastType) == false) {
                        sLastType = oItem.getType();
                        nodeType = new TreeControlNode(nodeParent.getName()
                            + WebappContextSession.NODE_SEPARATOR + oItem.getType()
                            , "icon/mbeantype.gif", oItem.getType(), null, "content", false);
                        nodeDomain.addChild(nodeType);
                    }
                    nodeParent = nodeType;
                }
            }

            // Mbean node
            nodeMBean = new TreeControlNode("mbeans" + WebappContextSession.NODE_SEPARATOR
                + oItem.getObjectName(), "icon/mbean.gif", oItem.getObjectName()
                , "ListMBeanDetails.do?select=" + URLEncoder.encode(oItem.getObjectName(), "UTF-8")
                , "content", false);
            nodeParent.addChild(nodeMBean);

        }
    }

    /**
     * Append nodes for all defined J2EE MBeans.
     *
     * @param p_ParentNode Parent node for the tree control
     * @param p_Resources The MessageResources for our localized messages
     * @param p_List The list of Mbeans
     *
     * @exception Exception if an exception occurs building the tree
     */
    public void getJ2eeMBeans(TreeControlNode p_ParentNode, MessageResources p_Resources
        , ArrayList p_List)
        throws Exception {

        TreeControlNode nodeMBeans = new TreeControlNode("mbeans"
            + WebappContextSession.NODE_SEPARATOR + "j2ee", "icon/mbeans.gif"
            , p_Resources.getMessage("treenode.mbeans.j2ee"), "ListJ2eeMBeans.do", "content", false);
        p_ParentNode.addChild(nodeMBeans);
        // Sort list
        Collections.sort(p_List, new BeanComparator(new String[] {
            "domain", "objectName"}));

        // Detail MBean node
        J2eeMBeanItem oItem;
        String sLastDomain = "";
        String sLastType = "";
        TreeControlNode nodeMBean = null;
        TreeControlNode nodeDomain = null;
        TreeControlNode nodeType = null;
        TreeControlNode nodeParent = null;

        Iterator it = p_List.iterator();
        while (it.hasNext()) {
            oItem = (J2eeMBeanItem) it.next();

            nodeParent = nodeMBeans;
            if (oItem.getDomain() != null) {
                // Domain node
                if (oItem.getDomain().equals(sLastDomain) == false) {
                    sLastDomain = oItem.getDomain();
                    nodeDomain = new TreeControlNode(nodeParent.getName()
                        + WebappContextSession.NODE_SEPARATOR + oItem.getDomain()
                        , "icon/mbeandomain.gif", oItem.getDomain(), null, "content", false);
                    nodeMBeans.addChild(nodeDomain);
                }
                nodeParent = nodeDomain;
            }
            if (oItem.getJ2eeType() != null) {
                // Type node
                if (oItem.getJ2eeType().equals(sLastType) == false) {
                    sLastType = oItem.getJ2eeType();
                    nodeType = new TreeControlNode(nodeParent.getName()
                        + WebappContextSession.NODE_SEPARATOR + oItem.getJ2eeType()
                        , "icon/mbeantype.gif", oItem.getJ2eeType(), null, "content", false);
                    nodeDomain.addChild(nodeType);
                }
                nodeParent = nodeType;
            }

            // Mbean node
            nodeMBean = new TreeControlNode("mbeans" + WebappContextSession.NODE_SEPARATOR
                + oItem.getObjectName(), "icon/mbean.gif", oItem.getObjectName()
                , "ListMBeanDetails.do?select=" + URLEncoder.encode(oItem.getObjectName(), "UTF-8")
                , "content", false);
            nodeParent.addChild(nodeMBean);
        }
    }

    /**
     * Append nodes for all defined Unknown MBeans.
     *
     * @param p_ParentNode Parent node for the tree control
     * @param p_Resources The MessageResources for our localized messages
     * @param p_List The list of Mbeans
     *
     * @exception Exception if an exception occurs building the tree
     */
    public void getUnknownMBeans(TreeControlNode p_ParentNode, MessageResources p_Resources
        , ArrayList p_List)
        throws Exception {

        TreeControlNode nodeMBeans = new TreeControlNode("mbeans"
            + WebappContextSession.NODE_SEPARATOR + "unknown", "icon/mbeans.gif"
            , p_Resources.getMessage("treenode.mbeans.unknown"), "ListUnknownMBeans.do", "content", false);
        p_ParentNode.addChild(nodeMBeans);
        // Sort list
        Collections.sort(p_List, new BeanComparator(new String[] {
            "domain", "objectName"}));

        // Detail MBean node
        MBeanItem oItem;
        String sLastDomain = "";
        TreeControlNode nodeMBean = null;
        TreeControlNode nodeDomain = null;
        TreeControlNode nodeType = null;
        TreeControlNode nodeParent = null;

        Iterator it = p_List.iterator();
        while (it.hasNext()) {
            oItem = (MBeanItem) it.next();

            nodeParent = nodeMBeans;
            if (oItem.getDomain() != null) {
                // Domain node
                if (oItem.getDomain().equals(sLastDomain) == false) {
                    sLastDomain = oItem.getDomain();
                    nodeDomain = new TreeControlNode(nodeParent.getName()
                        + WebappContextSession.NODE_SEPARATOR + oItem.getDomain()
                        , "icon/mbeandomain.gif", oItem.getDomain(), null, "content", false);
                    nodeMBeans.addChild(nodeDomain);
                }
                nodeParent = nodeDomain;

            }

            // Mbean node
            nodeMBean = new TreeControlNode("mbeans" + WebappContextSession.NODE_SEPARATOR
                + oItem.getObjectName(), "icon/mbean.gif", oItem.getObjectName()
                , "ListMBeanDetails.do?select=" + URLEncoder.encode(oItem.getObjectName(), "UTF-8")
                , "content", false);
            nodeParent.addChild(nodeMBean);

        }
    }

    /**
     * Append nodes Attributes and Operations for a defined MBean.
     *
     * @param p_ParentNode The MBean node
     * @param p_MBeanServer The current MBean server
     * @param p_ObjectName The MBean Object name
     * @param p_Resources Resource
     *
     * @throws Exception
     */

    protected void getMBeanInfo(TreeControlNode p_ParentNode, MBeanServer p_MBeanServer
        , ObjectName p_ObjectName, MessageResources p_Resources)
        throws Exception {
        // Get all infos of a MBean
        MBeanInfo oMBeanInfo = p_MBeanServer.getMBeanInfo(p_ObjectName);
        // Get attributes infos
        MBeanAttributeInfo[] aoMBeanAttributeInfo = oMBeanInfo.getAttributes();
        if (aoMBeanAttributeInfo.length > 0) {
            // Append attributes node
            TreeControlNode nodeAttributes = new TreeControlNode("Attributes"
                + WebappContextSession.NODE_SEPARATOR + p_ObjectName.toString()
                , "icon/JonasQuestion.gif", p_Resources.getMessage("treenode.allmbeans.attributes"), null
                , "content", false);
            p_ParentNode.addChild(nodeAttributes);
            // Loop to append each attribute node
            for (int i = 0; i < aoMBeanAttributeInfo.length; i++) {
                TreeControlNode nodeAttr = new TreeControlNode(String.valueOf(i)
                    + WebappContextSession.NODE_SEPARATOR + p_ObjectName.toString()
                    + WebappContextSession.NODE_SEPARATOR + aoMBeanAttributeInfo[i].getName()
                    , "icon/property.gif", aoMBeanAttributeInfo[i].getName(), null, "content", false);
                nodeAttributes.addChild(nodeAttr);
            }
        }
        // Get operations infos
        MBeanOperationInfo[] aoMBeanOperationInfo = oMBeanInfo.getOperations();
        if (aoMBeanOperationInfo.length > 0) {
            // Append operations node
            TreeControlNode nodeOperations = new TreeControlNode("Operations"
                + WebappContextSession.NODE_SEPARATOR + p_ObjectName.toString()
                , "icon/JonasQuestion.gif", p_Resources.getMessage("treenode.allmbeans.operations"), null
                , "content", false);
            p_ParentNode.addChild(nodeOperations);
            // Loop to append each operation node
            for (int i = 0; i < aoMBeanOperationInfo.length; i++) {
                TreeControlNode nodeOpe = new TreeControlNode(String.valueOf(i)
                    + WebappContextSession.NODE_SEPARATOR + p_ObjectName.toString()
                    + WebappContextSession.NODE_SEPARATOR + aoMBeanOperationInfo[i].getName()
                    , "icon/action.gif", aoMBeanOperationInfo[i].getName(), null, "content", false);
                nodeOperations.addChild(nodeOpe);
            }
        }
    }

}
