/*
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 1999 Bull S.A.
 * Contact: jonas-team@objectweb.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id:  $
 * --------------------------------------------------------------------------
 */

package org.objectweb.jonas.webapp.admin.mbean;

import java.util.ArrayList;
import java.util.Collections;
import java.util.Iterator;
import java.util.List;
import java.util.Set;

import javax.management.MBeanServer;
import javax.management.ObjectName;

/**
 * @author Michel-Ange ANTON
 */
public class MBeanUtils {

// ----------------------------------------------------------- Constants

    private static Object s_Synchro = new Object();

// --------------------------------------------------------- Public Methods

    /**
     * Verify if the Mbean gotten by the query in the current MbeanServer exists.
     *
     * @param p_On Query Mbean name to search
     * @return true if MBean exists
     * @throws ManagementException
     */
    /*
        public static boolean hasMBeanName(ObjectName p_On)
            throws ManagementException {
            synchronized (s_Synchro) {
                ArrayList al = new ArrayList();
                Iterator itNames = JonasManagementRepr.queryNames(p_On).iterator();
                if (itNames.hasNext()) {
                    return true;
                }
                return false;
            }
        }
     */
    /**
     * Return the first Mbean name gotten by the query in the current MbeanServer.
     *
     * @param p_On Query Mbean name to search
     * @return The first MBean name or null if not found
     * @throws ManagementException
     */
    public static String getFirstMBeanName(MBeanServer p_MBeanServer, ObjectName p_On)
        throws Exception {
//            synchronized (s_Synchro) {
        ArrayList al = new ArrayList();
        Iterator itNames = p_MBeanServer.queryNames(p_On, null).iterator();
        if (itNames.hasNext()) {
            return itNames.next().toString();
        }
        return null;
//            }
    }

    /**
     * Return the list of Mbean name gotten by the query in the current MbeanServer.
     *
     * @param p_On Query Mbean name to search
     * @return A list of string Mbean name
     * @throws ManagementException
     */
    public static List getListMBeanName(MBeanServer p_MBeanServer, ObjectName p_On)
        throws Exception {
//        synchronized (s_Synchro) {
        ArrayList al = new ArrayList();
        Iterator itNames = p_MBeanServer.queryNames(p_On, null).iterator();
        while (itNames.hasNext()) {
            ObjectName item = (ObjectName) itNames.next();
            al.add(item.toString());
        }
        Collections.sort(al);
        return al;
//      }
    }

    /**
     * Return the first <code>ObjectName</code> Mbean gotten by the query
     * in the current MbeanServer.
     *
     * @param p_On Query Mbean name to search
     * @return The first <code>ObjectName</code> or null if not found
     * @throws ManagementException
     */
    /*
         public static ObjectName getFirstMbean(ObjectName p_On)
        throws ManagementException {
        synchronized (s_Synchro) {
            ArrayList al = new ArrayList();
            Iterator itNames = JonasManagementRepr.queryNames(p_On).iterator();
            if (itNames.hasNext()) {
                return (ObjectName) itNames.next();
            }
            return null;
        }
         }
     */

    /**
     * Return the list of <code>ObjectName</code> Mbean gotten by the query
     * in the current MbeanServer.
     *
     * @param p_On Query Mbeans to search
     * @return The list of <code>ObjectName</code>
     * @throws ManagementException
     */
    /*
        public static List getListMbean(MBeanServer p_MBeanServer, ObjectName p_On)
            throws Exception {
            synchronized (s_Synchro) {
                ArrayList al = new ArrayList();
                Iterator itNames = p_MBeanServer.queryNames(p_On, null).iterator();
                while (itNames.hasNext()) {
                    al.add(itNames.next());
                }
                return al;
            }
        }
     */
    /**
     * Return the set of <code>ObjectName</code> for all Mbeans in the current MbeanServer.
     *
     * @param p_MBeanServer The current MBean server
     * @return The set of <code>ObjectName</code>
     *
     * @throws Exception
     */

    public static Set getMBeans(MBeanServer p_MBeanServer)
        throws Exception {
        return p_MBeanServer.queryNames(null, null);
    }

    /**
     * Return the set of <code>ObjectName</code> Mbeans gotten by the query
     * in the current MbeanServer.
     *
     * @param p_MBeanServer The current MBean server
     * @param p_On Query Mbeans to search
     * @return The set of <code>ObjectName</code>
     *
     * @throws Exception
     */

    public static Set getMBeans(MBeanServer p_MBeanServer, ObjectName p_On)
        throws Exception {
        return p_MBeanServer.queryNames(p_On, null);
    }

    /**
     * Extract a value of a Mbean name.
     *
     * @param p_Name Name of property
     * @param p_MBeanName Complete Mbean name
     * @return The value or null if not found
     */
    /*
         public static String extractValueMbeanName(String p_Name, String p_MBeanName) {
        String sValue = null;
        try {
            String sSearch = p_Name.trim() + "=";
            int iPos = p_MBeanName.indexOf(sSearch);
            if (iPos > -1) {
                sValue = p_MBeanName.substring(iPos + sSearch.length());
                iPos = sValue.indexOf(",");
                if (iPos > -1) {
                    sValue = sValue.substring(0, iPos);
                }
            }
        }
        catch (NullPointerException e) {
            // none
            sValue = null;
        }
        return sValue;
         }
     */

    /**
     * Create a list for all Mbeans in the server.
     *
     * @param p_MBeanServer The current MBean server
     * @return An array of lists
     *
     * @throws Exception
     */
    public static ArrayList getMBeanItemsList(MBeanServer p_MBeanServer)
        throws Exception {

        MBeanItem oItem;

        // Create the new list
        ArrayList al = new ArrayList();
        // Dispath each Mbean in its list
        Iterator it = getMBeans(p_MBeanServer, null).iterator();
        while (it.hasNext()) {
            oItem = MBeanItem.build((ObjectName) it.next());
            al.add(oItem);
        }
        return al;
    }

    /**
     * Create a list for each family of Mbean.
     *
     * @param p_MBeanServer The current MBean server
     * @return An array of lists
     *
     * @throws Exception
     */
    public static ArrayList[] getFamiliesMBeanItemsLists(MBeanServer p_MBeanServer)
        throws Exception {

        MBeanItem oItem;

        // Create the new lists
        ArrayList[] als = new ArrayList[3];
        for (int i = 0; i < MBeanItem.SIZE_FAMILIES; i++) {
            als[i] = new ArrayList();
        }
        // Dispath each Mbean in its list
        Iterator it = getMBeans(p_MBeanServer, null).iterator();
        while (it.hasNext()) {
            oItem = MBeanItem.build((ObjectName) it.next());
            als[oItem.getFamily()].add(oItem);
        }
        return als;
    }

    /**
     * Extract a value of a Mbean name.
     *
     * @param p_Name Name of property
     * @param p_MBeanName Complete Mbean name
     * @return The value or null if not found
     */
    public static String extractValueMbeanName(String p_Name, String p_MBeanName) {
        String sValue = null;
        try {
            String sSearch = p_Name.trim() + "=";
            int iPos = p_MBeanName.indexOf(sSearch);
            if (iPos > -1) {
                sValue = p_MBeanName.substring(iPos + sSearch.length());
                iPos = sValue.indexOf(",");
                if (iPos > -1) {
                    sValue = sValue.substring(0, iPos);
                }
            }
        }
        catch (NullPointerException e) {
            // none
            sValue = null;
        }
        return sValue;
    }
}
