/*
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 1999 Bull S.A.
 * Contact: jonas-team@objectweb.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id:  $
 * --------------------------------------------------------------------------
 */

package org.objectweb.jonas.webapp.admin.security;

import javax.management.Attribute;
import javax.management.ObjectName;

import org.objectweb.jonas.webapp.admin.mbean.MBeanTreeBuilder;
import org.objectweb.jonas.webapp.common.CommonBaseAction;
import org.objectweb.jonas.webapp.taglib.TreeControl;
import org.objectweb.jonas.webapp.taglib.TreeControlNode;

/**
 * @author Michel-Ange ANTON
 */

abstract public class BaseSecurityAction extends CommonBaseAction {

// --------------------------------------------------------- Protected Methods

    /**
     * Refresh the tree.
     *
     * @throws Exception
     */
    protected void refreshTree()
        throws Exception {
        // Refresh Security Tree
        refreshSecurityTree();
        // Refresh MBeans Tree
        refreshMBeansTree();
        // Force display to refresh
        m_WhereAreYou.setTreeToRefresh(true);
    }

    /**
     * Refresh the security branch tree.
     *
     * @throws Exception
     */
    protected void refreshSecurityTree()
        throws Exception {
        throw new Exception("Ta pas fini la methode BaseSecurityAction.refreshSecurityTree");
        /*
                 // Get the security service node name
                 String sCurrentNodeNameItem =
                m_WhereAreYou.getCurrentDomainName() + WebappContextSession.NODE_SEPARATOR +
                m_WhereAreYou.getCurrentJonasServerName() + WebappContextSession.NODE_SEPARATOR +
                "security";
                 // Get current tree
                 TreeControl oControl = m_WhereAreYou.getTreeControl();
                 // Get branch root node
                 TreeControlNode oBranchRootNode = oControl.findNode(sCurrentNodeNameItem);
                 // Enable auto-refresh mode
                 oControl.enableAutoRefresh();
                 // Remove old children
                 TreeControlNode[] aoNodes = oBranchRootNode.findChildren();
                 for (int i = 0; i < aoNodes.length; i++) {
            aoNodes[i].remove();
                 }
                 // Build node for the Service
                 JonasTreeBuilder oBuilder = new JonasTreeBuilder();
                 oBuilder.getSecurityFactories(oBranchRootNode, m_Resources);
                 // Disable auto-refresh mode
                 oControl.disableAutoRefresh();
         */
    }

    /**
     * Refresh the MBeans tree.
     *
     * @throws Exception
     */

    protected void refreshMBeansTree()
        throws Exception {
        // Get current tree
        TreeControl oControl = m_WhereAreYou.getTreeControl();
        // MBeans node present ?
        TreeControlNode oMBeansNode = oControl.findNode("mbeans");
        if (oMBeansNode != null) {
            // Enable auto-refresh mode
            oControl.enableAutoRefresh();
            // Remove node
            oMBeansNode.remove();
            // Build node and his children
            MBeanTreeBuilder oBuilder = new MBeanTreeBuilder();
            oBuilder.getMBeans(oControl.getRoot(), getMBeanServer(), getMessages());
            // Disable auto-refresh mode
            oControl.disableAutoRefresh();
        }
    }

    /**
     * MBean <code>ObjectName</code> accessor.
     *
     * @param p_ObjectName Instance of ObjectName to access to MBean
     * @param ps_AttrName Attribute name of MBean
     * @return The value of attribute
     */
    protected String getStringAttribute(ObjectName p_ObjectName, String ps_AttrName)
        throws Exception {
        String s = (String) getMBeanServer().getAttribute(p_ObjectName, ps_AttrName);
        return s;
    }

    /**
     * MBean <code>ObjectName</code> accessor.
     *
     * @param p_ObjectName Instance of ObjectName to access to MBean
     * @param ps_AttrName Attribute name of MBean
     * @param p_Value Value to write
     */
    protected void setStringAttribute(ObjectName p_ObjectName, String ps_AttrName, String p_Value)
        throws Exception {
        Attribute oAttr = new Attribute(ps_AttrName, p_Value);
        getMBeanServer().setAttribute(p_ObjectName, oAttr);
    }
}
