/**
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 1999 Bull S.A.
 * Contact: jonas-team@objectweb.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id:  $
 * --------------------------------------------------------------------------
 */

package org.objectweb.jonas.webapp.admin.security;

import javax.management.MBeanServer;
import javax.management.MalformedObjectNameException;
import javax.management.ObjectName;

import org.apache.commons.modeler.Registry;

/*
 Attention : en fonction de l'API JMX utilis, il peut y avoir des soucis.
 La mthode ObjectName.getInstance(...) n'existe qu'en v1.2 de la spec JMX.
 Hors, JOnAS 3.x.x utilise une version 1.1 et JOnAS 4.1.x utilise une version 1.2.
 Le code de cette prsente classe JonasObjectName est extrait de JOnAS v4.1.0.
 Il a donc fallut la modifier pour mes test en 3.3.6 !
 */

/**
     * A set of static classes used to build the names of proprietary MBeans used in JOnAS and in Joram.
 * @author Bruno Michel
 * @author Guillaume Riviere
 * @author Florent Benoit
 * @author Ludovic Bert
 * @author Miroslav Halas
 * @author Adriana Danes
 */
public class JonasObjectName {

    /**
     * domain name
     */
    private static String domain = null;

    /**
     * @return Returns the domain name of the JOnAS server.
     */
    private static String getDomain() {
        if (domain == null) {
            MBeanServer oMBeanServer = new Registry().getMBeanServer();
            domain = oMBeanServer.getDefaultDomain();
        }
        return domain;
    }

    public static ObjectName securityService() {
        try {
            return new ObjectName(getDomain() + ":type=service,name=security");
            //return ObjectName.getInstance(getDomain() + ":type=service,name=security");
        }
        catch (MalformedObjectNameException e) {
            // this should never occur
            return null;
        }
    }

    public static ObjectName jmxService() {
        try {
            return new ObjectName("JMImplementation:type=MBeanServerDelegate");
            //return ObjectName.getInstance("JMImplementation:type=MBeanServerDelegate");
        }
        catch (MalformedObjectNameException e) {
            // this should never occur
            return null;
        }
    }

    /**
     * Return an objectName for a Security memory factory.
     * @param name the name of the security memory factory
     * @return an objectName for the security memory factory.
     * @throws MalformedObjectNameException if the objectname can't be build
     */
    public static ObjectName securityMemoryFactory(String name)
        throws MalformedObjectNameException {
        return new ObjectName(getDomain() + ":type=securityfactory,subtype=memory,name=" + name);
        //return ObjectName.getInstance(getDomain() + ":type=securityfactory,subtype=memory,name=" + name);
    }

    /**
     * Return an objectName for a Security datasource factory.
     * @param name the name of the security datasource factory
     * @return an objectName for the security datasource factory.
     * @throws MalformedObjectNameException if the objectname can't be build
     */
    public static ObjectName securityDatasourceFactory(String name)
        throws MalformedObjectNameException {
        return new ObjectName(getDomain() + ":type=securityfactory,subtype=datasource,name=" + name);
        //return ObjectName.getInstance(getDomain() + ":type=securityfactory,subtype=datasource,name=" + name);
    }

    /**
     * Return an objectName for a Security ldap factory.
     * @param name the name of the security ldap factory
     * @return an objectName for the security ldap factory.
     * @throws MalformedObjectNameException if the objectname can't be build
     */
    public static ObjectName securityLdapFactory(String name)
        throws MalformedObjectNameException {
        return new ObjectName(getDomain() + ":type=securityfactory,subtype=ldap,name=" + name);
        //return ObjectName.getInstance(getDomain() + ":type=securityfactory,subtype=ldap,name=" + name);
    }

    /**
     * Return an objectName for a user
     * @param resource the name of the resource on which the user depends
     * @param username the name of the user
     * @return an objectName for the user MBean
     * @throws MalformedObjectNameException if the objectname can't be build
     */
    public static ObjectName user(String resource, String username)
        throws MalformedObjectNameException {
        return new ObjectName(getDomain() + ":type=security,subtype=users,resource=" + resource
            + ",name=" + username);
//        return ObjectName.getInstance(getDomain() + ":type=security,subtype=users,resource=" + resource
//            + ",name=" + username);
    }

    /**
     * Return an objectName for all the users
     * @return an objectName for all the users
     * @throws MalformedObjectNameException if the objectname can't be build
     */
    public static ObjectName allUsers() {
        try {
            return new ObjectName(getDomain() + ":type=security,subtype=users,*");
            //return ObjectName.getInstance(getDomain() + ":type=security,subtype=users,*");
        }
        catch (MalformedObjectNameException e) {
            // this should never occur
            return null;
        }
    }

    /**
     * Return an objectName for a group
     * @param resource the name of the resource on which the group depends
     * @param groupname the name of the group
     * @return an objectName for the group MBean
     * @throws MalformedObjectNameException if the objectname can't be build
     */
    public static ObjectName group(String resource, String groupname)
        throws MalformedObjectNameException {
        return new ObjectName(getDomain() + ":type=security,subtype=groups,resource=" + resource
            + ",name=" + groupname);
//        return ObjectName.getInstance(getDomain() + ":type=security,subtype=groups,resource=" + resource
//            + ",name=" + groupname);
    }

    /**
     * Return an objectName for all the groups
     * @return an objectName for all the groups
     * @throws MalformedObjectNameException if the objectname can't be build
     */
    public static ObjectName allGroups() {
        try {
            return new ObjectName(getDomain() + ":type=security,subtype=groups,*");
            //return ObjectName.getInstance(getDomain() + ":type=security,subtype=groups,*");
        }
        catch (MalformedObjectNameException e) {
            // this should never occur
            return null;
        }
    }

    /**
     * Return an objectName for a role
     * @param resource the name of the resource on which the role depends
     * @param rolename the name of the role
     * @return an objectName for the user MBean
     * @throws MalformedObjectNameException if the objectname can't be build
     */
    public static ObjectName role(String resource, String rolename)
        throws MalformedObjectNameException {
        return new ObjectName(getDomain() + ":type=security,subtype=roles,resource=" + resource
            + ",name=" + rolename);
//        return ObjectName.getInstance(getDomain() + ":type=security,subtype=roles,resource=" + resource
//            + ",name=" + rolename);
    }

    /**
     * Return an objectName for all the roles
     * @return an objectName for all the roles
     * @throws MalformedObjectNameException if the objectname can't be build
     */
    public static ObjectName allRoles() {
        try {
            return new ObjectName(getDomain() + ":type=security,subtype=roles,*");
            //return ObjectName.getInstance(getDomain() + ":type=security,subtype=roles,*");
        }
        catch (MalformedObjectNameException e) {
            // this should never occur
            return null;
        }
    }

    /**
     * Return an objectName for the Security factory name.
     *
     * @param pName The name of the wanted security factory
     * @return an objectName for the security factory name.
     * @throws MalformedObjectNameException if the objectname can't be build
     */
    public static ObjectName securityFactories(String pName)
        throws MalformedObjectNameException {
        return new ObjectName(getDomain() + ":type=securityfactory,name=" + pName);
        //return ObjectName.getInstance(getDomain() + ":type=securityfactory,name=" + pName);
    }

    /**
     * Return an objectName for the Security factory name.
     *
     * @return an objectName for the security factory name.
     * @throws MalformedObjectNameException if the objectname can't be build
     */
    public static ObjectName allSecurityFactories()
        throws MalformedObjectNameException {
        return new ObjectName(getDomain() + ":type=securityfactory,*");
        //return ObjectName.getInstance(getDomain() + ":type=securityfactory,*");
    }

    /**
     * Return an objectName for all the Security memory factories.
     *
     * @return an objectName for all the security memory factories
     * @throws MalformedObjectNameException if the objectname can't be build
     */
    public static ObjectName allSecurityMemoryFactories()
        throws MalformedObjectNameException {
        return new ObjectName(getDomain() + ":type=securityfactory,subtype=memory,*");
        //return ObjectName.getInstance(getDomain() + ":type=securityfactory,subtype=memory,*");
    }

    /**
     * Return an objectName for all the Security datasource factories.
     *
     * @return an objectName for all the security datasource factories
     * @throws MalformedObjectNameException if the objectname can't be build
     */
    public static ObjectName allSecurityDatasourceFactories()
        throws MalformedObjectNameException {
        return new ObjectName(getDomain() + ":type=securityfactory,subtype=datasource,*");
        //return ObjectName.getInstance(getDomain() + ":type=securityfactory,subtype=datasource,*");
    }

    /**
     * Return an objectName for all the Security ldap factories.
     *
     * @return an objectName for all the security ldap factories
     * @throws MalformedObjectNameException if the objectname can't be build
     */
    public static ObjectName allSecurityLdapFactories()
        throws MalformedObjectNameException {
        return new ObjectName(getDomain() + ":type=securityfactory,subtype=ldap,*");
        //return ObjectName.getInstance(getDomain() + ":type=securityfactory,subtype=ldap,*");
    }

    /**
     * Return an objectName for all users in a resource.
     *
     * @param pResource the name of the resource on which the user depends
     * @return an objectName for the user MBean
     * @throws MalformedObjectNameException if the objectname can't be build
     */
    public static ObjectName allUsers(String pResource)
        throws MalformedObjectNameException {
        return new ObjectName(getDomain() + ":type=security,subtype=users,resource=" + pResource
            + ",*");
//        return ObjectName.getInstance(getDomain() + ":type=security,subtype=users,resource=" + pResource
//            + ",*");
    }

    /**
     * Return an objectName for all roles in a resource.
     *
     * @param pResource the name of the resource on which the role depends
     * @return an objectName for the user MBean
     * @throws MalformedObjectNameException if the objectname can't be build
     */
    public static ObjectName allRoles(String pResource)
        throws MalformedObjectNameException {
        return new ObjectName(getDomain() + ":type=security,subtype=roles,resource=" + pResource
            + ",*");
//        return ObjectName.getInstance(getDomain() + ":type=security,subtype=roles,resource=" + pResource
//            + ",*");
    }

    /**
     * Return an objectName for all groups in a resource.
     *
     * @param pResource the name of the resource on which the group depends
     * @return an objectName for the user MBean
     * @throws MalformedObjectNameException if the objectname can't be build
     */
    public static ObjectName allGroups(String pResource)
        throws MalformedObjectNameException {
        return new ObjectName(getDomain() + ":type=security,subtype=groups,resource=" + pResource
            + ",*");
//        return ObjectName.getInstance(getDomain() + ":type=security,subtype=groups,resource=" + pResource
//            + ",*");
    }

    /**
     * A different implementation should allow returning a logical name.
     * This String is used within the Exception messages when throwing a ManagementException.
         * This is done currently in the invoke method of ManagementReprImpl/Mx4jManagementReprImpl classes.
     * @return String representation of the ObjectName
     */
    public String toString() {
        return super.toString();
    }

}
