/*
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 1999 Bull S.A.
 * Contact: jonas-team@objectweb.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: $
 * --------------------------------------------------------------------------
 */

package org.objectweb.jonas.webapp.admin.security;

import java.util.ArrayList;
import java.util.Collections;
import java.util.Iterator;

import javax.management.MBeanServer;
import javax.servlet.http.HttpServletRequest;

import org.apache.struts.Globals;
import org.apache.struts.action.ActionServlet;
import org.apache.struts.util.MessageResources;
import org.objectweb.jonas.webapp.admin.mbean.MBeanUtils;
import org.objectweb.jonas.webapp.common.WebappContextSession;
import org.objectweb.jonas.webapp.taglib.TreeBuilder;
import org.objectweb.jonas.webapp.taglib.TreeControl;
import org.objectweb.jonas.webapp.taglib.TreeControlNode;

/**
 * Create all the node for the example's tree.
 *
 * @author Michel-Ange ANTON
 */
public class SecurityTreeBuilder implements TreeBuilder {

// ---------------------------------------------------- TreeBuilder Methods

    /**
     * Add the required nodes to the specified <code>treeControl</code>
     * instance.
     *
     * @param p_TreeControl The <code>TreeControl</code> to which we should add our nodes
     * @param p_Servlet The controller servlet for the admin application
     * @param p_Request The servlet request we are processing
     */
    public void buildTree(TreeControl p_TreeControl, ActionServlet p_Servlet
        , HttpServletRequest p_Request) {
        try {
            TreeControlNode oRootNode = p_TreeControl.getRoot();
            MessageResources oResources = ((MessageResources) p_Request.getAttribute(Globals.
                MESSAGES_KEY));
            WebappContextSession oWhere = (WebappContextSession) p_Request.getSession().
                getAttribute(WebappContextSession.SESSION_NAME);
            MBeanServer oMBeanServer = oWhere.getMBeanServer();
            if (null != oMBeanServer) {
                addSecurityFactories(oRootNode, oMBeanServer, oResources);
            }
        }
        catch (Throwable t) {
            t.printStackTrace(System.out);
        }
    }

// ------------------------------------------------------ Protected Methods

    /**
     * Append nodes for Security factories for the JOnAS server.
     *
     * @param p_RootNode Root node for the tree control
     * @param p_MBeanServer The current MBean server
     * @param p_Resources The MessageResources for our localized messages
     * @exception Exception if an exception occurs building the tree
     */
    public void addSecurityFactories(TreeControlNode p_RootNode, MBeanServer p_MBeanServer
        , MessageResources p_Resources)
        throws Exception {
        ArrayList al = null;
        String sName = null;
        TreeControlNode nodeSecurityFactories = null;
        TreeControlNode nodeSecurityFactory = null;

        // Add security root node
        TreeControlNode oRootSecurityNode = new TreeControlNode("security", "icon/security.gif"
            , p_Resources.getMessage("treenode.security"), "ListRealms.do", "content", false);
        p_RootNode.addChild(oRootSecurityNode);

        // Security Memory factories
        al = getSecurityMemoryFactories(p_MBeanServer);
        if (al.size() > 0) {
            // Security factories node
            nodeSecurityFactories = new TreeControlNode(oRootSecurityNode.getName()
                + WebappContextSession.NODE_SEPARATOR + "factory.memory"
                , "icon/security_memory.gif", p_Resources.getMessage("treenode.security.memory")
                , "ListRealms.do?realm=memory", "content", false);
            oRootSecurityNode.addChild(nodeSecurityFactories);
            // List Security factories
            for (int i = 0; i < al.size(); i++) {
                sName = al.get(i).toString();
                // Security factory node
                nodeSecurityFactory = new TreeControlNode(nodeSecurityFactories.getName()
                    + WebappContextSession.NODE_SEPARATOR + sName, "icon/security_memory.gif"
                    , sName, "EditMemoryRealm.do?resource=" + sName, "content", false);
                nodeSecurityFactories.addChild(nodeSecurityFactory);
                // Add detail to memory security node
                //addSecurityMemoryFactory(nodeSecurityFactory, p_Resources, sName);
            }
        }
        // Security Datasource factories
        al = getSecurityDatasourceFactories(p_MBeanServer);
        if (al.size() > 0) {
            // Security factories node
            nodeSecurityFactories = new TreeControlNode(oRootSecurityNode.getName()
                + WebappContextSession.NODE_SEPARATOR + "factory.datasource"
                , "icon/security_database.gif"
                , p_Resources.getMessage("treenode.security.datasource")
                , "ListRealms.do?realm=datasource", "content", false);
            oRootSecurityNode.addChild(nodeSecurityFactories);
            // List Security factories
            for (int i = 0; i < al.size(); i++) {
                sName = al.get(i).toString();
                // Security factory node
                nodeSecurityFactory = new TreeControlNode(nodeSecurityFactories.getName()
                    + WebappContextSession.NODE_SEPARATOR + sName, "icon/security_database.gif"
                    , sName, "EditDatasourceRealm.do?resource=" + sName, "content", false);
                nodeSecurityFactories.addChild(nodeSecurityFactory);
            }
        }
        // Security Ldap factories
        al = getSecurityLdapFactories(p_MBeanServer);
        if (al.size() > 0) {
            // Security factories node
            nodeSecurityFactories = new TreeControlNode(oRootSecurityNode.getName()
                + WebappContextSession.NODE_SEPARATOR + "factory.ldap", "icon/security_ldap.gif"
                , p_Resources.getMessage("treenode.security.ldap"), "ListRealms.do?realm=ldap"
                , "content", false);
            oRootSecurityNode.addChild(nodeSecurityFactories);
            // List Security factories
            for (int i = 0; i < al.size(); i++) {
                sName = al.get(i).toString();
                // Security factory node
                nodeSecurityFactory = new TreeControlNode(nodeSecurityFactories.getName()
                    + WebappContextSession.NODE_SEPARATOR + sName, "icon/security_ldap.gif", sName
                    , "EditLdapRealm.do?resource=" + sName, "content", false);
                nodeSecurityFactories.addChild(nodeSecurityFactory);
            }
        }
    }

    /**
     * Return the list of Security Memory Factories in the current server.
     *
     * @param p_MBeanServer The current MBean server
     * @return The list of Security Memory Factories
     * @throws Exception
     */
    public static ArrayList getSecurityMemoryFactories(MBeanServer p_MBeanServer)
        throws Exception {
        String sName;
        ArrayList al = new ArrayList();
        Iterator itNames = MBeanUtils.getListMBeanName(p_MBeanServer
            , JonasObjectName.allSecurityMemoryFactories()).iterator();
        while (itNames.hasNext()) {
            sName = MBeanUtils.extractValueMbeanName(",name", itNames.next().toString());
            if (sName != null) {
                al.add(sName);
            }
        }
        Collections.sort(al);
        return al;
    }

    /**
     * Return the list of Security Datasource Factories in the current server.
     *
     * @param p_MBeanServer The current MBean server
     * @return The list of Security Datasource Factories
     * @throws Exception
     */
    public static ArrayList getSecurityDatasourceFactories(MBeanServer p_MBeanServer)
        throws Exception {
        String sName;
        ArrayList al = new ArrayList();
        Iterator itNames = MBeanUtils.getListMBeanName(p_MBeanServer
            , JonasObjectName.allSecurityDatasourceFactories()).iterator();
        while (itNames.hasNext()) {
            sName = MBeanUtils.extractValueMbeanName(",name", itNames.next().toString());
            if (sName != null) {
                al.add(sName);
            }
        }
        Collections.sort(al);
        return al;
    }

    /**
     * Return the list of Security Ldap Factories in the current server.
     *
     * @param p_MBeanServer The current MBean server
     * @return The list of Security Ldap Factories
     * @throws Exception
     */
    public static ArrayList getSecurityLdapFactories(MBeanServer p_MBeanServer)
        throws Exception {
        String sName;
        ArrayList al = new ArrayList();
        Iterator itNames = MBeanUtils.getListMBeanName(p_MBeanServer
            , JonasObjectName.allSecurityLdapFactories()).iterator();
        while (itNames.hasNext()) {
            sName = MBeanUtils.extractValueMbeanName(",name", itNames.next().toString());
            if (sName != null) {
                al.add(sName);
            }
        }
        Collections.sort(al);
        return al;
    }

    /**
     * Append nodes for Memory Security Factory for the specified server.
     *
     * @param p_SecurityMemoryNode Server node for the tree control
     * @param p_Resources The MessageResources for our localized messages
     * @param p_ResourceFactory The name of resource
     * @exception Exception if an exception occurs building the tree
     */
    protected void addSecurityMemoryFactory(TreeControlNode p_SecurityMemoryNode
        , MessageResources p_Resources, String p_ResourceFactory)
        throws Exception {
        TreeControlNode node = null;
        // Users node
        node = new TreeControlNode(p_SecurityMemoryNode.getName()
            + WebappContextSession.NODE_SEPARATOR + "users", "icon/security_memory.gif"
            , p_Resources.getMessage("treenode.jonas.server.security.factory.memory.users")
            , "EditMemoryRealmUsers.do?resource=" + p_ResourceFactory, "content", false);
        p_SecurityMemoryNode.addChild(node);
        // Groups node
        node = new TreeControlNode(p_SecurityMemoryNode.getName()
            + WebappContextSession.NODE_SEPARATOR + "groups", "icon/security_memory.gif"
            , p_Resources.getMessage("treenode.jonas.server.security.factory.memory.groups")
            , "EditMemoryRealmGroups.do?resource=" + p_ResourceFactory, "content", false);
        p_SecurityMemoryNode.addChild(node);
        // Roles node
        node = new TreeControlNode(p_SecurityMemoryNode.getName()
            + WebappContextSession.NODE_SEPARATOR + "roles", "icon/security_memory.gif"
            , p_Resources.getMessage("treenode.jonas.server.security.factory.memory.roles")
            , "EditMemoryRealmRoles.do?resource=" + p_ResourceFactory, "content", false);
        p_SecurityMemoryNode.addChild(node);
    }

}
