/*
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 1999 Bull S.A.
 * Contact: jonas-team@objectweb.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: $
 * --------------------------------------------------------------------------
 */

package org.objectweb.jonas.webapp.common;

import java.util.Properties;
import java.util.StringTokenizer;

import javax.management.MBeanServer;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;

import org.apache.struts.action.Action;
import org.apache.struts.action.ActionError;
import org.apache.struts.action.ActionErrors;
import org.apache.struts.action.ActionForm;
import org.apache.struts.action.ActionForward;
import org.apache.struts.action.ActionMapping;
import org.apache.struts.util.MessageResources;

/**
 * @author Michel-Ange ANTON
 */

public abstract class CommonBaseAction extends Action {

// ----------------------------------------------------- Constants

// --------------------------------------------------------- Instance Variables

    /**
     * The MessageResources we will be retrieving messages from.
     */
    protected MessageResources m_Resources = null;
    protected HttpSession m_Session = null;
    protected ActionErrors m_Errors = null;
    protected WebappContextSession m_WhereAreYou = null;
    protected MBeanServer m_MBeanServer = null;

// --------------------------------------------------------- Public Methods

    /**
     * Call by the method <code>execute</code>.
     *
     * @param p_Mapping The ActionMapping used to select this instance
     * @param p_Form The optional ActionForm bean for this request (if any)
     * @param p_Request The HTTP request we are processing
     * @param p_Response The HTTP response we are creating
     *
     * @return The forward where redirect
     *
     * @throws Exception if an error occurs
     */
    public abstract ActionForward executeAction(ActionMapping p_Mapping, ActionForm p_Form
        , HttpServletRequest p_Request, HttpServletResponse p_Response)
        throws Exception;

    /**
     * Process the specified HTTP request, and create the corresponding HTTP
     * response (or forward to another web component that will create it).
     * Return an <code>ActionForward</code> instance describing where and how
     * control should be forwarded, or <code>null</code> if the response has
     * already been completed.
     *
     * @param p_Mapping The ActionMapping used to select this instance
     * @param p_Form The optional ActionForm bean for this request (if any)
     * @param p_Request The HTTP request we are processing
     * @param p_Response The HTTP response we are creating
     *
     * @return The forward where redirect
     *
     * @throws Exception if an error occurs
     *
     * @see org.apache.struts.action.Action
     */
    public ActionForward execute(ActionMapping p_Mapping, ActionForm p_Form
        , HttpServletRequest p_Request, HttpServletResponse p_Response)
        throws Exception {
        ActionForward oActionForward = null;

        // Instances variables
        initialize(p_Request);

        // Verify that a instance of WhereAreYou object exists
        if (m_WhereAreYou == null) {
            oActionForward = (p_Mapping.findForward("Main Index"));
        }
        else {
            oActionForward = executeAction(p_Mapping, p_Form, p_Request, p_Response);
        }
        return oActionForward;
    }

    /**
     * Initialize all instances variables.
     * @param p_Request The HTTP request we are processing
     */
    protected void initialize(HttpServletRequest p_Request) {
        initializeRessources(p_Request);
        initializeServer(p_Request);
    }

    /**
     * Initialize the instance variables Session, WhereAreYou, MessageResources, Errors.
     * @param p_Request The HTTP request we are processing
     */
    protected void initializeRessources(HttpServletRequest p_Request) {
        // Acquire the resources that we need
        m_Session = p_Request.getSession();
        m_WhereAreYou = (WebappContextSession) m_Session.getAttribute(WebappContextSession.
            SESSION_NAME);
        if (m_Resources == null) {
            m_Resources = getResources(p_Request);
        }
        m_Errors = new ActionErrors();
    }

    /**
     * Initialize the instance variables Resources : MBeanServer.
     * @param p_Request The HTTP request we are processing
     */
    protected void initializeServer(HttpServletRequest p_Request) {
        m_MBeanServer = m_WhereAreYou.getMBeanServer();
    }

    /**
     * Return the current session HTTP.
     * @return The current session HTTP
     */
    protected HttpSession getSession() {
        return m_Session;
    }

    /**
     * Return the current WebappContextSession (WhereAreYou).
     * @return The current WebappContextSession
     */
    protected WebappContextSession getWhereAreYou() {
        return m_WhereAreYou;
    }

    /**
     * Return the current MessageResources.
     * @return The current MessageResources
     */
    protected MessageResources getMessages() {
        return m_Resources;
    }

    /**
     * Return the current ActionErrors.
     * @return The current ActionErrors
     */
    protected ActionErrors getErrors() {
        return m_Errors;
    }

    /**
     * Return the current MBeanServer.
     * @return The current MBeanServer or null
     */
    protected MBeanServer getMBeanServer() {
        return m_MBeanServer;
    }

    /**
     * Return the name of the branch in the tree for the selected node.
     *
     * @param p_Width Depth of the branch
     * @return The branch name
     */
    protected String getTreeBranchName(int p_Width) {
        StringBuffer sb = new StringBuffer();
        try {
            StringTokenizer st = new StringTokenizer(m_WhereAreYou.getSelectedNameNode()
                , WebappContextSession.NODE_SEPARATOR);
            for (int i = 0; (st.hasMoreTokens() && (i < p_Width)); i++) {
                if (i > 0) {
                    sb.append(WebappContextSession.NODE_SEPARATOR);
                }
                sb.append(st.nextToken());
            }
        }
        catch (NullPointerException e) {
            // none action
        }
        return sb.toString();
    }

    /**
     * Add a global error in <code>m_Errors</code> and log it.
     *
     * @param p_Throwable Error to add
     */
    protected void addGlobalError(Throwable p_Throwable) {
        String sMessResource;
        String sMessageError = p_Throwable.getMessage();

        // displaying
        m_Errors.add("errors.detail", new ActionError("errors.detail"
            , buildMessageError(p_Throwable)));
        // logging
        getServlet().log(m_Resources.getMessage("errors.global"), p_Throwable);
    }

    /**
     * Build the error message of an exception with this cause.
     * @param p_Throwable The error to manage
     * @return The HTML string to display
     */
    protected String buildMessageError(Throwable p_Throwable) {
        StringBuffer sb = new StringBuffer();
        // L'exception
        sb.append(htmlMessageError(p_Throwable, false));
        // La cause
        if (p_Throwable.getCause() != null) {
            sb.append(htmlMessageError(p_Throwable.getCause(), true));
        }
        return sb.toString();
    }

    /**
     * Build the HTML error message.
     * @param p_Throwable The error to manage
     * @param p_Cause True if the exception should be displayed like a cause
     * @return The HTML string to display
     */
    protected String htmlMessageError(Throwable p_Throwable, boolean p_Cause) {
        StringBuffer sb = new StringBuffer();
        if (true == p_Cause) {
            sb.append("<ul><li><strong>Cause : </strong>");
        }
        // Le nom de l'exception
        sb.append("<div><strong>Exception : </strong>");
        sb.append(p_Throwable.getClass().getName());
        sb.append("</div>");
        // Le message de l'exception
        if (p_Throwable.getMessage() != null) {
            sb.append("<div><strong>Message : </strong>");
            sb.append(p_Throwable.getMessage());
            sb.append("</div>");
        }
        StackTraceElement[] aStacks = p_Throwable.getStackTrace();
        if (0 < aStacks.length) {
            sb.append("<strong>Stack :</strong><ul><pre>");
            for (int i = 0; i < aStacks.length; i++) {
                sb.append(aStacks[i].toString()).append("\n");
                if (10 < i) {
                    sb.append("...\n");
                    break;
                }
            }
            sb.append("</pre></ul>");
        }
        if (true == p_Cause) {
            sb.append("</li></ul>");
        }
        return sb.toString();
    }

    /**
     * Properties accessor.
     *
     * @param p_Props Instance of Properties
     * @param ps_AttrName Attribute name of Properties
     * @return The value of attribute
     */
    protected String getStringAttribute(Properties p_Props, String ps_AttrName) {
        return p_Props.getProperty(ps_AttrName);
    }

    /**
     * Properties accessor.
     *
     * @param p_Props Instance of Properties
     * @param ps_AttrName Attribute name of Properties
     * @param p_Default The returned value if the attribute don't exists in the properties
     * @return The value of attribute
     */
    protected String getStringAttribute(Properties p_Props, String ps_AttrName, String p_Default) {
        return p_Props.getProperty(ps_AttrName, p_Default);
    }

    /**
     * Properties accessor.
     *
     * @param p_Props Instance of Properties
     * @param ps_AttrName Attribute name of Properties
     * @param p_Value Value to write
     */
    protected void setStringAttribute(Properties p_Props, String ps_AttrName, String p_Value) {
        if (p_Value != null) {
            p_Props.setProperty(ps_AttrName, p_Value);
        }
    }

    /**
     * Properties accessor.
     *
     * @param p_Props Instance of Properties
     * @param ps_AttrName Attribute name of Properties
     * @param p_Value Value to write
     * @param p_Default The forced value if the value is null or empty
     */
    protected void setStringAttribute(Properties p_Props, String ps_AttrName, String p_Value
        , String p_Default) {
        if ((p_Value != null) && (p_Value.length() == 0)) {
            p_Props.setProperty(ps_AttrName, p_Default);
        }
        else {
            p_Props.setProperty(ps_AttrName, p_Value);
        }
    }

    /**
     * Properties accessor.
     *
     * @param p_Props Instance of Properties
     * @param ps_AttrName Attribute name of Properties
     * @return The value of attribute (return 0 if attribute don't exists)
     */
    protected int getIntegerAttribute(Properties p_Props, String ps_AttrName) {
        return getIntegerAttribute(p_Props, ps_AttrName, 0);
    }

    /**
     * Properties accessor.
     *
     * @param p_Props Instance of Properties
     * @param ps_AttrName Attribute name of Properties
     * @param p_Default The returned value if the attribute don't exists in the properties
     * @return The value of attribute
     */
    protected int getIntegerAttribute(Properties p_Props, String ps_AttrName, int p_Default) {
        try {
            String s = getStringAttribute(p_Props, ps_AttrName);
            if (s != null) {
                return Integer.parseInt(s);
            }
        }
        catch (Exception e) {
            // None
        }
        return p_Default;
    }

    /**
     * Properties accessor.
     *
     * @param p_Props Instance of Properties
     * @param ps_AttrName Attribute name of Properties
     * @param p_Value Value to write
     */
    protected void setIntegerAttribute(Properties p_Props, String ps_AttrName, int p_Value) {
        p_Props.setProperty(ps_AttrName, String.valueOf(p_Value));
    }

    /**
     * Properties accessor.
     *
     * @param p_Props Instance of Properties
     * @param ps_AttrName Attribute name of Properties
     * @return The value of attribute (return 0 if attribute don't exists)
     */
    protected long getLongAttribute(Properties p_Props, String ps_AttrName) {
        return getLongAttribute(p_Props, ps_AttrName, 0L);
    }

    /**
     * Properties accessor.
     *
     * @param p_Props Instance of Properties
     * @param ps_AttrName Attribute name of Properties
     * @param p_Default The returned value if the attribute don't exists in the properties
     * @return The value of attribute
     */
    protected long getLongAttribute(Properties p_Props, String ps_AttrName, long p_Default) {
        try {
            String s = getStringAttribute(p_Props, ps_AttrName);
            if (s != null) {
                return Long.parseLong(s);
            }
        }
        catch (Exception e) {
            // None
        }
        return p_Default;
    }

    /**
     * Properties accessor.
     *
     * @param p_Props Instance of Properties
     * @param ps_AttrName Attribute name of Properties
     * @param p_Value Value to write
     */
    protected void setLongAttribute(Properties p_Props, String ps_AttrName, long p_Value) {
        p_Props.setProperty(ps_AttrName, String.valueOf(p_Value));
    }

    /**
     * Properties accessor.
     *
     * @param p_Props Instance of Properties
     * @param ps_AttrName Attribute name of Properties
     * @return The value of attribute (return false if attribute don't exists)
     */
    protected boolean getBooleanAttribute(Properties p_Props, String ps_AttrName) {
        return getBooleanAttribute(p_Props, ps_AttrName, false);
    }

    /**
     * Properties accessor.
     *
     * @param p_Props Instance of Properties
     * @param ps_AttrName Attribute name of Properties
     * @param p_Default The returned value if the attribute don't exists in the properties
     * @return The value of attribute
     */
    protected boolean getBooleanAttribute(Properties p_Props, String ps_AttrName, boolean p_Default) {
        try {
            String s = getStringAttribute(p_Props, ps_AttrName);
            if (s != null) {
                return Boolean.getBoolean(s);
            }
        }
        catch (Exception e) {
            // None
        }
        return p_Default;
    }

    /**
     * Properties accessor.
     *
     * @param p_Props Instance of Properties
     * @param ps_AttrName Attribute name of Properties
     * @param p_Value Value to write
     */
    protected void setBooleanAttribute(Properties p_Props, String ps_AttrName, boolean p_Value) {
        p_Props.setProperty(ps_AttrName, String.valueOf(p_Value));
    }

    protected Properties getPropsFromString(String ps_Props) {
        // Clean the string
        String sProps = ps_Props.trim();
        sProps = removeChar(sProps, '\r');
        sProps = removeChar(sProps, '\n');
        Properties sp = new Properties();
        StringTokenizer st = new StringTokenizer(sProps, ",");
        while (st.hasMoreTokens()) {
            String token = st.nextToken();
            int pos = token.indexOf("=");
            String propName = token.substring(0, pos);
            String propValue = token.substring(pos + 1);
            sp.setProperty(propName, propValue);
        }
        return sp;
    }

    /**
     * Remove a specific character in a String
     * Do not use replaceAll as it is a JDK 1.4 method
     * @param string the given string
     * @param c character to remove in the String
     * @return a string
     */
    protected static String removeChar(String string, char c) {
        StringBuffer sb = new StringBuffer();
        sb.setLength(string.length());
        int i = 0;
        for (int j = 0; j < string.length(); j++) {
            char cur = string.charAt(j);
            if (cur != c) {
                sb.setCharAt(i++, cur);
            }
        }
        return sb.toString();
    }

}
